/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef _CTMPL_BINTREE_C_
#define _CTMPL_BINTREE_C_
#define bt_root(x)	((x)->root)
#define bt_count(x)	((x)->count)
#define bt_isempty(x)	(bt_count(x) == 0)
#define btn_key(t)	((t)->data.first)
#define btn_value(t)	((t)->data.second)
#define btn_left(t)	((t)->left)
#define btn_right(t)	((t)->right)
#define btn_parent(t)	((t)->parent)
#endif	/* _CTMPL_BINTREE_C_ */

#include "ctmpl/memory.h"
#include "ctmpl/memory.c"

static CTMPL_INLINE void PKG_NAME_(bt_set_ctl)(PKG_NAME_(bt));
static void PKG_NAME_(bt_copy_recursive)(PKG_NAME_(btnode) k, 
					 PKG_NAME_(bt) t,
					 PKG_NAME_(btnode_duplicate_func_t) f);
static CTMPL_INLINE void 
PKG_NAME_(btnode_delete)(PKG_NAME_(bt) bt,
			 PKG_NAME_(btnode) btn,
			 PKG_NAME_(btnode_access_func_t) df);
static PKG_NAME_(btnode) PKG_NAME_(btnode_find_min)(PKG_NAME_(btnode) k);
static PKG_NAME_(btnode) PKG_NAME_(btnode_find_max)(PKG_NAME_(btnode) k);
static PKG_NAME_(btnode) PKG_NAME_(btnode_find)(PKG_NAME_(bt) bt,
					     PKG_NAME_(btnode) k, 
					     PKG_NAME_(btkey_t) key);
static PKG_NAME_(btnode) PKG_NAME_(btnode_insert)(PKG_NAME_(bt) bt,
					       PKG_NAME_(btnode) btn1,
					       PKG_NAME_(btnode) btn2);
static void PKG_NAME_(btnode_remove)(PKG_NAME_(bt) bt,
				     PKG_NAME_(btnode) k);

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(bt) 
PKG_NAME_(bt_new)(PKG_NAME_(bt_cmpkey_func_t) ck)
{
	return PKG_NAME_(bt_new_by)(ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(bt) 
PKG_NAME_(bt_new_by)(PKG_NAME_(bt_cmpkey_func_t) cmpkey_func,
		     m_alloc_func_t af,	
		     m_free_func_t ff)
{
	PKG_NAME_(bt) ret;
	ctmpl_assert(cmpkey_func != NULL);

	/* default alloc/free function */
	if(!af)	 af = PKG_NAME_(m_alloc);
	if(!ff)	 ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "bt", 
			   sizeof(struct PKG_NAME_(bt)));
	ret->root = NULL;
	ret->cmpkey = cmpkey_func;
	ret->alloc = af;
	ret->free = ff;
	PKG_NAME_(bt_set_ctl)(ret);
	return ret;
}

static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(bt_count)(PKG_NAME_(bt) bt){ return bt_count(bt); }

static CTMPL_INLINE void 
PKG_NAME_(bt_clear_by)(PKG_NAME_(bt) t, 
		       PKG_NAME_(btnode_access_func_t) df)
{
	PKG_NAME_(btnode_delete)(t, bt_root(t), df);
	bt_count(t) = 0;
	bt_root(t) = NULL;
}

static CTMPL_INLINE void 
PKG_NAME_(bt_delete_by)(PKG_NAME_(bt) t, 
			PKG_NAME_(btnode_access_func_t) df)
{
	PKG_NAME_(bt_clear_by)(t, df);
	CTMPL_FREE(t->free, PKG_NAME_STRING "bt", t);
}


static CTMPL_INLINE PKG_NAME_(bt) 
PKG_NAME_(bt_dup_by)(PKG_NAME_(bt) bt, 
		     PKG_NAME_(btnode_duplicate_func_t) f)
{
	PKG_NAME_(bt) ret;
	ctmpl_assert(bt);

	ret = CTMPL_MALLOC(bt->alloc, PKG_NAME_STRING "bt", 
			   sizeof(struct PKG_NAME_(bt)));
	ret->root = NULL;
	ret->cmpkey = bt->cmpkey;
	ret->alloc = bt->alloc;
	ret->free = bt->free;
	PKG_NAME_(bt_set_ctl)(ret);
	PKG_NAME_(bt_copy_recursive)(bt_root(bt), ret, f);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(bt_foreach_recursive)(PKG_NAME_(btnode) tn, 
				PKG_NAME_(btnode_foreach_func_t) f,
		      		void *arg)
{
	if(!tn)
		return;
	PKG_NAME_(bt_foreach_recursive)(btn_left(tn), f, arg);	
	if(f)
		f(btn_key(tn), btn_value(tn), arg);
	PKG_NAME_(bt_foreach_recursive)(btn_right(tn), f, arg);	
}

static CTMPL_INLINE void 
PKG_NAME_(bt_foreach)(PKG_NAME_(bt) t, 
		      PKG_NAME_(btnode_foreach_func_t) f,
		      void *arg)
{
	ctmpl_assert(t != NULL);
	PKG_NAME_(bt_foreach_recursive)(bt_root(t), f, arg);
}

static CTMPL_INLINE void
PKG_NAME_(bt_put)(PKG_NAME_(bt) t, 
		  PKG_NAME_(btkey_t) k, 
		  PKG_NAME_(btval_t) v)
{
	PKG_NAME_(btnode) node;
	ctmpl_assert(t != NULL);
	
#ifndef PKG_USE_MULTI /* not multi map */
	/* check value whose key is same */
	node = PKG_NAME_(btnode_find)(t, bt_root(t), k);

	/* value of same key already exists in table, swap */
	if(node){
		btn_key(node) = k;
		btn_value(node) = v;
		return;
	}
	
#endif /* PKG_USE_MULTI  */

	/* new node */
	node = CTMPL_MALLOC(t->alloc,
		PKG_NAME_STRING "btnode", sizeof(struct PKG_NAME_(btnode)));
	btn_key(node) = k;
	btn_value(node) = v;
	btn_parent(node) = NULL;
	btn_left(node) = NULL;
	btn_right(node) = NULL;

	bt_root(t) = PKG_NAME_(btnode_insert)(t, bt_root(t), node);
	return;
}

static PKG_NAME_(bt_iterator)
PKG_NAME_(bt_get)(PKG_NAME_(bt) t, 
                  PKG_NAME_(btkey_t) k)
{
	PKG_NAME_(bt_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
#ifndef PKG_USE_MULTI /* not multi map */
	iterator_subtype(ret) = 0;
#else 
	iterator_subtype(ret) = 1;
#endif /* PKG_USE_MULTI */
	iterator_aggregate(ret) = t;
	ctmpl_assert(t != NULL);
	
	iterator_item(ret) = PKG_NAME_(btnode_find)(t, bt_root(t), k);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(btnode_delete)(PKG_NAME_(bt) bt,
			 PKG_NAME_(btnode) btn,
			 PKG_NAME_(btnode_access_func_t) df)
{
	if(!btn)
		return;

	PKG_NAME_(btnode_delete)(bt, btn_left(btn), df);
	PKG_NAME_(btnode_delete)(bt, btn_right(btn), df);
	if(df)		(*df)(btn_key(btn), btn_value(btn));
	CTMPL_FREE(bt->free, PKG_NAME_STRING "btnode", btn);
}

static PKG_NAME_(btnode) 
PKG_NAME_(btnode_find_min)(PKG_NAME_(btnode) k)
{
	if(!k)	return NULL;
	else if(btn_left(k))
		return PKG_NAME_(btnode_find_min)(btn_left(k));
	return k;
}

static PKG_NAME_(btnode) 
PKG_NAME_(btnode_find_max)(PKG_NAME_(btnode) k)
{
	if(!k)	return NULL;
	else if(btn_right(k))
		return PKG_NAME_(btnode_find_max)(btn_right(k));
	return k;
}

static PKG_NAME_(btnode) 
PKG_NAME_(btnode_find)(PKG_NAME_(bt) bt, 
		   PKG_NAME_(btnode) btn, 
		   PKG_NAME_(btkey_t) key)
{
	int result;
	if(!btn)
		return NULL;
	result = (*bt->cmpkey)(key, btn_key(btn));
	if(result < 0)
		return PKG_NAME_(btnode_find)(bt, btn_left(btn), key);
	else if(result > 0)
		return PKG_NAME_(btnode_find)(bt, btn_right(btn), key);
	return btn;
}

static PKG_NAME_(btnode) PKG_NAME_(btnode_insert)(PKG_NAME_(bt) bt,
					      PKG_NAME_(btnode) btn1,
					      PKG_NAME_(btnode) btn2)
{
	int result;

	/* no parent node */
	if(!btn1){
		++bt_count(bt);
		return btn2;
	}

	result = (*bt->cmpkey)(btn_key(btn2), btn_key(btn1));
	if(result < 0){
		btn_left(btn1) = 
			PKG_NAME_(btnode_insert)(bt, btn_left(btn1), btn2);
		btn_parent(btn_left(btn1)) = btn1;
	}
#ifndef PKG_USE_MULTI /* not multi map */
	else if(result > 0){
#else  /* PKG_USE_MULTI */
	else if(result >= 0){
#endif /* PKG_USE_MULTI */
		btn_right(btn1) = 
			PKG_NAME_(btnode_insert)(bt, btn_right(btn1), btn2);
		btn_parent(btn_right(btn1)) = btn1;
	}
	else /* already same node is here */
		ctmpl_assert("Already same node exists!" == NULL);
	
	return btn1;
}

static void PKG_NAME_(btnode_remove)(PKG_NAME_(bt) bt,
				     PKG_NAME_(btnode) btn)
{
	PKG_NAME_(btnode) tmp = NULL;
	/* two children */
	if(btn_left(btn) && btn_right(btn)){
		/* minimum node from right tree */
		tmp = PKG_NAME_(btnode_find_min)(btn_right(btn));
		/* remove this node once */
		PKG_NAME_(btnode_remove)(bt, tmp);

		/* copy link variable into tmp */
		btn_parent(tmp) = btn_parent(btn);
		btn_left(tmp) = btn_left(btn);
		btn_right(tmp) = btn_right(btn);
		++bt_count(bt);	/* manually install */

		/* connect pointer to tmp */
		if(btn_parent(btn)){
			if(btn_left(btn_parent(btn)) == btn)
				btn_left(btn_parent(btn)) = tmp;
			else 
				btn_right(btn_parent(btn)) = tmp;
		}
		if(btn_left(btn))
			btn_parent(btn_left(btn)) = tmp;
		if(btn_right(btn))
			btn_parent(btn_right(btn)) = tmp;
	} 
	else {
		/* change parent pointer of child node */
		if(btn_left(btn)){
			btn_parent(btn_left(btn)) = btn_parent(btn);
			tmp = btn_left(btn);
		}
		else if(btn_right(btn)){
			btn_parent(btn_right(btn)) = btn_parent(btn);
			tmp = btn_right(btn);
		}

		/* change child pointer of parent node */
		if(btn_parent(btn)){
			if(btn_left(btn_parent(btn)) == btn)
				btn_left(btn_parent(btn)) = tmp;
			else if(btn_right(btn_parent(btn)) == btn)
				btn_right(btn_parent(btn)) = tmp;
			else 
				ctmpl_assert("Impl error" == NULL);
		}
	}

	/* if this poiner is root */
	if(bt_root(bt) == btn)
		bt_root(bt) = tmp;
	/* reset poiner */
	btn_parent(btn) = NULL;
	btn_left(btn) = NULL;
	btn_right(btn) = NULL;
	--bt_count(bt);
}

static void
PKG_NAME_(bt_copy_recursive)(PKG_NAME_(btnode) k,
			     PKG_NAME_(bt) t,
			     PKG_NAME_(btnode_duplicate_func_t) f)
{
	struct PKG_NAME_(bt_iterator_data) dup;
	if(k == NULL)
		return;
	if(f){
		dup = f(btn_key(k), btn_value(k));
		PKG_NAME_(bt_put)(t, dup.first, dup.second);
	}
	else
		PKG_NAME_(bt_put)(t, btn_key(k), btn_value(k));
	PKG_NAME_(bt_copy_recursive)(btn_left(k), t, f);
	PKG_NAME_(bt_copy_recursive)(btn_right(k), t, f);
}



static PKG_NAME_(btnode) 
PKG_NAME_(btnode_itr_next)(PKG_NAME_(bt), PKG_NAME_(btnode));
static PKG_NAME_(btnode) 
PKG_NAME_(btnode_itr_prev)(PKG_NAME_(bt), PKG_NAME_(btnode));

static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_first)(PKG_NAME_(bt) bt)
{
	PKG_NAME_(bt_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = bt;
	iterator_item(ret) = PKG_NAME_(btnode_find_min)(bt_root(bt));
	return ret;
}
static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_last)(PKG_NAME_(bt) bt)
{
	PKG_NAME_(bt_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = bt;
	iterator_item(ret) = PKG_NAME_(btnode_find_max)(bt_root(bt));
	return ret;
}

static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_at)(PKG_NAME_(bt) bt, iterator_pos_t idx)
{
	int prev = 0;
	PKG_NAME_(bt_iterator) ret;
	PKG_NAME_(btnode) btn;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = bt;
	
	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	btn = (prev? PKG_NAME_(btnode_find_max)(bt_root(bt)):
		     PKG_NAME_(btnode_find_min)(bt_root(bt)));

	while(idx){
		btn = (prev? PKG_NAME_(btnode_itr_prev)(bt, btn):
			     PKG_NAME_(btnode_itr_next)(bt, btn));
		--idx;
	}
	iterator_item(ret) = btn;
	return ret;
}

static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_next)(PKG_NAME_(bt_iterator) itr)
{
	PKG_NAME_(bt) bt = iterator_aggregate(itr);
	PKG_NAME_(bt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return itr;
	iterator_item(ret) = PKG_NAME_(btnode_itr_next)
			(iterator_aggregate(ret), iterator_item(itr));

	/* multi map iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*bt->cmpkey)(btn_key(iterator_item(itr)), 
		         btn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}
static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_prev)(PKG_NAME_(bt_iterator) itr)
{
	PKG_NAME_(bt) bt = iterator_aggregate(itr);
	PKG_NAME_(bt_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return itr;
	iterator_item(ret) = PKG_NAME_(btnode_itr_prev)
			(iterator_aggregate(ret), iterator_item(itr));

	/* multi map iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*bt->cmpkey)(btn_key(iterator_item(itr)), 
		         btn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}

static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_erase_gonext)(PKG_NAME_(bt_iterator) itr)
{
	PKG_NAME_(bt) t;
	PKG_NAME_(btnode) n;
	PKG_NAME_(bt_iterator) ret;

	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(bt_itr_next)(itr);
	t = iterator_aggregate(itr);
	n = iterator_item(itr);
	PKG_NAME_(btnode_remove)(t, n);
	CTMPL_FREE(t->free, PKG_NAME_STRING "btnode", n);
	return ret;
}

static PKG_NAME_(bt_iterator) 
PKG_NAME_(bt_itr_erase_goprev) (PKG_NAME_(bt_iterator) itr)
{
	PKG_NAME_(bt) t;
	PKG_NAME_(btnode) n;
	PKG_NAME_(bt_iterator) ret;

	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(bt_itr_prev)(itr);
	t = iterator_aggregate(itr);
	n = iterator_item(itr);
	PKG_NAME_(btnode_remove)(t, n);
	CTMPL_FREE(t->free, PKG_NAME_STRING "btnode", n);
	return ret;
}

static int 
PKG_NAME_(bt_itr_is_equal)(PKG_NAME_(bt_iterator) x,
                           PKG_NAME_(bt_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static PKG_NAME_(bt_iterator_data) 
PKG_NAME_(bt_itr_retrieve)(PKG_NAME_(bt_iterator) itr)
{
	return (PKG_NAME_(bt_iterator_data))iterator_item(itr);
}

static PKG_NAME_(btnode) 
PKG_NAME_(btnode_itr_next)(PKG_NAME_(bt) bt, PKG_NAME_(btnode) btn)
{
	PKG_NAME_(btnode) pp, pq;
	if(!btn)
		return NULL;

	/* has right node */
	if(btn_right(btn))
		return PKG_NAME_(btnode_find_min)(btn_right(btn));

	/* this node is parent's left node */
	if(btn_parent(btn) && btn_left(btn_parent(btn)) == btn) 
		return btn_parent(btn);

	pp = pq = btn;
	while((pp = btn_parent(pq)) != NULL){
		/* this node is parent's left node */
		if(btn_left(btn_parent(pq)) == pq) 
			return pp;
		pq = pp;
	}
	return NULL;
}

static PKG_NAME_(btnode) 
PKG_NAME_(btnode_itr_prev)(PKG_NAME_(bt) bt, PKG_NAME_(btnode) btn)
{
	PKG_NAME_(btnode) pp, pq;
	if(!btn)
		return NULL;

	/* has left node */
	if(btn_left(btn))
		return PKG_NAME_(btnode_find_max)(btn_left(btn));

	/* this node is parent's right node */
	if(btn_parent(btn) && btn_right(btn_parent(btn)) == btn) 
		return btn_parent(btn);

	pp = pq = btn;
	while((pp = btn_parent(pq)) != NULL){
		/* this node is parent's right node */
		if(btn_right(btn_parent(pq)) == pq) 
			return pp;
		pq = pp;
	}
	return NULL;
}

static PKG_NAME_(bt_ctl) PKG_NAME_(bt_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(bt_delete_by),
	PKG_NAME_(bt_clear_by),
	PKG_NAME_(bt_dup_by),
	PKG_NAME_(bt_foreach),
	PKG_NAME_(bt_count),
	PKG_NAME_(bt_itr_first),
	PKG_NAME_(bt_itr_last),
	PKG_NAME_(bt_itr_at),
	PKG_NAME_(bt_itr_next),
	PKG_NAME_(bt_itr_prev),
	PKG_NAME_(bt_itr_erase_gonext),
	PKG_NAME_(bt_itr_erase_goprev),
	PKG_NAME_(bt_itr_is_equal),	
	PKG_NAME_(bt_itr_retrieve),
	PKG_NAME_(bt_get),
	PKG_NAME_(bt_put)
};

static CTMPL_INLINE void 
PKG_NAME_(bt_set_ctl)(PKG_NAME_(bt) btp)
{
	btp->ctl = &PKG_NAME_(bt_ctl_sw);
}
