/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/**
 * 	ctmpl/bt.h
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 * 		$Id: bintree.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *	<Binary Tree Template>
 */

typedef struct PKG_NAME_(bt) *PKG_NAME_(bt);
typedef struct PKG_NAME_(btnode) *PKG_NAME_(btnode);
typedef struct PKG_NAME_(bt_iterator) PKG_NAME_(bt_iterator);
typedef struct PKG_NAME_(bt_ctl) PKG_NAME_(bt_ctl);
typedef struct PKG_NAME_(bt_iterator_data) *PKG_NAME_(bt_iterator_data); 

#include "ctmpl/memory.h"
#include "ctmpl/ctmpl.h"

#if !defined(PKG_MAPKEY_T) || !defined(PKG_MAPVAL_T)
#error "PKG_MAPKEY_T & PKG_MAPVAL_T must be defined."
#else
typedef PKG_MAPKEY_T PKG_NAME_(btkey_t);
typedef PKG_MAPVAL_T PKG_NAME_(btval_t);
#endif /* !defined(PKG_MAPKEY_T) || !defined(PKG_MAPVAL_T) */
/*	
 *  PKG_NAME_(bt_cmpkey_func_t):
 *	this function compares two keys.
 */
typedef int (*PKG_NAME_(bt_cmpkey_func_t))
	(PKG_NAME_(btkey_t), PKG_NAME_(btkey_t));


/* iterator for bt */
struct PKG_NAME_(bt_iterator){
	short			iter_type;  /* iterator type */
	short			iter_subtype;  /* iterator subtype */
	PKG_NAME_(bt) 		iter_aggr;  /* iterator aggregate */
	PKG_NAME_(btnode) 	iter_item;  /* iterator item */
};

struct PKG_NAME_(bt_iterator_data){
	PKG_NAME_(btkey_t) first;	/* key */
	PKG_NAME_(btval_t) second;	/* value */
};

struct PKG_NAME_(btnode){
	struct PKG_NAME_(bt_iterator_data) data;
	PKG_NAME_(btnode)  parent;	/* parent node */
	PKG_NAME_(btnode)  left;	/* left node */
	PKG_NAME_(btnode)  right;	/* right node */
};

struct PKG_NAME_(bt) {
	/* root node */
	PKG_NAME_(btnode) 	root;

	/* number of contained value */
	int 			count;	

	/* compare key function */
	PKG_NAME_(bt_cmpkey_func_t)  cmpkey;  
	m_alloc_func_t 	alloc;	    /* malloc function */
	m_free_func_t 	free;	    /* free function */

	/* iterator controller */
	PKG_NAME_(bt_ctl) *ctl;
};

/*
 *  btnode_access_func_t
 *	this function deletes bt node. 
 */
typedef void  (*PKG_NAME_(btnode_access_func_t))(PKG_NAME_(btkey_t),
						 PKG_NAME_(btval_t));

/*
 *  btnode_foreach_func_t
 */
typedef void  (*PKG_NAME_(btnode_foreach_func_t))(PKG_NAME_(btkey_t),
						  PKG_NAME_(btval_t),
						  void *);


/*
 *  btnode_duplicate_func_t
 *	this function duplicate bt node. 
 */
typedef struct PKG_NAME_(bt_iterator_data) (*PKG_NAME_(btnode_duplicate_func_t))
					(PKG_NAME_(btkey_t), PKG_NAME_(btval_t));


/* iterator controller for hasbtet */
struct PKG_NAME_(bt_ctl){
	PKG_NAME_(bt_iterator) enditr;
	void		       (*delete_by)(PKG_NAME_(bt), 
					PKG_NAME_(btnode_access_func_t));
	void		       (*clear_by)(PKG_NAME_(bt), 
					  PKG_NAME_(btnode_access_func_t));
	PKG_NAME_(bt)	       (*dup_by)(PKG_NAME_(bt), 
					 PKG_NAME_(btnode_duplicate_func_t));
	void		       (*foreach)(PKG_NAME_(bt), 
					  PKG_NAME_(btnode_foreach_func_t),
					  void *);
	iterator_cnt_t	       (*count)(PKG_NAME_(bt));
	PKG_NAME_(bt_iterator) (*first)(PKG_NAME_(bt));
	PKG_NAME_(bt_iterator) (*last)(PKG_NAME_(bt));
	PKG_NAME_(bt_iterator) (*at)(PKG_NAME_(bt), iterator_pos_t);
	PKG_NAME_(bt_iterator) (*next)(PKG_NAME_(bt_iterator));
	PKG_NAME_(bt_iterator) (*prev)(PKG_NAME_(bt_iterator));
	PKG_NAME_(bt_iterator) (*erase_gonext)(PKG_NAME_(bt_iterator));
	PKG_NAME_(bt_iterator) (*erase_goprev)(PKG_NAME_(bt_iterator));
	int			(*is_equal)(PKG_NAME_(bt_iterator),
					    PKG_NAME_(bt_iterator));
	PKG_NAME_(bt_iterator_data) 
			       (*retrieve)(PKG_NAME_(bt_iterator));
	PKG_NAME_(bt_iterator) (*find)(PKG_NAME_(bt), PKG_NAME_(btkey_t));
	void		       (*insert_pair)(PKG_NAME_(bt), 
					      PKG_NAME_(btkey_t),
					      PKG_NAME_(btval_t));
};

/* prototypes */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(bt) 
PKG_NAME_(bt_new)(PKG_NAME_(bt_cmpkey_func_t));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(bt) 
PKG_NAME_(bt_new_by)(PKG_NAME_(bt_cmpkey_func_t),
	 	     m_alloc_func_t, 
		     m_free_func_t);

