/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _CTMPL_BINTREE_SET_C_
#define _CTMPL_BINTREE_SET_C_
#define bts_root(x)	bt_root((x)->bt)
#define bts_count(x)	bt_count((x)->bt)
#define bts_isempty(x)	(bts_count(x) == 0)
#define btsn_key(t)	((t)->data.first)
#define btsn_value(t)	((t)->data.second)
#define btsn_left(t)	((t)->left)
#define btsn_right(t)	((t)->right)
#define btsn_parent(t)	((t)->parent)
#endif	/* _CTMPL_BINTREE_C_ */

#include "ctmpl/bintree.c"

static CTMPL_INLINE void PKG_NAME_(bts_set_ctl)(PKG_NAME_(bts));
static void PKG_NAME_(bts_copy_recursive)(PKG_NAME_(btsnode) k, 
					  PKG_NAME_(bts) t,
					  PKG_NAME_(btsnode_duplicate_func_t) f);
static CTMPL_INLINE void 
PKG_NAME_(btsnode_delete)(PKG_NAME_(bts) bts,
			  PKG_NAME_(btsnode) btsn,
			  PKG_NAME_(btsnode_access_func_t) df);

static CTMPL_INLINE void PKG_NAME_(bts_set_ctl)(PKG_NAME_(bts));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(bts) 
PKG_NAME_(bts_new)(PKG_NAME_(bts_cmpval_func_t) ck)
{
	return PKG_NAME_(bts_new_by)(ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(bts) 
PKG_NAME_(bts_new_by)(PKG_NAME_(bts_cmpval_func_t) cmp_func,
		     m_alloc_func_t af,	m_free_func_t ff)
{
	PKG_NAME_(bts) ret;
	ctmpl_assert(cmp_func != NULL);

	/* default alloc/free function */
	if(!af)	 af = PKG_NAME_(m_alloc);
	if(!ff)	 ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "bts", 
			   sizeof(struct PKG_NAME_(bts)));
	ret->bt = PKG_NAME_(bt_new_by)(cmp_func, af, ff);
	PKG_NAME_(bts_set_ctl)(ret);
	return ret;
}

static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(bts_count)(PKG_NAME_(bts) bts){ return bts_count(bts); }

static CTMPL_INLINE void
PKG_NAME_(bts_put)(PKG_NAME_(bts) t, 
		   PKG_NAME_(btsval_t) v)
{
	PKG_NAME_(bt_put)(t->bt, v, v);
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_get)(PKG_NAME_(bts) t, 
		   PKG_NAME_(btsval_t) v)
{
	PKG_NAME_(bt_iterator) btitr;
	PKG_NAME_(bts_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_aggregate(ret) = t;
	ctmpl_assert(t != NULL);
	
	btitr = iterator_find(t->bt, v);
	iterator_subtype(ret) = iterator_subtype(btitr);
	iterator_item(ret) = iterator_item(btitr);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(bts_clear_by)(PKG_NAME_(bts) t, 
		        PKG_NAME_(btsnode_access_func_t) df)
{
	PKG_NAME_(btsnode_delete)(t, bts_root(t), df);
	bts_root(t) = NULL;
	bts_count(t) = 0;
}

static CTMPL_INLINE void 
PKG_NAME_(bts_delete_by)(PKG_NAME_(bts) t, 
			PKG_NAME_(btsnode_access_func_t) df)
{
	m_free_func_t free_func = t->bt->free;
	PKG_NAME_(bts_clear_by)(t, df);
	PKG_NAME_(bt_delete_by)(t->bt, NULL);
	CTMPL_FREE(free_func, PKG_NAME_STRING "bts", t);
}


static CTMPL_INLINE PKG_NAME_(bts) 
PKG_NAME_(bts_dup_by)(PKG_NAME_(bts) bts, 
		      PKG_NAME_(btsnode_duplicate_func_t) f)
{
	PKG_NAME_(bts) ret;
	ctmpl_assert(bts);

	ret = PKG_NAME_(bts_new_by)(bts->bt->cmpkey,
				    bts->bt->alloc,
			  	    bts->bt->free);
	PKG_NAME_(bts_copy_recursive)(bts_root(bts), ret, f);
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(bts_foreach_recursive)(PKG_NAME_(btsnode) tn, 
				 PKG_NAME_(btsnode_foreach_func_t) f,
		      		 void *arg)
{
	if(!tn)
		return;
	PKG_NAME_(bts_foreach_recursive)(btsn_left(tn), f, arg);	
	if(f)
		f(btsn_value(tn), arg);
	PKG_NAME_(bts_foreach_recursive)(btsn_right(tn), f, arg);	
}

static CTMPL_INLINE void 
PKG_NAME_(bts_foreach)(PKG_NAME_(bts) t, 
		       PKG_NAME_(btsnode_foreach_func_t) f,
		       void *arg)
{
	ctmpl_assert(t != NULL);
	PKG_NAME_(bts_foreach_recursive)(bts_root(t), f, arg);
}

static CTMPL_INLINE void 
PKG_NAME_(btsnode_delete)(PKG_NAME_(bts) bts,
			  PKG_NAME_(btsnode) btsn,
			  PKG_NAME_(btsnode_access_func_t) df)
{
	if(!btsn)
		return;

	PKG_NAME_(btsnode_delete)(bts, btsn_left(btsn), df);
	PKG_NAME_(btsnode_delete)(bts, btsn_right(btsn), df);
	if(df)		(*df)(btsn_value(btsn));
	CTMPL_FREE(bts->bt->free, PKG_NAME_STRING "btnode", btsn);
}


static void
PKG_NAME_(bts_copy_recursive)(PKG_NAME_(btsnode) k,
			     PKG_NAME_(bts) t,
			     PKG_NAME_(btsnode_duplicate_func_t) f)
{
	PKG_NAME_(btsval_t) dup;	
	if(k == NULL)
		return;
	if(f){
		dup = f(btsn_value(k));
		PKG_NAME_(bt_put)(t->bt, dup, dup);
	}
	else
		PKG_NAME_(bt_put)(t->bt, btsn_key(k), btsn_value(k));
	PKG_NAME_(bts_copy_recursive)(btsn_left(k), t, f);
	PKG_NAME_(bts_copy_recursive)(btsn_right(k), t, f);
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_first)(PKG_NAME_(bts) bts)
{
	PKG_NAME_(bts_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = bts;
	iterator_item(ret) = PKG_NAME_(btnode_find_min)(bts_root(bts));
	return ret;
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_last)(PKG_NAME_(bts) bts)
{
	PKG_NAME_(bts_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = bts;
	iterator_item(ret) = PKG_NAME_(btnode_find_max)(bts_root(bts));
	return ret;
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_at)(PKG_NAME_(bts) bts, iterator_pos_t idx)
{
	int prev = 0;
	PKG_NAME_(bts_iterator) ret;
	PKG_NAME_(btsnode) btsn;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = bts;
	
	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	btsn = (prev? PKG_NAME_(btnode_find_max)(bts_root(bts)):
		     PKG_NAME_(btnode_find_min)(bts_root(bts)));

	while(idx){
		btsn = (prev? PKG_NAME_(btnode_itr_prev)(bts->bt, btsn):
			     PKG_NAME_(btnode_itr_next)(bts->bt, btsn));
		--idx;
	}
	iterator_item(ret) = btsn;
	return ret;
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_next)(PKG_NAME_(bts_iterator) itr)
{
	PKG_NAME_(bts) t = iterator_aggregate(itr);
	PKG_NAME_(bts_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = PKG_NAME_(btnode_itr_next)
			(iterator_aggregate(ret)->bt, iterator_item(itr));

	/* multi set iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*t->bt->cmpkey)(btn_key(iterator_item(itr)),
			    btn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;

	return ret;
}
static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_prev)(PKG_NAME_(bts_iterator) itr)
{
	PKG_NAME_(bts) t = iterator_aggregate(itr);
	PKG_NAME_(bts_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;
	iterator_item(ret) = PKG_NAME_(btnode_itr_prev)
			(iterator_aggregate(ret)->bt, iterator_item(itr));

	/* multi set iterator */
	if(iterator_subtype(ret) && iterator_item(ret) &&
	   (*t->bt->cmpkey)(btn_key(iterator_item(itr)),
			    btn_key(iterator_item(ret))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_erase_gonext)(PKG_NAME_(bts_iterator) itr)
{
	PKG_NAME_(bts) t = iterator_aggregate(itr);
	PKG_NAME_(btsnode) n = iterator_item(itr);
	PKG_NAME_(bts_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(bts_itr_next)(itr);
	PKG_NAME_(btnode_remove)(t->bt, n);
	CTMPL_FREE(t->bt->free, PKG_NAME_STRING "btnode", n);
	return ret;
}

static PKG_NAME_(bts_iterator) 
PKG_NAME_(bts_itr_erase_goprev) (PKG_NAME_(bts_iterator) itr)
{
	PKG_NAME_(bts) t = iterator_aggregate(itr);
	PKG_NAME_(btsnode) n = iterator_item(itr);
	PKG_NAME_(bts_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(bts_itr_prev)(itr);
	PKG_NAME_(btnode_remove)(t->bt, n);
	CTMPL_FREE(t->bt->free, PKG_NAME_STRING "btnode", n);
	return ret;
}

static int 
PKG_NAME_(bts_itr_is_equal)(PKG_NAME_(bts_iterator) x,
                            PKG_NAME_(bts_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static PKG_NAME_(btsval_t) 
PKG_NAME_(bts_itr_retrieve)(PKG_NAME_(bts_iterator) itr)
{
	return btn_key(iterator_item(itr));
}

static PKG_NAME_(bts_ctl) PKG_NAME_(bts_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(bts_delete_by),
	PKG_NAME_(bts_clear_by),
	PKG_NAME_(bts_dup_by),
	PKG_NAME_(bts_foreach),
	PKG_NAME_(bts_count),
	PKG_NAME_(bts_itr_first),
	PKG_NAME_(bts_itr_last),
	PKG_NAME_(bts_itr_at),
	PKG_NAME_(bts_itr_next),
	PKG_NAME_(bts_itr_prev),
	PKG_NAME_(bts_itr_erase_gonext),
	PKG_NAME_(bts_itr_erase_goprev),
	PKG_NAME_(bts_itr_is_equal),
	PKG_NAME_(bts_itr_retrieve),
	PKG_NAME_(bts_get),
	PKG_NAME_(bts_put)
};

static CTMPL_INLINE void 
PKG_NAME_(bts_set_ctl)(PKG_NAME_(bts) btsp)
{
	btsp->ctl = &PKG_NAME_(bts_ctl_sw);
}
