/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/**
 * 	ctmpl/hashset.h
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 * 		$Id: hashset.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *	<Hashset Template>
 *	
 */

#if  !defined(PKG_SETVAL_T)
#error "PKG_SETVAL_T must be defined."
#endif /* PKG_SETVAL_T */

/* use slist.h */
#include "ctmpl/slist.h"
typedef struct PKG_NAME_(hs) *PKG_NAME_(hs);
typedef PKG_NAME_(slistnode)  PKG_NAME_(hsnode);
typedef PKG_SETVAL_T 	      PKG_NAME_(hsval_t);
typedef struct PKG_NAME_(hs_iterator) PKG_NAME_(hs_iterator);
typedef struct PKG_NAME_(hs_ctl) PKG_NAME_(hs_ctl);

/*	
 *  PKG_NAME_(hsval_hash_func_t):
 *	this function generates integer key from value.
 *	returned integer can be greater than size of hashset.
 */
typedef unsigned int (*PKG_NAME_(hsval_hash_func_t))(PKG_NAME_(hsval_t));

/*	
 *  PKG_NAME_(hsval_cmp_func_t):
 *	this function compares two value.
 */
typedef int (*PKG_NAME_(hsval_cmp_func_t))
	(PKG_NAME_(hsval_t), PKG_NAME_(hsval_t));

/*
 *  hsval_access_func_t
 *	this function deletes value of hashset node. 
 */
typedef void  (*PKG_NAME_(hsval_access_func_t))(PKG_NAME_(hsval_t));

/*
 *  hsval_foreach_func_t
 */
typedef void  (*PKG_NAME_(hsval_foreach_func_t))(PKG_NAME_(hsval_t), 
						 void *);

/*
 *  hsval_duplicate_func_t
 *	this function returns duplicated value of hashset node. 
 */
typedef PKG_NAME_(hsval_t) 
	(*PKG_NAME_(hsval_duplicate_func_t))(PKG_NAME_(hsval_t));

/* iterator for hsahset */
struct PKG_NAME_(hs_iterator){
	short			iter_type;  /* iterator type */
	short			iter_subtype;  /* iterator subtype */
	PKG_NAME_(hs) 		iter_aggr;  /* iterator aggregate */
	PKG_NAME_(hsnode) 	iter_item;  /* iterator item */
};

struct PKG_NAME_(hs) {
	PKG_NAME_(slist) *set;	/* set */
	int size;	/* set size */
	int count;	/* number of contained value */
	PKG_NAME_(hsval_hash_func_t) hash;  /* hash function */
	PKG_NAME_(hsval_cmp_func_t)  cmp;   /* compare function */
	m_alloc_func_t 	alloc;	    /* malloc function */
	m_free_func_t 	free;	    /* free function */

	/* iterator controller for hashset */
	PKG_NAME_(hs_ctl) *ctl;
};

/* iterator controller for hashset */
struct PKG_NAME_(hs_ctl){
	PKG_NAME_(hs_iterator) 	enditr;
	void		      	(*delete_by)(PKG_NAME_(hs), 
				   PKG_NAME_(listdata_access_func_t));
	void		      	(*clear_by)(PKG_NAME_(hs), 
				   PKG_NAME_(listdata_access_func_t));
	PKG_NAME_(hs)		(*dup_by)(PKG_NAME_(hs),
				   PKG_NAME_(listdata_duplicate_func_t));
	void		      	(*foreach)(PKG_NAME_(hs), 
				   PKG_NAME_(listdata_foreach_func_t), void *);
	iterator_cnt_t	    	(*count)(PKG_NAME_(hs));

	PKG_NAME_(hs_iterator) (*first)(PKG_NAME_(hs));
	PKG_NAME_(hs_iterator) (*last)(PKG_NAME_(hs));
	PKG_NAME_(hs_iterator) (*at)(PKG_NAME_(hs), iterator_pos_t);
	PKG_NAME_(hs_iterator) (*next)(PKG_NAME_(hs_iterator));
	PKG_NAME_(hs_iterator) (*prev)(PKG_NAME_(hs_iterator));
	PKG_NAME_(hs_iterator) (*erase_gonext)(PKG_NAME_(hs_iterator));
	PKG_NAME_(hs_iterator) (*erase_goprev)(PKG_NAME_(hs_iterator));
	int		       (*is_equal)(PKG_NAME_(hs_iterator), 
					   PKG_NAME_(hs_iterator));
	PKG_NAME_(hsval_t)     (*retrieve)(PKG_NAME_(hs_iterator));
	PKG_NAME_(hs_iterator) (*find)(PKG_NAME_(hs),PKG_NAME_(hsval_t));
	void		       (*insert)(PKG_NAME_(hs),PKG_NAME_(hsval_t));
};

/* prototypes */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(hs) 
PKG_NAME_(hs_new)(int, 	 	/* initial table size */
		  PKG_NAME_(hsval_hash_func_t), 
		  PKG_NAME_(hsval_cmp_func_t));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(hs) 
PKG_NAME_(hs_new_by)(int,  	/* initial table size */
		     PKG_NAME_(hsval_hash_func_t), 
		     PKG_NAME_(hsval_cmp_func_t),
	 	     m_alloc_func_t, 
		     m_free_func_t);
