/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "ctmpl/slist.c"

#ifndef _CTMPL_HASHTABLE_C_
#define _CTMPL_HASHTABLE_C_
#define ht_size(x)	((x)->size)
#define ht_count(x)	((x)->count)
#define ht_isempty(x)	(ht_count(x) == 0)
#define ht_hash(t, k)	(((*(t)->hashkey)(k)) % t->size)
#define ht_entry(t, k)	((t)->table[ht_hash(t, k)])

#define htn_key(t)	((t)->first)
#define htn_value(t)	((t)->second)

/*  !! obsolete !!  */
/**********************************************/
/*  !!be careful!! last argument is htnode!!  */
/**********************************************/
/* ht/int/slist/listnode/PKG_NAME_(htnode) */
#define CTMPL_HT_FOREACH(T, I, L, LN, HN) \
	for((I) = 0; (I) < (T)->size; (I)++) \
		if(((L) = (T)->table[I]) != NULL) \
			CTMPL_SLIST_FOREACH(L, LN, (void *)HN)
#endif	/* _CTMPL_HASHTABLE_C_ */

static CTMPL_INLINE void PKG_NAME_(ht_set_ctl)(PKG_NAME_(ht));

/* static functions */
static CTMPL_INLINE PKG_NAME_(slistnode) PKG_NAME_(htslnode_get)
	(PKG_NAME_(ht) t, PKG_NAME_(htkey_t) k);

static CTMPL_INLINE PKG_NAME_(htnode) PKG_NAME_(htnode_put)
	(PKG_NAME_(ht) t, PKG_NAME_(htnode) n);


/* these functions are used for PKG_NAME_(slist) of PKG_NAME_(ht) */
static CTMPL_INLINE PKG_NAME_(listdata_t) PKG_NAME_(ht_slistnode_add)
	(PKG_NAME_(slist) s, PKG_NAME_(listdata_t) v);

static CTMPL_INLINE  struct PKG_NAME_(ht_iterator_data)
PKG_NAME_(htnode_remove)(PKG_NAME_(ht) t, PKG_NAME_(slist) l,
			 PKG_NAME_(slistnode) n)
{
	struct PKG_NAME_(ht_iterator_data) ret;

	PKG_NAME_(htnode) htn = (PKG_NAME_(htnode))
		PKG_NAME_(listnode_remove)((PKG_NAME_(list))l, n);

	ret.first = htn_key(htn);
	ret.second = htn_value(htn);

	ht_count(t)--;
	ctmpl_assert(ht_count(t) >= 0);
	CTMPL_FREE(t->free, PKG_NAME_STRING "htnode", htn);

	return ret;
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(ht) 
PKG_NAME_(ht_new)(int s, 
		  PKG_NAME_(ht_hash_func_t) hk, 
		  PKG_NAME_(ht_cmpkey_func_t) ck)
{
	return PKG_NAME_(ht_new_by)(s, hk, ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(ht) 
PKG_NAME_(ht_new_by)(int s, 
		     PKG_NAME_(ht_hash_func_t) hash_func, 
		     PKG_NAME_(ht_cmpkey_func_t) cmpkey_func,
		     m_alloc_func_t af, 	
		     m_free_func_t ff)
{
	PKG_NAME_(ht) ret;
	ctmpl_assert(s > 0);
	ctmpl_assert(hash_func != NULL);
	ctmpl_assert(cmpkey_func != NULL);

        /* default alloc/free function */
	if(!af)		af = PKG_NAME_(m_alloc);
	if(!ff)		ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "ht", 
			sizeof(struct PKG_NAME_(ht)));
	ret->table = CTMPL_MALLOC(af,
		PKG_NAME_STRING "ht->table", sizeof(PKG_NAME_(slist)) * s);
	ret->size = s;
	ret->hashkey = hash_func;
	ret->cmpkey = cmpkey_func;
	ret->alloc = af;
	ret->free = ff;
	PKG_NAME_(ht_set_ctl)(ret);
	return ret;
}

static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(ht_count)(PKG_NAME_(ht) t)
{
	return ht_count(t);
}

static CTMPL_INLINE void
PKG_NAME_(ht_put)(PKG_NAME_(ht) t, 
		  PKG_NAME_(htkey_t) k, 
		  PKG_NAME_(htval_t) v)
{
	PKG_NAME_(htnode) node;
	PKG_NAME_(slistnode) slnode;
	ctmpl_assert(t != NULL);
	
	/* check value whose key is same */
	slnode = PKG_NAME_(htslnode_get)(t, k);

	/* value of same key already exists in table, swap */
	if(slnode){
#ifndef  PKG_USE_MULTI
		node = sln_data(slnode);
		htn_key(node) = k;
		htn_value(node) = v;
#else
                node = CTMPL_MALLOC(t->alloc,
                                    PKG_NAME_STRING "htnode", 
                                    sizeof(struct PKG_NAME_(ht_iterator_data)))
;
                htn_key(node) = k;
                htn_value(node) = v;
                PKG_NAME_(listnode_insert)((PKG_NAME_(list))ht_entry(t, k), 
					   sln_prev(slnode), node);
		ht_count(t)++;
#endif /* PKG_USE_MULTI */
		return;
	}
	
	/* new node */
	node = CTMPL_MALLOC(t->alloc, PKG_NAME_STRING "htnode", 
			sizeof(struct PKG_NAME_(ht_iterator_data)));
	htn_key(node) = k;
	htn_value(node) = v;

	(void)PKG_NAME_(htnode_put)(t, node); /* no node returns */
	return; 
}

static CTMPL_INLINE PKG_NAME_(ht_iterator)
PKG_NAME_(ht_get)(PKG_NAME_(ht) t, 
		  PKG_NAME_(htkey_t) k)
{
	PKG_NAME_(ht_iterator) ret;
	iterator_set_forward(ret);
#ifndef  PKG_USE_MULTI
	iterator_subtype(ret) = 0;
#else  
	iterator_subtype(ret) = 1;
#endif /* PKG_USE_MULTI */

	iterator_aggregate(ret) = t;
	ctmpl_assert(t != NULL);

	iterator_item(ret) = PKG_NAME_(htslnode_get)(t, k);
	return ret;
}
static CTMPL_INLINE void  
PKG_NAME_(ht_clear_by)(PKG_NAME_(ht) t,
		       PKG_NAME_(htnode_access_func_t) df)
{
	int i;
	ctmpl_assert(t);

	for(i = 0; i < ht_size(t); i++) {
		PKG_NAME_(slist) sl;
		PKG_NAME_(listnode) ln, next; 
		struct PKG_NAME_(ht_iterator_data) idat;

		if((sl = t->table[i]) != NULL){
			for(ln = sl_headnode(sl); ln ; ln = next){
				next = ln->next;
				idat = PKG_NAME_(htnode_remove)(t, sl, ln);
				if(df)	(*df)(idat.first, idat.second);
			}
			aggregator_delete(sl);
			t->table[i] = NULL;
		}
	}
}

static CTMPL_INLINE void 
PKG_NAME_(ht_delete_by)(PKG_NAME_(ht) t, 
			PKG_NAME_(htnode_access_func_t) df)
{
	PKG_NAME_(ht_clear_by)(t, df);
	CTMPL_FREE(t->free, PKG_NAME_STRING "ht->table", t->table);
	CTMPL_FREE(t->free, PKG_NAME_STRING "ht", t);
}

static CTMPL_INLINE PKG_NAME_(ht) 
PKG_NAME_(ht_dup_by)(PKG_NAME_(ht) t, PKG_NAME_(htnode_duplicate_func_t) f)
{
	int i;
	PKG_NAME_(slist) sl;
	PKG_NAME_(listnode) ln;
	PKG_NAME_(htnode) htn;
	PKG_NAME_(ht) ret;
	struct PKG_NAME_(ht_iterator_data) dup;

	ctmpl_assert(t != NULL);
	ret = PKG_NAME_(ht_new_by)(t->size, t->hashkey, t->cmpkey,
				   t->alloc, t->free);

	CTMPL_HT_FOREACH(t, i, sl, ln, htn){
		if(!f)
			PKG_NAME_(ht_put)(ret, htn_key(htn), htn_value(htn));	
		else {
			dup = f(htn_key(htn), htn_value(htn));
			PKG_NAME_(ht_put)(ret, dup.first, dup.second);	
		}
	}

	return ret;
}

static CTMPL_INLINE void  
PKG_NAME_(ht_foreach)(PKG_NAME_(ht) t,
		      PKG_NAME_(htnode_foreach_func_t) func,
		      void *arg)
{
	int i;
	PKG_NAME_(slist) sl;
	PKG_NAME_(listnode) ln;
	PKG_NAME_(htnode) htn;

	ctmpl_assert(t != NULL);

	CTMPL_HT_FOREACH(t, i, sl, ln, htn){
		if(func)
			func(htn_key(htn), htn_value(htn), arg);	
	}
}

static CTMPL_INLINE PKG_NAME_(slistnode) 
PKG_NAME_(htslnode_get)(PKG_NAME_(ht) t, 
		      	PKG_NAME_(htkey_t) k)
{
	/* no check because of static function */
	PKG_NAME_(slist) sl = ht_entry(t, k);
	PKG_NAME_(slistnode) n1;
	int cmp = 0;
	int pos, mid, left, right, last;

	if(sl == NULL || sl_isempty(sl))
		return NULL;

	left = 0;
	last = right = sl_nodecount(sl) - 1;
	n1 = sl_headnode(sl);
	pos = 0;
	
	while(left < right){
		mid = (left + right) / 2;
		n1 = PKG_NAME_(slistnode_go)(n1, mid - pos);
		pos = mid;

		cmp = (*t->cmpkey)(htn_key(sln_data(n1)), k);
		if(cmp < 0)
			left = mid + 1;
		else 
			right = mid;
	}
	n1 = PKG_NAME_(slistnode_go)(n1, left - pos);
	cmp = (*t->cmpkey)(htn_key(sln_data(n1)), k);
	if(!cmp)
		return n1;
	return NULL;
}

static CTMPL_INLINE PKG_NAME_(htnode) 
PKG_NAME_(htnode_put)(PKG_NAME_(ht) t, 
		      PKG_NAME_(htnode) n)
{
	PKG_NAME_(slist) sl = ht_entry(t, htn_key(n));
	PKG_NAME_(htnode) ret;	/* old node */

	/* no list */
	if(sl == NULL){
		ht_entry(t, htn_key(n)) = PKG_NAME_(slist_new_by)
				((PKG_NAME_(listdata_compare_func_t))t->cmpkey,
			 		t->alloc, t->free);
		sl = ht_entry(t, htn_key(n));
	}
	
	ret = (PKG_NAME_(htnode))PKG_NAME_(ht_slistnode_add)(sl, n);
	if(ret == NULL) /* not replace */
		ht_count(t)++;
	return ret;
}


static CTMPL_INLINE PKG_NAME_(listdata_t) 
PKG_NAME_(ht_slistnode_add)(PKG_NAME_(slist) s, 
			    PKG_NAME_(listdata_t) v)
{
	int cmp = 0;
	int pos, mid, left, right, last;
	PKG_NAME_(list) lp;
	PKG_NAME_(listdata_t) ret = 0; 
	register PKG_NAME_(listnode) n1;
	PKG_NAME_(ht_cmpkey_func_t) compare = 
		(PKG_NAME_(ht_cmpkey_func_t))s->compare;
	PKG_NAME_(htkey_t) key = htn_key(v);

	ctmpl_assert(s != NULL);

	lp = (PKG_NAME_(list))s;

	if(sl_isempty(s) ||
	  (*compare)(key, htn_key(sln_data(sl_headnode(s)))) < 0){
		iterator_prepend(lp, v);
		return NULL;
	}
	else if((*compare)(key, htn_key(sln_data(sl_tailnode(s)))) > 0){
		iterator_append(lp, v);
		return NULL;
	}

	left = 0;
	last = right = sl_nodecount(s) - 1;
	n1 = sl_headnode(s);
	pos = 0;
	
	while(left < right){
		mid = (left + right) / 2;
		n1 = PKG_NAME_(slistnode_go)(n1, mid - pos);
		pos = mid;

		cmp = (*compare)(htn_key(sln_data(n1)), key);
		if(cmp < 0)
			left = mid + 1;
		else 
			right = mid;
	}
	n1 = PKG_NAME_(slistnode_go)(n1, left - pos);
	cmp = (*compare)(htn_key(sln_data(n1)), key);

	if(!cmp){/* same data entry */
			ret = sln_data(n1);
			sln_data(n1) = v;
	}
	else {
		if(cmp > 0)
			n1 = sln_prev(n1);
		PKG_NAME_(listnode_insert)(lp, n1, v);
	}
	return ret;
}

static CTMPL_INLINE PKG_NAME_(slist)
PKG_NAME_(ht_find_first_slist)(PKG_NAME_(ht) htp)
{
	int i;
	if(ht_count(htp) == 0)
		return NULL;

	for(i = 0; i < ht_size(htp); i++)
		if(htp->table[i] && sl_nodecount(htp->table[i]))
			return htp->table[i];

	ctmpl_assert(!"can't reach here!!");
	return NULL;
}

static CTMPL_INLINE PKG_NAME_(slist)
PKG_NAME_(ht_find_last_slist)(PKG_NAME_(ht) htp)
{
	int i;
	if(ht_count(htp) == 0)
		return NULL;

	for(i = ht_size(htp) - 1; i >= 0; --i)
		if(htp->table[i] && sl_nodecount(htp->table[i]))
			return htp->table[i];

	ctmpl_assert(!"can't reach here!!");
	return NULL;
}

static CTMPL_INLINE PKG_NAME_(slistnode)
PKG_NAME_(ht_find_next)(PKG_NAME_(ht) htp, PKG_NAME_(slistnode) htnp)
{
	int i = 0;
	/* no more */
	if(htnp == NULL)
		return NULL;

	if(sln_next(htnp))
		return sln_next(htnp);

	/* find next slist containning data */
	i = ht_hash(htp, htn_key(sln_data(htnp)));
	for(++i ; i < ht_size(htp); ++i){
		if(htp->table[i] && sl_nodecount(htp->table[i]))
			return sl_headnode(htp->table[i]);
	}
	return NULL; /* no more node */
}

static CTMPL_INLINE PKG_NAME_(slistnode)
PKG_NAME_(ht_find_prev)(PKG_NAME_(ht) htp, PKG_NAME_(slistnode) htnp)
{
	int i = 0;

	/* no more */
	if(htnp == NULL)
		return NULL;

	if(sln_prev(htnp))
		return sln_prev(htnp);

	/* find prev slist containning data */
	i = ht_hash(htp, htn_key(sln_data(htnp)));
	for(--i ; i >= 0; --i){
		if(htp->table[i] && sl_nodecount(htp->table[i]))
			return sl_tailnode(htp->table[i]);
	}
	return NULL; /* no more node */
}

static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_first)(PKG_NAME_(ht) htp)
{
	PKG_NAME_(slist) sl = NULL;
	PKG_NAME_(ht_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = htp;

	sl = PKG_NAME_(ht_find_first_slist)(htp);
	if(!sl)
		iterator_item(ret) = NULL;
	else
		iterator_item(ret) = sl_headnode(sl);
	return ret;
}


static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_last)(PKG_NAME_(ht) htp)
{
	PKG_NAME_(slist) sl = NULL;
	PKG_NAME_(ht_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = htp;

	sl = PKG_NAME_(ht_find_last_slist)(htp);
	if(!sl)
		iterator_item(ret) = NULL;
	else
		iterator_item(ret) = sl_tailnode(sl);
	return ret;
}

static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_at)(PKG_NAME_(ht) htp, iterator_pos_t idx) 
{
	int prev = 0;
	PKG_NAME_(slist) sl = NULL;
	PKG_NAME_(slistnode) node = NULL;
	PKG_NAME_(ht_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = htp;

	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	sl = (prev? PKG_NAME_(ht_find_last_slist)(htp):
		    PKG_NAME_(ht_find_first_slist)(htp));

	if(!sl)
		iterator_item(ret) = NULL;
	else {
		node = (prev? sl_tailnode(sl): sl_headnode(sl));
		while(idx){
			node = (prev? PKG_NAME_(ht_find_prev)(htp, node):
				      PKG_NAME_(ht_find_next)(htp, node));
			--idx;
		}
		iterator_item(ret) = node;
	}
	return ret;
}

static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_next)(PKG_NAME_(ht_iterator) itr)
{
	PKG_NAME_(ht) ht = iterator_aggregate(itr);
	PKG_NAME_(ht_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = 
		PKG_NAME_(ht_find_next)(iterator_aggregate(ret),
					iterator_item(itr));
	if(iterator_item(ret) && iterator_subtype(ret) && 
	   (*ht->cmpkey)(htn_key(sln_data(iterator_item(itr))), 
			htn_key(sln_data(iterator_item(ret)))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}

static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_prev)(PKG_NAME_(ht_iterator) itr)
{
	PKG_NAME_(ht) ht = iterator_aggregate(itr);
	PKG_NAME_(ht_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = 
		PKG_NAME_(ht_find_prev)(iterator_aggregate(ret),
					iterator_item(itr));
	if(iterator_item(ret) && iterator_subtype(ret) && 
	   (*ht->cmpkey)(htn_key(sln_data(iterator_item(itr))), 
			htn_key(sln_data(iterator_item(ret)))) != 0)
		iterator_item(ret) = NULL;
	return ret;
}

static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_erase_gonext)(PKG_NAME_(ht_iterator) itr)
{
	PKG_NAME_(ht_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	ret = iterator_next(itr);
	PKG_NAME_(htnode_remove)(iterator_aggregate(ret),
			         ht_entry(iterator_aggregate(ret),
				 htn_key(sln_data(iterator_item(itr)))),
			         iterator_item(itr));
	return ret;
}

static PKG_NAME_(ht_iterator) 
PKG_NAME_(ht_itr_erase_goprev)(PKG_NAME_(ht_iterator) itr)
{
	PKG_NAME_(ht_iterator) ret;
	if(iterator_is_end(itr))
		return itr;

	ret = iterator_prev(itr);
	PKG_NAME_(htnode_remove)(iterator_aggregate(ret),
			         ht_entry(iterator_aggregate(ret),
				 htn_key(sln_data(iterator_item(itr)))),
			         iterator_item(itr));
	return ret;
}

static int 
PKG_NAME_(ht_itr_is_equal)(PKG_NAME_(ht_iterator) x,
                           PKG_NAME_(ht_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }

static PKG_NAME_(ht_iterator_data)
PKG_NAME_(ht_itr_retrieve)(PKG_NAME_(ht_iterator) itr)
{
	return (PKG_NAME_(ht_iterator_data))sln_data(iterator_item(itr));
}

static PKG_NAME_(ht_ctl) PKG_NAME_(ht_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(ht_delete_by),
	PKG_NAME_(ht_clear_by),
	PKG_NAME_(ht_dup_by),
	PKG_NAME_(ht_foreach),
	PKG_NAME_(ht_count),
	
	PKG_NAME_(ht_itr_first),
	PKG_NAME_(ht_itr_last),
	PKG_NAME_(ht_itr_at),
	PKG_NAME_(ht_itr_next),
	PKG_NAME_(ht_itr_prev),
	PKG_NAME_(ht_itr_erase_gonext),
	PKG_NAME_(ht_itr_erase_goprev),
	PKG_NAME_(ht_itr_is_equal),
	PKG_NAME_(ht_itr_retrieve),
	PKG_NAME_(ht_get),
	PKG_NAME_(ht_put)
};

static CTMPL_INLINE void 
PKG_NAME_(ht_set_ctl)(PKG_NAME_(ht) htp)
{
	htp->ctl = &PKG_NAME_(ht_ctl_sw);
}

