/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/**
 * 	ctmpl/memory.h
 * 	$Id: memory.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 */		

#ifndef _CTMPL_MEMORY_H_
#define _CTMPL_MEMORY_H_

/*
 *      m_alloc_func_t & m_free_func_t
 *              - these functions are used to allocate & free 
 *              memory in templates.
 *
 *      m_alloc_func_t used to allocate memory for all spaces which
 *	are used in templates and has to act following,
 *              - m_alloc_func_t has to return allocated memory 
 *              initialized by all zero.  
 *              - If failed, allocator has to abort.
 *			     ^^^^^^^^^^^^^^^^^^^^^^^
 */
typedef void *(*m_alloc_func_t)(const char * varname,
				size_t       msize,
                                const char * filename,
				int          line);
 
typedef void (*m_free_func_t)(const char * varname,
			      void *       mptr,
			      const char * filename,
			      int          line);


#define CTMPL_MALLOC(f, y, z)	(*f)((y), (z), __FILE__, __LINE__)
#define CTMPL_FREE(f, y, z)	(*f)((y), (z), __FILE__, __LINE__)

#endif /*_CTMPL_MEMORY_H_ */
