/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 *	ctmpl/slist.h
 *	$Id: slist.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *
 *	<Sorted Doubly Linked List Template>
 */

#include "ctmpl/ctmpl.h"

#define PKG_SEQVAL_T         PKG_SETVAL_T
#include "ctmpl/list.h"
#undef  PKG_SEQVAL_T          

typedef struct PKG_NAME_(slist) *PKG_NAME_(slist);
typedef PKG_NAME_(listnode) 	 PKG_NAME_(slistnode);
typedef PKG_NAME_(listdata_t) 	 PKG_NAME_(slistval_t);
typedef struct PKG_NAME_(slist_iterator) PKG_NAME_(slist_iterator);
typedef struct PKG_NAME_(slist_ctl) PKG_NAME_(slist_ctl);

/* prototypes */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(slist) 
PKG_NAME_(slist_new)(PKG_NAME_(listdata_compare_func_t));
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(slist) 
PKG_NAME_(slist_new_by)(PKG_NAME_(listdata_compare_func_t),
			m_alloc_func_t, 
			m_free_func_t);

struct PKG_NAME_(slist) {
	struct PKG_NAME_(list) _list;
	PKG_NAME_(listdata_compare_func_t) compare;
	PKG_NAME_(slist_ctl) *ctl;
};

/* iterator for slist */
struct PKG_NAME_(slist_iterator){
	short		   	iter_type;     /* iterator type */
	short		   	iter_subtype;  /* iterator subtype */
	PKG_NAME_(slist)	iter_aggr;     /* iterator aggregate */
	PKG_NAME_(slistnode)    iter_item;     /* iterator item */
};

/* iterator controller for list */
struct PKG_NAME_(slist_ctl){
	PKG_NAME_(slist_iterator) enditr;
	void		     	  (*delete_by)(PKG_NAME_(slist), 
				   PKG_NAME_(listdata_access_func_t));
	void		     	  (*clear_by)(PKG_NAME_(slist), 
				   PKG_NAME_(listdata_access_func_t));
	PKG_NAME_(slist)	  (*dup_by)(PKG_NAME_(slist),
				   PKG_NAME_(listdata_duplicate_func_t));
	void		     	  (*foreach)(PKG_NAME_(slist), 
					    PKG_NAME_(listdata_foreach_func_t),
					    void *);

	iterator_cnt_t	   	  (*count)(PKG_NAME_(slist));

	PKG_NAME_(slist_iterator) (*first)(PKG_NAME_(slist));
	PKG_NAME_(slist_iterator) (*last)(PKG_NAME_(slist));
	PKG_NAME_(slist_iterator) (*at)(PKG_NAME_(slist), iterator_pos_t);
	PKG_NAME_(slist_iterator) (*next)(PKG_NAME_(slist_iterator));
	PKG_NAME_(slist_iterator) (*prev)(PKG_NAME_(slist_iterator));
	PKG_NAME_(slist_iterator) (*erase_gonext)(PKG_NAME_(slist_iterator));
	PKG_NAME_(slist_iterator) (*erase_goprev)(PKG_NAME_(slist_iterator));
	int			  (*is_equal)(PKG_NAME_(slist_iterator),
					      PKG_NAME_(slist_iterator));
	PKG_NAME_(slistval_t)    (*retrieve)(PKG_NAME_(slist_iterator));

	PKG_NAME_(slist_iterator) (*find)(PKG_NAME_(slist), 
					  PKG_NAME_(slistval_t));
	void			  (*insert)(PKG_NAME_(slist), 
					    PKG_NAME_(slistval_t));
};

