/*
 *			ndtpserver.c
 * $Id: ndtpserver.c,v 1.15 2004/02/11 20:59:26 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "memory.h"
#include "debug.h"
#include "tcp.h"
#include "dictbook.h"
#include "ndtpserver.h"
#include "ndtpbook.h"
#include "subr.h"


static inline int srvread(struct ndtpserver *s, char *b, int l){
	int ret = tc_readLine(&((s)->ns_session), b, l);
	if(DEBUG_IS_ENABLED(DEBUG_NDTP))		
		printf("(ndtp)%d<-%s", ret, b);	
	return ret;
}
static inline int srvwrite(struct ndtpserver *s, const char *b, int l){
	int ret = tc_writeLine(&((s)->ns_session), b, l);
	if(DEBUG_IS_ENABLED(DEBUG_NDTP))		
		printf("(ndtp)%d->%s", ret, b);	
	return ret;
}

#define badproto(x)						\
	do { 							\
		fprintf(stderr, "Bad Protocol Found\n");	\
		fprintf(stderr, "\t - at %s:%d\n", __FILE__, __LINE__); \
		fprintf(stderr, "\t - received \"%s\"\n", x); 	\
		abort();					\
	} while(0)

#define return_syserr(x, y)	do {  				\
			ndtp_errno = -1; 			\
			ndtpserver_check_err(x);		\
			return y;				\
		} while(0)

#define return_result(x)					\
	do { 							\
		if((x)[0] != '$')				\
			badproto(buf);				\
		if((x)[1] == '*'){				\
			ndtp_errno = 0;				\
			return NDTP_SUCCESS; /* success */	\
		}						\
		ndtp_errno = (x)[1];				\
		return NDTP_FAIL;				\
	} while(0); return 0 

/* error number for NDTP */
int ndtp_errno;

static void
ndtpserver_check_err(struct ndtpserver *sp)
{
	booklist_iterator itr = iterator_first(sp->ns_books);

	if(tc_is_connected(&sp->ns_session))
		return; /* still connected */

	/* connection is closed */
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct dictbook *dp = iterator_data(itr);
		dp->db_flags |= DBFLAG_NOTAVAIL;
	}
}

struct ndtpserver *
ndtpserver_alloc(const char *svrp)
{
	int   port;
	char  buf[1024];
	struct ndtpserver *ret;
	const char *p = index(svrp, ':');
	
	/* check */
	if(p){
		port = atoi(p + 1);
		if(port <=0 || port > 65535)
			return NULL;
	}

	strncpy(buf, svrp, 1024);

	if(!p)
		strcat(buf, ":2010");

	ret = MALLOC("ndtpserver", sizeof(struct ndtpserver));
	ret->ns_id = (const char *)xp_strdup(buf);
	ret->ns_books = booklist_new(8, 4, xpdic_malloc, xpdic_free);
	ret->ns_nopen = 0; 
	return ret;
}

int  
ndtpserver_connect(struct ndtpserver *srvp)
{
	char addr[1024];
	char *p;
	int port;

	if(DEBUG_IS_ENABLED(DEBUG_NDTP))
		printf("(ndtp-%s) connect(nref=%d)\n", srvp->ns_id, srvp->ns_nopen);

	if(srvp->ns_nopen > 0){
		++srvp->ns_nopen; 
		return 1;
	}
	srvp->ns_nopen = 1;

	strncpy(addr, srvp->ns_id, 1024);
	p = index(addr, ':');
	port = atoi(p + 1);
	*p = '\0';
	
	return tc_connect(&srvp->ns_session, addr, (short)port);
}

void
ndtpserver_disconnect(struct ndtpserver *srvp)
{
	--srvp->ns_nopen;

	if(DEBUG_IS_ENABLED(DEBUG_NDTP))
		printf("(ndtp-%s) disconnect(nref=%d)\n", srvp->ns_id, srvp->ns_nopen);

	if(srvp->ns_nopen == 0)
		tc_disconnect(&srvp->ns_session);
}

int  
ndtpserver_load(struct ndtpserver *srvp)
{
	int 			nread;
	char 			buf[1024];
	const char		*strp;
	strlist 		slst;
	strlist_iterator 	itr;
	int			idx, n1,n2,n3;
	char 			name[256], id[256];
	struct ndtpbook 	*bp;

	if(srvwrite(srvp, "t\n", 2) < 0)
		return 0;

	slst = strlist_new(4, 4, xpdic_malloc, xpdic_free);
	while(1){
		if((nread = srvread(srvp, buf, 1024)) <= 0){
			strlist_delete(slst);
			return NULL;
		}

		if(buf[0] == '$')
			break;
		if(buf[strlen(buf)-1] == '\n')
			buf[strlen(buf)-1] = '\0';
		iterator_append(slst, xp_strdup(buf));
	}

	itr = iterator_first(slst);	
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		strp = iterator_data(itr);
		sscanf(strp, "%d\t%s\t%s\t%d\t%d\%d", &idx, name, id, &n1, &n2, &n3);
		if((bp = ndtpbook_new(srvp, name, idx, id)) != NULL)
			iterator_append(srvp->ns_books, (struct dictbook *)bp);
	}
	
	strlist_delete(slst);
	return NDTP_SUCCESS;
}

void 
ndtpserver_free(struct ndtpserver *srvp)
{
	aggregator_delete_by(srvp->ns_books, ndtpbook_free);
	if(tc_is_connected(&srvp->ns_session))
		tc_disconnect(&srvp->ns_session);
	xp_strfree(srvp->ns_id);   /* allocated by xp_strdup */
	FREE("ndtpserver", srvp);
}

const char *
ndtp_strerr(int i)
{
	extern int errno;
	if(i < 0)
		return strerror(errno);
	switch(i){
	case '?':
		return "Invalid argument";
	case '<':
		return "Dictionary is not set.";
	case '&':
		return "Dictionary is not available";
	case 'N':
		return "Permission denied";
	}
	return "Unknown Error";
}


strlist ndtp_get_booklist(struct ndtpserver *);
int 	
ndtp_setbook_idx(struct ndtpserver *srvp, int idx)
{
	int  nread;
	char buf[8];

	snprintf(buf, 8, "L%d\n", idx);
	if(srvwrite(srvp, buf, strlen(buf)) < 0)
		return_syserr(srvp, NDTP_FAIL);
	if((nread = srvread(srvp, buf, 8)) <= 0)
		return_syserr(srvp, NDTP_FAIL);
	return_result(buf);
}

int 	
ndtp_setbook_name(struct ndtpserver *srvp, const char *name)
{
	int  nread;
	char buf[128];

	snprintf(buf, 128, "L%s\n", name);
	if(srvwrite(srvp, buf, strlen(buf)) < 0)
		return_syserr(srvp, NDTP_FAIL);
	if((nread = srvread(srvp, buf, 128)) <= 0)
		return_syserr(srvp, NDTP_FAIL);
	return_result(buf);
}

int 	
ndtp_get_fontsize(struct ndtpserver *srvp, char *lst, int nlst)
{
	int  n = 0, nread;
	char buf[8];

	if(srvwrite(srvp, "XI\n", 3) < 0)
		return_syserr(srvp, NDTP_FAIL);
	if((nread = srvread(srvp, buf, 8)) <= 0)
		return_syserr(srvp, NDTP_FAIL);
	if(strncmp(buf, "$I", 2) != 0)
		badproto(buf);
	while(1){
		if((nread = srvread(srvp, buf, 8)) <= 0)
			return_syserr(srvp, NDTP_FAIL);
		buf[strlen(buf) - 1] = '\0';

		if(buf[0] == '$')	/* end of font list */
			break;
		if(lst && n < nlst)
			lst[n++] = (char)atoi(buf);
	}
		
	if(buf[1] == '*')
		return n;	/* return number of font size */
	return buf[1];
}

int 	
ndtp_set_fontsize(struct ndtpserver *srvp, int size)
{
	int  nread;
	char buf[8];

	snprintf(buf, 8, "XL%d\n", size);
	if(srvwrite(srvp, buf, strlen(buf)) < 0)
		return_syserr(srvp, NDTP_FAIL);
	if((nread = srvread(srvp, buf, 8)) <= 0)
		return_syserr(srvp, NDTP_FAIL);
	return_result(buf);
}

strlist 
ndtp_lookup(struct ndtpserver *srvp, const char *word)
{
	int nread;
	char buf[1024];
	strlist slst;

	snprintf(buf, 1024, "Pa%s\n", word);
	if(srvwrite(srvp, buf, strlen(buf)) < 0)
		return_syserr(srvp, NDTP_FAIL);

	if((nread = srvread(srvp, buf, 8)) <= 0)
		return_syserr(srvp, NDTP_FAIL);

	if(buf[0] != '$')
		badproto(buf);
	if(buf[1] != '0'){
		ndtp_errno = 0;
		return NULL; 	/* no such entry */
	}

	slst = strlist_new(8, 4, xpdic_malloc, xpdic_free);

	while(1){
		if((nread = srvread(srvp, buf, 1024)) <= 0){
			strlist_delete(slst);
			return NULL;
		}
		if(buf[0] == '$')	/* end of font list */
			break;
		if(buf[strlen(buf)-1] == '\n')
			buf[strlen(buf)-1] = '\0';
		iterator_append(slst, xp_strdup(buf));
	}
	if(buf[1] == '$'){
		if(iterator_count(slst) == 0){
			strlist_delete(slst);
			ndtp_errno = 0;
			return NULL;  /* no such entry */
		}
		return slst;
	}

	ndtp_errno = buf[1];
	strlist_delete(slst);
	return NULL;
}

strlist 
ndtp_getentry(struct ndtpserver *srvp, const char *ptr)
{
#define ENTRY_BUFSIZE	4096
	int nread;
	char buf[ENTRY_BUFSIZE];/* don't know what happens if greater thing is reads */
	strlist slst;

	snprintf(buf, 1024, "S%s\n", ptr);
	if(srvwrite(srvp, buf, strlen(buf)) < 0)
		return_syserr(srvp, NULL);

	if((nread = srvread(srvp, buf, 8)) <= 0)
		return_syserr(srvp, NULL);

	if(buf[0] != '$')
		badproto(buf);
	if(buf[1] != '1'){
		ndtp_errno = 0;
		return NULL; 	/* no such entry */
	}

	slst = strlist_new(8, 4, xpdic_malloc, xpdic_free);

	while(1){
		int slen;
		if((nread = srvread(srvp, buf, ENTRY_BUFSIZE)) <= 0){
			strlist_delete(slst);
			return_syserr(srvp, NULL);
		}
		if(buf[0] == '$')	/* end of font list */
			break;
		slen = strlen(buf);

		if(buf[slen-1] == '\n')
			buf[slen-1] = '\0';
		iterator_append(slst, xp_strdup(buf));
	}
	if(buf[1] == '$'){
		if(iterator_count(slst) == 0){
			strlist_delete(slst);
			ndtp_errno = 0;
			return NULL;  /* no such entry */
		}
		return slst;
	}

	ndtp_errno = buf[1];
	strlist_delete(slst);
	return NULL;
}

strlist 
ndtp_getfont(struct ndtpserver *srvp, const char *ptr)
{
	int nread;
	char buf[1024];
	strlist slst;

	snprintf(buf, 1024, "Xb%s\n", ptr);
	if(srvwrite(srvp, buf, strlen(buf)) < 0)
		return_syserr(srvp, NULL);

	if((nread = srvread(srvp, buf, 8)) <= 0)
		return_syserr(srvp, NULL);

	if(buf[0] != '$')
		badproto(buf);
	if(buf[1] != 'I'){
		ndtp_errno = 0;
		return NULL;
	}

	slst = strlist_new(8, 4, xpdic_malloc, xpdic_free);

	while(1){
		if((nread = srvread(srvp, buf, 1024)) <= 0){
			strlist_delete(slst);
			return_syserr(srvp, NULL);
		}
		if(buf[0] == '$')	/* end of font list */
			break;
		if(buf[strlen(buf)-1] == '\n')
			buf[strlen(buf)-1] = '\0';
		iterator_append(slst, xp_strdup(buf));
	}
	if(buf[1] == '$'){
		if(iterator_count(slst) == 0){
			strlist_delete(slst);
			ndtp_errno = 0;
			return NULL;  /* no such entry */
		}
		return slst;
	}

	ndtp_errno = buf[1];
	strlist_delete(slst);
	return NULL;
}

