/*
 *			ndtpserver.h
 * $Id: ndtpserver.h,v 1.3 2004/01/11 06:27:16 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _XP_NDTP_SERVER_H_
#define _XP_NDTP_SERVER_H_

#include "strlist.h"
#include "booklist.h"
struct tcpcon;

/* ndtp server structure */
struct ndtpserver {
	const char 	*ns_id;		/* server:port */
	struct tcpcon	ns_session;	/* tcp session to this server */
	int		ns_nopen;	/* # of opened books */
	booklist	ns_books;	/* books cotained by this server */
};

#define NDTP_FAIL	0
#define NDTP_SUCCESS	1

/* these interfaces for ndtpbooks */
struct ndtpserver *ndtpserver_alloc(const char *);
int  		   ndtpserver_connect(struct ndtpserver *);
void  		   ndtpserver_disconnect(struct ndtpserver *);
int  		   ndtpserver_load(struct ndtpserver *);  
void 		   ndtpserver_free(struct ndtpserver *);  

/* these interfaces for ndtpbooks */
const char 	  *ndtp_strerr(int);
strlist		   ndtp_get_booklist(struct ndtpserver *);
int 		   ndtp_setbook_idx(struct ndtpserver *, int idx);
int 		   ndtp_setbook_name(struct ndtpserver *, const char *name);
int 		   ndtp_get_fontsize(struct ndtpserver *, char *lst, int nlst);
int 		   ndtp_set_fontsize(struct ndtpserver *, int size);
strlist 	   ndtp_lookup(struct ndtpserver *, const char *word);
strlist 	   ndtp_getentry(struct ndtpserver *, const char *ptr);
strlist 	   ndtp_getfont(struct ndtpserver *, const char *ptr);

extern int	   ndtp_errno;		/* error number for ndtp */
#define NDTP_NOERR	0		/* "No Error" */
#define NDTP_EINVAL	'?'		/* "Invalid argument" */
#define NDTP_ENOTSET	'<'		/* "Dictionary is not set." */
#define NDTP_ENOTAVAIL	'&'		/* "Dictionary is not available" */
#define NDTP_EACCESS	'N'		/* "Permission denied" */

#endif /* _XP_NDTP_SERVER_H_ */
