#include <stdio.h>
#include <unistd.h>
#include "bitmap.h"
#include "xwindow.h"
#include "memory.h"
#include "stk/stkbase.h"

/*
 *	stk library sample program1
 *
 *	top1 
 *	 |
 *    frame(401, 301) 
 *	 |
 *    vlayout1
 *	 |
 *	 +-- hlayout1 
 *	 |      +- string  
 *	 |
 * 	 +-- hlayout2
 *	 |      +- string  
 *	....    
 * 	 +-- hlayout n
 *	        +- string  
 *
 *	top2 
 *	 |
 *    frame2(401, 301) 
 *	 |
 *    vlayout2	
 *	 |
 *     hlayout21 - xascroll1 vline xascroll2
 *	 |
 *	hline
 *	 |
 *     hlayout22 - xascroll3 vline xascroll4
 *
 *    xascroll[1-4] - scroll[1-4] - vlayout1
 *
 */
static xobject *cur;
static xobject *top1;
static xobject *top2;
xattr	*xatblue;
xattr	*xatgreen;
xattr	*xatred;
xattr	*xatblack;
xattr	*xatwhite;

void action_view_change(struct xactor *dst, xpoint dstpos, 
	    		xsignal sig, struct xsigarg *siginfo, void *actarg,
	    		struct xobject *top, xwindow *xwp)
{
	if(xsignal_type(sig) == XSIG_BTNRELEASE){
		stk_set(xwp, ((cur == top1)? top2 : top1));
		stk_draw();
		cur = ((cur == top1)? top2 : top1);
	}
}

void action_end(struct xactor *dst, xpoint dstpos, 
	    	xsignal sig, struct xsigarg *siginfo, void *actarg,
	    	struct xobject *top, xwindow *xwp)
{
	if(xsignal_type(sig) == XSIG_BTNRELEASE)
		stk_stop_loop();
}


void action_key(struct xactor *dst, xpoint dstpos, 
	    	xsignal sig, struct xsigarg *siginfo, void *actarg,
	    	struct xobject *top, xwindow *xwp)
{
	int	newsig = 0;
	int	flags = 0, nx, ny;
	struct xs_keyinfo *kip = (struct xs_keyinfo *)siginfo;
	xobject *o;
	xpoint pos = {0, 0};

	if(xsignal_type(sig) != XSIG_KEYRELEASE)
		return;

	if(!strcmp(kip->key, "Return")){
		stk_set(xwp, ((cur == top1)? top2 : top1));
		stk_draw();
		cur = ((cur == top1)? top2 : top1);
		return;
	}

	if(cur == top1)
		return;

	switch(*kip->key){
	case 'u':
		newsig = XSIG_SCRL_PAGEUP;
		break;
	case 'd':
		newsig = XSIG_SCRL_PAGEDOWN;
		break;
	case 'l':
		newsig = XSIG_SCRL_PAGELEFT;
		break;
	case 'r':
		newsig = XSIG_SCRL_PAGERIGHT;
		break;
	case 'R':
		newsig = XSIG_SCRL_RESET;
		break;
	case 'q':
		stk_stop_loop();
		return;
	}

	if(newsig == 0)
		return;

	/* find object able to receive the signal in the dst actor */
	nx = siginfo->x - dstpos.x;
	ny = siginfo->y - dstpos.y;
	flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ANY, newsig);	

	o = xcmp_find(dst, nx, ny, newsig, &pos);

	if(o && xobj_is_actor(o)){
		xactor  *a = (xactor *)o;
		pos.x += dstpos.x;
		pos.y += dstpos.y;
		xact_send_signal(a, pos, newsig, siginfo, top, xwp);
	}
}

void action_scroll(struct xactor *dst, xpoint dstpos, 
	    	   xsignal sig, struct xsigarg *siginfo, void *actarg,
	    	   struct xobject *top, xwindow *xwp)
{
	int i;
	struct xs_btninfo *btp = (struct xs_btninfo *)siginfo;
	struct xs_bmotion *bmp = (struct xs_bmotion *)siginfo;
	xposmark **pp = (xposmark **)actarg;
	struct xsigarg sa = {btp->pos.x, btp->pos.y};
	struct xsigarg *nsap = &sa;

	/* not redirect redirected signal */
	if(xsignal_is_redirect(sig))
		return;
	
	/* set redirect flag */
	xsignal_set_redirect(sig);

	if(xsignal_type(sig) == XSIG_BTNPRESS ||
	   xsignal_type(sig) == XSIG_BTNRELEASE)
		return; 	/* not redirect */

	else if(xsignal_type(sig) == XSIG_GRABMOTION){
		int n = 0, i = 0;

		for(i = 0; i < 4; i++){
			struct xs_bmotion nmt = {{bmp->pos.x, bmp->pos.y}, 
						 bmp->button, 0, 0};
			int nextx[] = { -bmp->mx,  bmp->mx, -bmp->mx };
			int nexty[] = { -bmp->my,  -bmp->my, bmp->my };

			nsap = (struct xsigarg *)&nmt;
			if(xpm_obj(pp[i]) == (xobject *)dst)
				continue;
			
			/* prepare signal argument for this actor */
			nmt.mx = nextx[n];
			nmt.my = nexty[n];
			++n;

			nsap = (struct xsigarg *)&nmt;
			xact_send_signal((struct xactor *)xpm_obj(pp[i]), 	
					 xpm_pos(pp[i]), sig, nsap, top, xwp);
		}
	}
	else {
		xsignal_set_redirect(sig);
		for(i = 0; i < 4; i++){
			if(xpm_obj(pp[i]) == (struct xobject *)dst)
				continue;
			
			xact_send_signal((struct xactor *)xpm_obj(pp[i]), 	
				 xpm_pos(pp[i]), sig, nsap, top, xwp);
		}
	}

	if(xsignal_type(sig) == XSIG_SCRL_RESET){
#define get_scrl(x)	((xactor *)xpm_obj(x))->xa_object
		xsc_move(get_scrl(pp[1]), 201, 0, 200, 0);	
		xsc_move(get_scrl(pp[2]), 0, 151, 0, 150);	
		xsc_move(get_scrl(pp[3]), 201, 151, 200, 150);	
		xobj_redraw_object(cur, cur, 0, 0, xwp, 0);
	}
}

#define mkstr(c, a)	xstring_new(c, strlen(c), a, 0)

xobject *
construct_vlayout1(xrootwindow *rwp, xwindow *xwp)
{
	xobject *ret = xvlayout_new(401, 301, 0, XCRULE_TOP);
	xobject *strblue, *strgreen, *strred;
	xactor *clickable1, *clickable2;
	xobject *expander = xexpander_new();
	xobject *hlp;

	/* create basis of clickable string */
	strblue = xstring_new("here", 4, xatblue, 0);
	strgreen = xstring_new("here", 4, xatgreen, 0);
	strred = xstring_new("here", 4, xatred, 0);

	/* create clickable1 */
	clickable1 = xclickable_new(strgreen);
	xact_register(clickable1, XSIG_ENTER, action_view_change, NULL,
		xcmpstpair_new(strblue, underline_new(strblue, xatblue), 0));
	xact_register(clickable1, XSIG_LEAVE, action_view_change, NULL, strgreen);
	xact_register(clickable1, XSIG_BTNPRESS, action_view_change, NULL, strred);
	xact_register(clickable1, XSIG_BTNRELEASE, action_view_change, NULL,
		xcmpstpair_new(strgreen, underline_new(strgreen, xatblue), 0));

	/* create clickabel2 */
	clickable2 = xclickable_new(strgreen);
	xact_register(clickable2, XSIG_ENTER, action_end, NULL,
		xcmpstpair_new(strblue, underline_new(strblue, xatblue), 0));
	xact_register(clickable2, XSIG_LEAVE, action_end, NULL, strgreen);
	xact_register(clickable2, XSIG_BTNPRESS, action_end, NULL, strred);
	xact_register(clickable2, XSIG_BTNRELEASE, action_end, NULL,
		xcmpstpair_new(strblue, underline_new(strblue, xatblue), 0));
	
	/* title of the page */
	xcont_add(ret, xspace_new(5, 10), XCRULE_CENTER);
	xcont_add(ret, mkstr("What is STK", xatblue), XCRULE_CENTER);
	xcont_add(ret, xspace_new(5, 5), XCRULE_CENTER);
	xcont_add(ret, xline_new(0, 0, 401, 0, xatblue), XCRULE_CENTER);
	xcont_add(ret, xspace_new(5, 5), XCRULE_CENTER);

	xcont_add(ret, mkstr(" STK is a simple toolkit that enables to create \"cheep\" GUI.", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 3), XCRULE_CENTER);
	xcont_add(ret, mkstr(" This simple toolkit is originally developed to make X Pop ", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 3), XCRULE_CENTER);
	
	xcont_add(ret, mkstr(" dictionary and it is easy to create some kinds of GUI programs ", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 3), XCRULE_CENTER);
	xcont_add(ret, mkstr(" such like postit.", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 3), XCRULE_CENTER);

	xcont_add(ret, xspace_new(5, 3), XCRULE_CENTER);
	xcont_add(ret, mkstr(" This sample program shows this brief with two views.", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 3), XCRULE_CENTER);

	xcont_add(ret, expander, XCRULE_CENTER);

	xcont_add(ret, mkstr("just enter",  xatblue), XCRULE_CENTER);
	xcont_add(ret, mkstr("or",  xatblue), XCRULE_CENTER);
	xcont_add(ret, mkstr(" if you click following string, the view will be changed...", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 5), XCRULE_CENTER);

	hlp = xhlayout_new(401, -1, 0, XCRULE_CENTER);
	xcont_add(hlp, mkstr("->>> ", xatblack), 0);
	xcont_add(hlp, (xobject *)clickable1, 0);
	xcont_add(hlp, mkstr(" <<<-", xatblack), 0);
	xcont_add(ret, hlp, XCRULE_CENTER);
	xcont_add(ret, expander, XCRULE_CENTER);

	xcont_add(ret, mkstr("  if you click following string, sample program will be terminated...", xatblue), XCRULE_LEFT);
	xcont_add(ret, xspace_new(5, 5), XCRULE_CENTER);

	hlp = xhlayout_new(401, -1, 0, XCRULE_CENTER);
	xcont_add(hlp, mkstr("->>> ", xatblack), 0);
	xcont_add(hlp, (xobject *)clickable2, 0);
	xcont_add(hlp, mkstr(" <<<-", xatblack), 0);
	xcont_add(ret, hlp, XCRULE_CENTER);
	xcont_add(ret, expander, XCRULE_CENTER);

	return ret;
}

int 
main(int argc, char **argv)
{
	int i;
	xrootwindow *rwp = xrootwindow_open(NULL);
	xwindow *xwp = xwindow_create(rwp, 10, 10, 401, 301, 0, 0xffffff, 0, 0, 1);

	xobject  *scrls[4];
	xposmark *poss[4];
	xobject  *keyact1;
	xobject  *skeyacts[4];

	xproxy *proxy1, *proxy2;
	xobject *frame1, *frame2;
	xobject *vlayout1;
	xobject *vlayout2 = xvlayout_new(401, 301, 0, XCRULE_CENTER);
	xobject *hlayout1 = xhlayout_new(401, -1, 0, XCRULE_CENTER);
	xobject *hlayout2 = xhlayout_new(401, -1, 0, XCRULE_CENTER);

	stk_initialize(rwp, NULL, NULL);

	xatblack = xattr_alloc(rwp);
	xatwhite = xattr_alloc(rwp);
	xatblue = xattr_alloc(rwp);
	xatgreen = xattr_alloc(rwp);
	xatred = xattr_alloc(rwp);

	/* prepair colors */
	xattr_set_fgcolor(xatblack, 0x000000);
	xattr_set_bgcolor(xatblack, 0xffffff);
	xattr_set_font(xatblack, "fixed");
	xattr_set_fgcolor(xatwhite, 0xffffff);
	xattr_set_bgcolor(xatwhite, 0x000000);
	xattr_set_font(xatwhite, "fixed");
	xattr_set_fgcolor(xatblue, 0x0000ff);
	xattr_set_font(xatblue, "fixed");
	xattr_set_fgcolor(xatgreen, 0x00ff00);
	xattr_set_font(xatgreen, "fixed");
	xattr_set_fgcolor(xatred, 0xff0000);
	xattr_set_font(xatred, "fixed");


	vlayout1 = construct_vlayout1(rwp, xwp);

	/* vew1: frame1 */
	keyact1 = (xobject *)xkeyaction_new(vlayout1);
	xact_register(keyact1, XSIG_KEYPRESS, action_key, NULL, NULL);
	xact_register(keyact1, XSIG_KEYRELEASE, action_key, NULL, NULL);
	frame1 = xframe_new(keyact1, 401, 301);
	proxy1 = xproxy_new(xcmpcast(frame1));
	xproxy_accept(proxy1, XSIG_ANY);
	top1 = (xobject *)proxy1;

	/* create for scroll view */
	for(i = 0; i < 4; i++){
		xactor *scrl;
		xactor *keyact;
		scrls[i] = xscroll_new(vlayout1, 200, 150);
		scrl = xascroll_new((xscroll *)scrls[i]);
		poss[i] = (xposmark *)xposmark_new((xobject *)scrl);
			
		xact_register(scrl, XSIG_ENTER, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_LEAVE, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_BTNPRESS, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_BTNRELEASE, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_GRABMOTION, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_SCRL_RESET, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_SCRL_PAGEUP, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_SCRL_PAGEDOWN, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_SCRL_PAGELEFT, action_scroll, poss, NULL);
		xact_register(scrl, XSIG_SCRL_PAGERIGHT, action_scroll, poss, NULL);

		keyact = xkeyaction_new((xobject *)poss[i]); 
		xact_register(keyact, XSIG_KEYPRESS, action_key, NULL, NULL);
		xact_register(keyact, XSIG_KEYRELEASE, action_key, NULL, NULL);

		skeyacts[i] = (xobject *)keyact;
	}

	xcont_add(hlayout1, (xobject *)skeyacts[0], XCRULE_CENTER);
	xcont_add(hlayout1, xline_new(0, 0, 0, xobj_height(skeyacts[1]),xatblack), 0);
	xcont_add(hlayout1, (xobject *)skeyacts[1], XCRULE_CENTER);
	xcont_add(hlayout2, (xobject *)skeyacts[2], XCRULE_CENTER);
	xcont_add(hlayout2, xline_new(0, 0, 0, xobj_height(skeyacts[2]),xatblack), 0);
	xcont_add(hlayout2, (xobject *)skeyacts[3], XCRULE_CENTER);
	xcont_add(vlayout2, hlayout1, 0);
	xcont_add(vlayout2, xline_new(0, 0, 401, 0, xatblack), 0);
	xcont_add(vlayout2, hlayout2, 0);

	/* create frame */
	frame2 = xframe_new(vlayout2, 401, 301);
	proxy2 = xproxy_new(xcmpcast(frame2));
	xproxy_accept(proxy2, XSIG_ANY);
	top2 = (xobject *)proxy2;

	if(argc > 1){
		xobj_dump(top1, stdout, 0);
		xobj_dump(top2, stdout, 0);
	}

	/* compose top2 to show */
	xcmp_compose(top2);
	xsc_move(scrls[1], 201, 0, 200, 0);	
	xsc_move(scrls[2], 0, 151, 0, 150);	
	xsc_move(scrls[3], 201, 151, 200, 150);	
	xcmp_relocate(top2, 0, 0);

	/* stk_set will compose/relocate top1 */
	stk_set(xwp, top1);
	cur = top1;
	stk_map();
	stk_draw();

	stk_start_loop(NULL, NULL);

	stk_unmap();
	stk_finalize(rwp);

	xobj_free(top1);
	xobj_free(top2);
	xattr_free(xatblack);
	xattr_free(xatblue);
	xattr_free(xatred);
	xattr_free(xatgreen);

	xwindow_destroy(xwp);
	xrootwindow_close(rwp);

	stk_memdump();
	return 0;
}

