/*
 *			xactor.h
 * $Id: xactor.h,v 1.4 2004/01/19 15:49:36 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _XP_XREACTOR_H_
#define _XP_XREACTOR_H_
struct xactor;

/* 
 *	function type to act for the signal 
 *		- dst: a target object
 *		- dstos: a position of the target object
 *		- sig: signal
 *		- siginfo: signal information
 *		- actarg: optional argument for this action function.
 *		- top: root object of this object tree.
 *		- xwp: xwindow
 */
typedef void (*xaction_t)(struct xactor *dst, xpoint dstpos, 
			  xsignal sig, struct xsigarg *siginfo, void *actarg,
			  struct xobject *top, xwindow *xwp);
/*
 *		X Action object 
 */
/* action structure */
typedef struct xaction {
	/* object drawn at the siganl: NULL means no drawing new object */
	xobject		*xac_object;	
	/* action to take after drawing new object(xrc_object) */
	xaction_t	xac_action;	
	/* versatile poiner for user for each action */
	void		*xac_ptr;	/* versatile pointer for user */
} xaction;


/* xactor */
typedef struct xactor {
	struct xcomposer xa_base;
	xobject 	*xa_object;	/* an object for normal state */
	int		(*xa_acceptable)(struct xactor *, xsignal); 

	/* get xaction structure for the signal */
	struct xaction *(*xa_getaction)(struct xactor *, xsignal); 

	/* 
	 *	register an action for the signal. 
	 *   if success, returns 1. Otherwise 0.
	 *   All registered xaction's will be freed by
	 *   xactor.
	 */
	int		(*xa_register)(struct xactor *, xsignal,	
				       xaction_t, void *actarg, 
				       struct xobject *); 
	/* 
	 * xr_receive() calls xrc_action() in the xaction structure.
	 *	- act: the actor receiving the signal
	 *	- pos: a position of the actor.
	 *	- sig: a signal sent to this actor
	 *	- sigarg: a signal information
	 *	- root: root object for the xwindow
	 *	- xwp: xwindow
	 */
	void		(*xa_receive)(struct xactor *act, xpoint pos, 
				      xsignal sig, struct xsigarg *sigarg, 
				      struct xobject *root,
				      xwindow *xwp); 
} xactor;

#define xactcast(x)		((struct xactor *)x)
#define xact_is_active(x) 	(xobjcast(x)->x_flags & XOBJFLAG_ACTIVE)
#define xact_activate(x) 	(xobjcast(x)->x_flags |= XOBJFLAG_ACTIVE)
#define xact_deactivate(x) 	(xobjcast(x)->x_flags &= ~XOBJFLAG_ACTIVE)
#define xact_acceptable(x,y) 	 xactcast(x)->xa_acceptable(xactcast(x), y)
#define xact_register(x,s,c,a,o) xactcast(x)->xa_register(xactcast(x),s,c,a,o)
#define xact_is_proxy(x) 	 xact_acceptable(x, XSIG_PROXY)

/* 
 *  send signal:
 *	- calls xa_receive() in the xactor structure 
 *	- if a xaction structure is registered for the signal,
 *	 this function invokes xac_action() in the xaction structure.
 */
void		 xact_send_signal(struct xactor *, xpoint, 
				  xsignal, struct xsigarg *sigarg, 
			     	  struct xobject *, xwindow *);

/* 
 * xproxy
 *	- an object accepting any signals registed in accept list.
 *	If XSIG_ANY is registered, all signals are accepted. 
 *	- After receiving a signal, xproxy finds an actor which can 
 *	accept the signal and sends the signal to the actor.
 *	- An aciton for XSIG_ANY is invoked at every time after sending 
 *	signal to internal actors.
 */
typedef struct xproxy {
	struct xactor 	xp_base;
	/* defined in xproxy.c */
} xproxy;

/* an argument must be composer */
struct xproxy *xproxy_new(struct xcomposer *);
void	       xproxy_accept(struct xproxy *, xsignal);

#if 0
/* 
 * xredirector 
 *	- an object accepting any signals registed in accept list.
 *	If XSIG_ANY is registered, all signals are accepted. 
 *	- After receiving a signal, xredirector finds an actor which can 
 *	accept the signal and sends the signal to the actor.
 *	- No action related to signals cannot be registered to xredirector.
 *	- xredirector contains xposmark to keep the position of the object.
 */
typedef struct xredirector {
	struct xactor 	xrd_base;
	struct xobject 	*xrd_wrap;	/* object passed as 1st argument */
	void		*xrd_accepts; 	/* accept list */
	void		*xrd_redirects; /* destination objects redirected */
} xredirector;

struct xredirector *xredirector_new(struct xobject *);
void		    xredirector_redirect(struct xredirector *src, 
					 struct xredirector *dst);
#endif


/*-------------*  Actor Objects  *------------------*/
/* clickable actor: ENTER, LEAVE, BTNPRESS, BTNRELEASE */
struct xactor *xclickable_new(struct xobject *);

/* key action actor: KEYPRESS, KEYRELEASE */
struct xactor * xkeyaction_new(struct xobject *o);

/* actable scroll object: ENTER, LEAVE, BTNPRESS, BTNRELEASE, GRABMOTION */
struct xactor *xascroll_new(struct xscroll *);

/*-------------*  Internal Functions  *------------------*/
#ifdef _XACTOR_INTERNAL

/* 
 * After registering xaction to xactor, the action data doesn't
 * have to be deleted by user. xaction is refrence counted internally.
 */
struct xaction *xaction_new(xobject *, xaction_t, void *);
void		xaction_delete(struct xaction *);

#define xact_compose(x)	 do {					   \
	if(x && x->xac_object && xobj_is_composer(x->xac_object))  \
		xcmp_compose(x->xac_object);			   \
} while(0)

/* utility functions for actor objects */
void
xactor_init(struct xactor *r, struct xobject *o,
	    x_draw_t, x_free_t, x_dump_t, 
	    void (*compose)(struct xcomposer *o),
	    struct xobject *(*find)(struct xcomposer *o, int x, int y, int sig, xpoint *),
	    void (*relocate)(struct xcomposer *o, int x, int y));
void xactor_draw(struct xobject *o, int x, int y, xwindow *xwp, 
		 int rx, int ry, unsigned int rw, unsigned int rh, int);  
void xactor_relocate(struct xcomposer *o, int x, int y);  
struct xobject *xactor_find(struct xcomposer *o, int x, int y, int sig, xpoint *);  

#endif /* _XACTOR_INTERNAL */
/*-------------*  /Internal Functions  *------------------*/

#endif /* _XP_XREACTOR_H_ */

