/*
 *			xcomposer.c
 * $Id: xcomposer.c,v 1.21 2004/01/24 10:44:31 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

 /* 
  *	This file contains following xcomposer definitions. 
  *		1. xposmark
  *		2. pair-composite
  *		3. selector 
  *		4. scroller
  *		5. frame
  */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "xobject.h"
#include "xoplist.h"

#define xo_abs(x)	((x) < 0 ? -(x) : (x))
#define xo_max(x, y)	((x) < (y) ? (y) : (x))
#define xo_min(x, y)	((x) < (y) ? (x) : (y))

/*------------* Common Functions *-------------*/
/* position difference for width/height */
static int vpos_diff(int max, int size, unsigned int rule)
{
	int ret = 0;
	if((rule & XCRULE_TOP) == XCRULE_TOP)
		ret = 0;
	else if((rule & XCRULE_BOTTOM) == XCRULE_BOTTOM)
		ret = max - size;
	else  /* XCRULE_CENTER */
		ret = (max - size) / 2;
	return ret;
}

static int hpos_diff(int max, int size, unsigned int rule)
{
	int ret = 0;
	if((rule & XCRULE_LEFT) == XCRULE_LEFT)
		ret = 0;
	else if((rule & XCRULE_RIGHT) == XCRULE_RIGHT)
		ret = max - size;
	else  /* XCRULE_CENTER */
		ret = (max - size) / 2;
	return ret;
}

static void 
xo_set(struct xobject *xp, int type, 
       unsigned int w, unsigned int h, 
       x_draw_t drawf, x_free_t freef, x_dump_t dumpf)
{
	xp->x_type = type;
	xp->x_flags = XOBJFLAG_ENABLE;
	xp->x_width = w;
	if(w == XC_VARIABLE)
		xp->x_flags |= XOBJFLAG_WVARIABLE;
		
	xp->x_height = h;
	if(h == XC_VARIABLE)
		xp->x_flags |= XOBJFLAG_HVARIABLE;

	xp->x_draw = drawf;
	xp->x_free = freef;
	xp->x_dump = dumpf;
	xp->x_refcnt = 0;
}

static void 
xo_free(struct xobject *xp){ FREE(xobj_typestr(xp), xp); }

static void 
xo_dump(struct xobject *xp, FILE *fp, int off)
{  
	int  n = 0;
	char buf[1024], info[1024];
	n = xod_indent(buf, off);
	xobj_info(xp, info, 1024);
	snprintf(buf + n, 1024 - n, "%s: %s", xobj_typestr(xp), info);
	fprintf(fp, "%s", buf);
}

/*------------* X Position Marker *-------------*/
static void
posmark_dump(struct xobject *xp, FILE *fp, int n)
{
	struct xposmark *xcp = (struct xposmark *)xp;
	xo_dump(xp, fp, n);
	fprintf(fp, ", mark(%d, %d)\n", xcp->xp_pos.x, xcp->xp_pos.y);
	xobj_dump(xcp->xp_obj, fp, n+1);
}

static void 
posmark_free(struct xobject *xp)
{ 
	struct xposmark *xcp = (struct xposmark *)xp;
	xobj_free(xcp->xp_obj);
	FREE(xobj_typestr(xp), xp); 
}

static void
posmark_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	     int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	struct xposmark *xcp = (struct xposmark *)xp;
	xobj_draw_rect(xcp->xp_obj, x, y, xwp, rx, ry, rw, rh, f);
}

/* posmark dones't return itself if type is any */
static struct xobject *
posmark_find(struct xcomposer *xp, int x, int y, int f, xpoint *p)
{
	struct xposmark *xcp = (struct xposmark *)xp;
	struct xobject *ret = NULL;

	if(xcmpfind_is_firstmatch(f)
	&& (xcmpfind_type(f) == XOBJTYPE_POSMARK))
		return (xobject *)xp;

	if(xobj_is_composer(xcp->xp_obj))
		ret = xcmp_find(xcp->xp_obj, x, y, f, p);

	if(ret)
		return ret;

	else if(xcmpfind_subtype(f) == 0 &&  /* additional condition */
		(xobj_type(xcp->xp_obj) == xcmpfind_type(f)))
		return xcp->xp_obj;

	else if(xcmpfind_type(f) == XOBJTYPE_POSMARK)
		return (xobject *)xp;
	return NULL;
}

/* update position */
static void
posmark_relocate(struct xcomposer *xp, int x, int y)
{
	struct xposmark *xcp = (struct xposmark *)xp;

	if(xobj_is_composer(xcp->xp_obj))
		xcmp_relocate(xcp->xp_obj, x, y);

	/* cache position */
	xcp->xp_pos.x = x;
	xcp->xp_pos.y = y;

}

static void
posmark_compose(struct xcomposer *xp)
{
	struct xposmark *xcp = (struct xposmark *)xp;

	if(xobj_is_composer(xcp->xp_obj))
		xcmp_compose(xcp->xp_obj);

	/* if size variable, update size */
	if(xcmp_wvariable(xcp->xp_obj))
		xobj_width(xp) = xobj_width(xcp->xp_obj);
	if(xcmp_hvariable(xcp->xp_obj))
		xobj_height(xp) = xobj_height(xcp->xp_obj);
}

struct xobject *
xposmark_new(xobject *obj)
{
	struct xposmark *ret = 
		MALLOC(xod_typestr(XOBJTYPE_POSMARK), 
			sizeof(struct xposmark));
		
	xo_set(xobjcast(ret), XOBJTYPE_POSMARK, 
	       xobj_width(obj), xobj_height(obj), 
	       posmark_draw, posmark_free, posmark_dump);

	/* set function pointers */
	xcmpcast(ret)->xc_compose = posmark_compose;
	xcmpcast(ret)->xc_find = posmark_find;
	xcmpcast(ret)->xc_relocate = posmark_relocate;

	/* if either of children has variable size */
	if(xcmp_wvariable(obj))
		xobj_flags(ret) |= XOBJFLAG_WVARIABLE;
	if(xcmp_hvariable(obj))
		xobj_flags(ret) |= XOBJFLAG_HVARIABLE;

	ret->xp_obj = obj;
	xobj_incref(obj);
	ret->xp_pos.x = ret->xp_pos.y = 0;
	ret->xp_data = NULL;
	return (struct xobject *)ret;
}

/*------------* Pair Composite *-------------*/
static void
cmppair_dump(struct xobject *xp, FILE *fp, int n)
{
	char buf[1024], *p = buf;
	struct xcmpstpair *xcp = (struct xcmpstpair *)xp;

	xod_indent(buf, n);
	while(*p)
		*p++ = ' ';

	xo_dump(xp, fp, n);
	fprintf(fp, "\n");
	if(xcp->xcp_first){
		fprintf(fp, "%s - first(%d, %d)\n", buf,
			xcp->xcp_firstoft.x, xcp->xcp_firstoft.y);
		xobj_dump(xcp->xcp_first, fp, n+1);
	}
	if(xcp->xcp_second){
		fprintf(fp, "%s - second(%d, %d)\n", buf,
			xcp->xcp_secondoft.x, xcp->xcp_secondoft.y);
		xobj_dump(xcp->xcp_second, fp, n+1);
	}
}

static void 
cmppair_free(struct xobject *xp)
{ 
	struct xcmpstpair *xcp = (struct xcmpstpair *)xp;
	xobj_free(xcp->xcp_first);
	xobj_free(xcp->xcp_second);
	FREE(xobj_typestr(xp), xp); 
}

/* xcontainer */
void
xcont_nextrect(struct xobject *xp, int x, int y, int rx, int ry, int rw, int rh,
               int *nrx, int *nry, int *nrw, int *nrh);

static void
cmppair_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	     int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	int nx, ny, nrx, nry, nrw, nrh;
	struct xcmpstpair *xcp = (struct xcmpstpair *)xp;

	/* at first, draw the second object to make it lower */
	if(xcp->xcp_second){
		nx = x + xcp->xcp_secondoft.x;
		ny = y + xcp->xcp_secondoft.y;

		xcont_nextrect(xp, xcp->xcp_secondoft.x, xcp->xcp_secondoft.y,
			       rx, ry, rw, rh, &nrx, &nry, &nrw, &nrh);
	
		xobj_draw_rect(xcp->xcp_second, nx, ny, xwp, nrx, nry, nrw, nrh, f); 
	}
		
	/* next, draw the first object overiding second*/
	if(xcp->xcp_first){
		nx = x + xcp->xcp_firstoft.x;
		ny = y + xcp->xcp_firstoft.y;

		xcont_nextrect(xp, xcp->xcp_firstoft.x, xcp->xcp_firstoft.y,
			       rx, ry, rw, rh, &nrx, &nry, &nrw, &nrh);
	
		xobj_draw_rect(xcp->xcp_first, nx, ny, xwp, nrx, nry, nrw, nrh, f); 
	}
}

static struct xobject *
cmppair_find(struct xcomposer *xp, int x, int y, int mask, xpoint *p)
{
	struct xcmpstpair *xcp = (struct xcmpstpair *)xp;
	struct xobject *ret = NULL;

	/* xcp_first must be checked first */
	if(xcp->xcp_first){
		if(xobj_is_composer(xcp->xcp_first))
			ret = xcmp_find(xcp->xcp_first, x - xcp->xcp_firstoft.x, 
					y - xcp->xcp_firstoft.y, mask, p);
		else if(mask == 0)/* upper primitive */
			ret = xcp->xcp_first;
	}

	if(!ret && xcp->xcp_second){
		if(xobj_is_composer(xcp->xcp_second))
			ret = xcmp_find(xcp->xcp_second, x - xcp->xcp_secondoft.x, 
					y - xcp->xcp_secondoft.y, mask, p);
		else if(mask == 0)/* lower primitive */
			ret = xcp->xcp_second;
	}
	return ret;
}

static void
cmppair_relocate(struct xcomposer *xp, int x, int y)
{
	struct xcmpstpair *xcp = (struct xcmpstpair *)xp;

	if(xcp->xcp_first && xobj_is_composer(xcp->xcp_first))
		xcmp_relocate(xcp->xcp_first, x + xcp->xcp_firstoft.x, 
				 y + xcp->xcp_firstoft.y);

	if(xcp->xcp_second && xobj_is_composer(xcp->xcp_second))
		xcmp_relocate(xcp->xcp_second, x + xcp->xcp_secondoft.x, 
				 y + xcp->xcp_secondoft.y);
}

static void
cmppair_compose(struct xcomposer *xp)
{
	struct xcmpstpair *xcp = (struct xcmpstpair *)xp;

	if(xcp->xcp_first && xobj_is_composer(xcp->xcp_first))
		xcmp_compose(xcp->xcp_first);

	if(xcp->xcp_second && xobj_is_composer(xcp->xcp_second))
		xcmp_compose(xcp->xcp_second);

	/* if size variable, update size */
	if(xcmp_wvariable(xp))
		xobj_width(xp) = xo_max(xobj_width(xcp->xcp_first), 
		       		 	xobj_width(xcp->xcp_second));
	if(xcmp_hvariable(xp))
		xobj_height(xp) = xo_max(xobj_height(xcp->xcp_first), 
		       		 	 xobj_height(xcp->xcp_second));

	xcp->xcp_firstoft.x = hpos_diff(xobj_width(xp), 
					 xobj_width(xcp->xcp_first), 
					 xcp->xcp_rule);
	xcp->xcp_firstoft.y = vpos_diff(xobj_height(xp), 
					 xobj_height(xcp->xcp_first), 
					 xcp->xcp_rule);
	xcp->xcp_secondoft.x = hpos_diff(xobj_width(xp), 
					 xobj_width(xcp->xcp_second), 
					 xcp->xcp_rule);
	xcp->xcp_secondoft.y = vpos_diff(xobj_height(xp), 
					  xobj_height(xcp->xcp_second), 
					  xcp->xcp_rule);
}

struct xobject *
xcmpstpair_new(struct xobject *first, struct xobject *second, int rule)
{
	struct xcmpstpair *ret = 
		MALLOC(xod_typestr(XOBJTYPE_CMPSTPAIR), 
			sizeof(struct xcmpstpair));
		
	xo_set(xobjcast(ret), XOBJTYPE_CMPSTPAIR, 
	       xo_max(xobj_width(first), xobj_width(second)),
	       xo_max(xobj_height(first), xobj_height(second)),
	       cmppair_draw, cmppair_free, cmppair_dump);

	/* set function pointers */
	xcmpcast(ret)->xc_compose = cmppair_compose;
	xcmpcast(ret)->xc_find = cmppair_find;
	xcmpcast(ret)->xc_relocate = cmppair_relocate;

	/* if either of children has variable size */
	if(xcmp_wvariable(first) || xcmp_wvariable(second))
		xobj_flags(ret) |= XOBJFLAG_WVARIABLE;
	if(xcmp_hvariable(first) || xcmp_hvariable(second))
		xobj_flags(ret) |= XOBJFLAG_HVARIABLE;

	ret->xcp_first = first;
	xobj_incref(first);
	ret->xcp_second = second;
	xobj_incref(second);
	ret->xcp_rule = rule;
	ret->xcp_firstoft.x = ret->xcp_firstoft.y = 0;
	ret->xcp_secondoft.x = ret->xcp_secondoft.y = 0;
	return (struct xobject *)ret;
}

/*------------* X Object Selector  *-------------*/

static void
xselector_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	       int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	struct xselector *xsp = (struct xselector *)xp;

	if((xsp->xs_objs)[xsp->xs_idx] == NULL)
		return;

	xobj_draw_rect(xsp->xs_objs[xsp->xs_idx], x, y, xwp, rx, ry, rw, rh, f);
}

static int
xobj_indent(char *bufp, int offset)
{
	int ret = 6;
	bufp[0] = '\0';
	strcpy(bufp, "(xobj)");
	
	while(offset > 0){
		strcat(bufp, "  ");
		--offset;
		ret += 2;
	}
	return ret;
}

static void
xselector_dump(struct xobject *xp,  FILE *fp, int oft)
{
	int i = 0;
	char buf[1024], *p;
	struct xselector *xsp = (struct xselector *)xp;

	xod_indent(buf, oft);
	fprintf(fp, "%sXObjectSelect(%dx%d): {\n", buf,
		xobj_width(xp), xobj_height(xp));

	p = buf;
	while(*p)
		*p++ = ' ';

	for(i = 0; i < xsp->xs_nobjs; ++i){
		fprintf(fp, "%s %d: ", buf, i);
		if(xsp->xs_objs[i]){
			fprintf(fp, "\n");
			xobj_dump(xsp->xs_objs[i], fp, oft + 1);
		}
		else
			fprintf(fp, "None\n");
	}
	fprintf(fp, "%s}",buf);
}

static void
xselector_free(struct xobject *xp)
{
	int i = 0;
	struct xselector *xsp = (struct xselector *)xp;

	for(i = 0; i < xsp->xs_nobjs; ++i){
		if(xsp->xs_objs[i])
			xobj_free(xsp->xs_objs[i]);
	}
	FREE("xselector/objs", xsp->xs_objs);
	FREE("xselector", xsp);
}


static struct xobject *
xselector_find(struct xcomposer *xp, int x, int y, int mask, xpoint *p)
{
	struct xselector *xsp = (struct xselector *)xp;
	struct xobject *cp, *ret = NULL;

	cp = xsp->xs_objs[xsp->xs_idx];

	if(cp){
		if(xobj_is_composer(cp))
			ret = xcmp_find(cp, x, y, mask, p);
		else if(mask == 0)
			ret = cp;
	}
	return ret;
}

static void
xselector_relocate(struct xcomposer *xp, int x, int y)
{
	struct xselector *xsp = (struct xselector *)xp;
	struct xobject *cp;

	cp = xsp->xs_objs[xsp->xs_idx];
	if(cp) 
		if(xobj_is_composer(cp))
			xcmp_compose(cp);
}

static void
xselector_compose(struct xcomposer *xp)
{
	struct xselector *xsp = (struct xselector *)xp;

	/* already composed */
	if(xcmp_wvariable(xp) && xsp->xs_objs[xsp->xs_idx])
		xobj_width(xp) = xobj_width(xsp->xs_objs[xsp->xs_idx]);
	if(xcmp_hvariable(xp) && xsp->xs_objs[xsp->xs_idx])
		xobj_height(xp) = xobj_height(xsp->xs_objs[xsp->xs_idx]);
}

struct xobject *
xselector_new(unsigned char nobj, unsigned int w, unsigned int h)
{
	struct xselector *ret = MALLOC("xselector", sizeof(struct xselector));
	ret->xs_nobjs = nobj;
	ret->xs_objs = MALLOC("xselector/objs", sizeof(struct xobject *) * nobj);
	
	xo_set(xobjcast(ret), XOBJTYPE_SELECTOR, w, h, 
		xselector_draw, xselector_free, xselector_dump);

	xcmpcast(ret)->xc_compose = xselector_compose;
	xcmpcast(ret)->xc_find = xselector_find;
	xcmpcast(ret)->xc_relocate = xselector_relocate;
	return (struct xobject *)ret;
}

void	
xsel_set(struct xobject *xp, unsigned char idx, struct xobject *obj)
{
	struct xselector *sel = (struct xselector *)xp;
	if(idx > sel->xs_nobjs)
		return;

	/* compose here */
	if(obj && xobj_is_composer(obj))
		xcmp_compose(obj);

	/* free previous entry */
	if(sel->xs_objs[idx])
		xobj_free(sel->xs_objs[idx]);

	xobj_incref(obj);
	sel->xs_objs[idx] = obj;

}

void            
xsel_select(struct xobject *xp, unsigned char idx)
{
	struct xselector *sel = (struct xselector *)xp;
	if(idx == sel->xs_idx || idx >= sel->xs_nobjs)
		return;
	sel->xs_idx = idx;
}


/*------------* X Scroll Objects *-------------*/
struct xscrl {
	struct xscroll    _base;
	struct xobject 	  *obj;  /* composited object */
	short		  xdiff, ydiff;  /* difference from root */
	xattr		  *xat;
	xwindow		  *pixmap;	/* pixmap scroll */
};

static void xscrl_free(struct xobject *xp){ 
	struct xscrl *xsrp = (struct xscrl *)xp;
	xobj_free(xsrp->obj);	/* free composit object */
	if(xsrp->pixmap)
		xwindow_destroy(xsrp->pixmap);
	FREE("xscroll", xp); 
}

static void 
xscrl_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	   int rx, int ry, unsigned int rw, unsigned int rh, int f){ 
	struct xscrl *xsrp = (struct xscrl *)xp;

	if(!xsrp->pixmap){
		long pixel = xw_get_bgcolor(xwp);

		if(xsrp->xat) 
			xattr_get_bgcolor(xsrp->xat, &pixel);
						
		xsrp->pixmap = xpixmap_create(xwp, pixel,
				  xobj_width(xsrp->obj), xobj_height(xsrp->obj));
		xw_clear(xsrp->pixmap);

		/* draw everything */
		xobj_draw(xsrp->obj, 0, 0, xsrp->pixmap, f); 
	}
	else {
		/* clear the rectangle if clear flag is set */
		if(f & XOBJDRAW_REDRAW){
			xw_clear_region(xsrp->pixmap, xsrp->xdiff + rx, 
					xsrp->ydiff + ry, rw, rh);
		
			/* re-draw part of the window */
			xobj_draw_rect(xsrp->obj, 0, 0, xsrp->pixmap, 
				  xsrp->xdiff + rx, xsrp->ydiff + ry, rw, rh, f); 
		}
	}

	xw_copy_region(xsrp->pixmap, xwp, xsrp->xdiff + rx, 
		       xsrp->ydiff + ry, rw, rh, rx + x, ry + y);
}

static void xscrl_dump(struct xobject *xp, FILE *fp, int oft){ 
	struct xscrl *xsrp = (struct xscrl *)xp;
	xo_dump(xp, fp, oft);
	fprintf(fp, ", diff(%d, %d)\n", xsrp->xdiff, xsrp->ydiff);
	fprintf(fp, "\n");
	xobj_dump(xsrp->obj, fp, oft + 1);
}

static void            
xscrl_compose(struct xcomposer *xcp)
{
	if(xobj_is_composer(((struct xscrl *)xcp)->obj))
		xcmp_compose(((struct xscrl *)xcp)->obj);
}

struct xobject *
xscrl_find(struct xcomposer *xcp, int x, int y, int mask, xpoint *p)
{
	struct xscrl *xsrp = (struct xscrl *)xcp;
	struct xobject *ret = NULL;

	if(xobj_is_composer(xsrp->obj)){
		ret = xcmp_find(xsrp->obj, x+xsrp->xdiff, 
				 y+xsrp->ydiff, mask, p);
		if(ret){
			if(p){
				p->x -= xsrp->xdiff; 
				p->y -= xsrp->ydiff; 
			}
			return ret;
		}
	}
	else if(mask == 0)	/* primitive */
		ret = xsrp->obj;
	return ret;
}

static void
xscrl_relocate(struct xcomposer *xp, int x, int y)
{
	struct xscrl *xsrp = (struct xscrl *)xp;
	if(xobj_is_composer(xsrp->obj))
		return xcmp_relocate(xsrp->obj, x-xsrp->xdiff, y-xsrp->ydiff);
}

void
xscrl_scroll(struct xscroll *xsp, int x, int y, xwindow *xwp, 
	     struct xobject *root, int mx, int my)
{
	struct xscrl *xsrp = (struct xscrl *)xsp;
	struct xobject *obj = xsrp->obj;
	short  nextx, nexty;

	nextx = xsrp->xdiff + mx;
	nexty = xsrp->ydiff + my;

	if(xobj_width(xsp) >= xobj_width(obj) || nextx < 0)
		nextx = 0;
	else if(nextx + xobj_width(xsp) >= xobj_width(obj))
		nextx = xobj_width(obj) - xobj_width(xsp);

	if(xobj_height(xsp) >= xobj_height(obj) || nexty < 0)
		nexty = 0;
	else if(nexty + xobj_height(xsp) >= xobj_height(obj))
		nexty = xobj_height(obj) - xobj_height(xsp);

	/* is it necessary to scroll? */
	if(xsrp->xdiff != nextx || xsrp->ydiff != nexty){
		xsrp->xdiff = nextx;
		xsrp->ydiff = nexty;

		/*
		xw_copy_region(xsrp->pixmap, xwp, nextx, nexty,
			       xobj_width(xsp), xobj_height(xsp), x, y);
		*/
		xobj_draw_object(root, xsp, x, y, xwp, 0);
		/* make link enable */
		xcmp_relocate(xsrp, x, y);
	 }
}

void
xscrl_move(struct xscroll *xsp, int x, int y, int mx, int my)
{
	struct xscrl *xsrp = (struct xscrl *)xsp;
	struct xobject *obj = xsrp->obj;
	short  nextx, nexty;

	nextx = xsrp->xdiff + mx;
	nexty = xsrp->ydiff + my;

	if(nextx + xobj_width(xsp) >= xobj_width(obj))
		nextx = xobj_width(obj) - xobj_width(xsp);
	else if(nextx < 0)
		nextx = 0;

	if(nexty + xobj_height(xsp) >= xobj_height(obj))
		nexty = xobj_height(obj) - xobj_height(xsp);
	else if(nexty < 0)
		nexty = 0;

	/* is it necessary to scroll? */
	if(xsrp->xdiff != nextx || xsrp->ydiff != nexty){
		xsrp->xdiff = nextx;
		xsrp->ydiff = nexty;

		/* make link enable */
		xcmp_relocate(xsrp, x, y);
	 }
}


void
xscrl_reset(struct xscroll *xsp, int x, int y, xwindow *xwp, 
	    struct xobject *root)
{
	struct xscrl *xsrp = (struct xscrl *)xsp;
	xsrp->xdiff = xsrp->ydiff = 0;
	xobj_draw_object(root, xsp, x, y, xwp, 0);
	xcmp_relocate(xsrp, x, y);
}


struct xobject *
xscroll_new_bg(struct xobject *xcp, unsigned int w, unsigned int h, 
	       xattr *bg)
{
	struct xscrl *ret = MALLOC("xscroll", sizeof(struct xscrl));
	/* struct xscroll *xsp = (struct xscroll *)ret; */

	xo_set((struct xobject *)ret, XOBJTYPE_SCROLL, 
		     w, h, xscrl_draw, xscrl_free, xscrl_dump);
	xcmpcast(ret)->xc_find = xscrl_find;
	xcmpcast(ret)->xc_compose = xscrl_compose;
	xcmpcast(ret)->xc_relocate = xscrl_relocate;
	ret->xdiff = ret->ydiff = 0;
	ret->pixmap = NULL;
	ret->obj = xcp;
	ret->xat = bg;
	xobj_incref(xcp);

	return (struct xobject *)ret;
}

struct xobject *
xscroll_new(struct xobject *xcp, unsigned int w, unsigned int h)
{
	return xscroll_new_bg(xcp, w,  h, NULL);
}


/*------------* X Frame Objects *-------------*/
struct xframe {
	struct xcomposer  _base;
	struct xobject 	  *obj;  /* composited object */
	xwindow		  *pixmap;	/* pixmap scroll */
	xattr		  *xat;
};

static void 
xframe_free(struct xobject *xp)
{ 
	struct xframe *xfrp = (struct xframe *)xp;
	xobj_free(xfrp->obj);	/* free composit object */
	if(xfrp->pixmap)
		xwindow_destroy(xfrp->pixmap);
	FREE("xframe", xp); 
}

static void 
xframe_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	   int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	struct xframe *xsrp = (struct xframe *)xp;

	if(!xsrp->pixmap){
		long pixel = xw_get_bgcolor(xwp);

		if(xsrp->xat) 
			xattr_get_bgcolor(xsrp->xat, &pixel);
						
		xsrp->pixmap = xpixmap_create(xwp, pixel,
				  xobj_width(xsrp->obj), xobj_height(xsrp->obj));
		xw_clear(xsrp->pixmap);

		/* draw everything */
		xobj_draw(xsrp->obj, 0, 0, xsrp->pixmap, f); 
	}
	else {
		/* clear the rectangle if clear flag is set */
		if(f & XOBJDRAW_REDRAW)
			xw_clear_region(xsrp->pixmap,  rx, ry, rw, rh);
		
			/* re-draw part of the window */
		/*xobj_draw_rect(xfrp->obj, x, y, xfrp->pixmap, rx, ry, rw, rh, f);*/
		xobj_draw_rect(xsrp->obj, 0, 0, xsrp->pixmap, rx, ry, rw, rh, f); 
	}

	xw_copy_region(xsrp->pixmap, xwp,  rx, ry, rw, rh, rx + x, ry + y);
}

static void 
xframe_dump(struct xobject *xp, FILE *fp, int oft)
{ 
	struct xframe *xfrp = (struct xframe *)xp;
	xo_dump(xp, fp, oft);
	fprintf(fp, "\n");
	xobj_dump(xfrp->obj, fp, oft + 1);
}

static void            
xframe_compose(struct xcomposer *xcp)
{
	struct xframe *xfrp = (struct xframe *)xcp;
	if(xobj_is_composer(((struct xframe *)xcp)->obj))
		xcmp_compose(((struct xframe *)xcp)->obj);

	if(xcmp_wvariable(xcp))
		xobj_width(xcp) = xobj_width(xfrp->obj);
	if(xcmp_hvariable(xcp))
		xobj_height(xcp) = xobj_height(xfrp->obj);
}

struct xobject *
xframe_find(struct xcomposer *xcp, int x, int y, int mask, xpoint *p)
{
	struct xframe *xfrp = (struct xframe *)xcp;
	if(xobj_is_composer(xfrp->obj))
		return xcmp_find(xfrp->obj, x, y, mask, p);
	else if(mask == 0)
		return xfrp->obj;
	return NULL;
}

static void
xframe_relocate(struct xcomposer *xp, int x, int y)
{
	struct xframe *xfrp = (struct xframe *)xp;
	if(xobj_is_composer(xfrp->obj))
		return xcmp_relocate(xfrp->obj, x, y);
}


struct xobject *
xframe_new_bg(struct xobject *xcp, unsigned int w, unsigned int h, xattr *at)
{
	struct xframe *ret = MALLOC("xframe", sizeof(struct xframe));
	xo_set((struct xobject *)ret, XOBJTYPE_FRAME,
		     w, h, xframe_draw, xframe_free, xframe_dump);
	xcmpcast(ret)->xc_find = xframe_find;
	xcmpcast(ret)->xc_compose = xframe_compose;
	xcmpcast(ret)->xc_relocate = xframe_relocate;
	ret->pixmap = NULL;
	ret->obj = xcp;
	xobj_incref(xcp);
	ret->xat = at;
	return (struct xobject *)ret;
}
struct xobject *
xframe_new(struct xobject *xcp, unsigned int w, unsigned int h)
{
	return xframe_new_bg(xcp, w,  h, NULL);
}
