/*
 *			xcontainer.c
 * $Id: xcontainer.c,v 1.12 2004/01/21 16:52:52 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* 
 *	This file contains three container definitions.
 *		1. x vertical layout 
 *		2. x hertical layout 
 *		3. x composition
 */

#include <stdio.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "xobject.h"
#include "xoplist.h"

#define xo_abs(x)	((x) < 0 ? -(x) : (x))
#define xo_max(x, y)	((x) < (y) ? (y) : (x))
#define xo_min(x, y)	((x) < (y) ? (x) : (y))

/*------------* X Container External Function *-------------*/
void 
xcont_add(struct xobject *xcp, struct xobject *xp, unsigned int rule)

{
	struct xoposition *xpp = MALLOC("xoposition", sizeof(struct xoposition));
	xpp->xp_object = xp;
	xpp->xp_rule = rule;
	xobj_incref(xp);		/* increment reference count */
	iterator_append(xcont_list(xcp), xpp);
}
void 
xcont_remove(struct xobject *xcp, int idx)
{
	xoplist_iterator itr;
	itr = iterator_at(xcont_list(xcp), idx);
	if(!iterator_is_end(itr)){
		struct xobject *p = iterator_data(itr)->xp_object;
		iterator_erase(itr);
		xobj_decref(p);
	}
}

int xcont_count(struct xobject *xcp)
{
	return iterator_count(xcont_list(xcp));
}
struct xoposition *xcont_at(struct xobject *xcp, int idx)
{
	xoplist_iterator itr;
	itr = iterator_at(xcont_list(xcp), idx);
	if(!iterator_is_end(itr))
		return iterator_data(itr);
	return NULL;
}

int     
xcont_theight(struct xobject *xcp)
{
	int ret = 0;
	xoplist_iterator itr;
	itr = iterator_first(xcont_list(xcp));
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xoposition *xpp;
		xpp = iterator_data(itr);
		ret += xobj_height(xpp->xp_object);
	}
	return ret;
}

int     
xcont_twidth(struct xobject *xcp)
{
	int ret = 0;
	xoplist_iterator itr;
	itr = iterator_first(xcont_list(xcp));
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xoposition *xpp;
		xpp = iterator_data(itr);
		ret += xobj_width(xpp->xp_object);
	}
	return ret;
}

/*------------* Common Function *-------------*/
static void 
xo_set(struct xobject *xp, int type, 
       unsigned int w, unsigned int h, 
       x_draw_t draw, x_free_t free, x_dump_t dump)
{
	xp->x_type = type;
	xp->x_flags = XOBJFLAG_ENABLE;
	xp->x_width = w;
	if(w == XC_VARIABLE)
		xp->x_flags |= XOBJFLAG_WVARIABLE;
		
	xp->x_height = h;
	if(h == XC_VARIABLE)
		xp->x_flags |= XOBJFLAG_HVARIABLE;

	xp->x_draw = draw;
	xp->x_free = free;
	xp->x_dump = dump;
	xp->x_refcnt = 0;
}

static void 
xo_dump(struct xobject *xp, FILE *fp, int off)
{  
	int  n = 0;
	char buf[1024], info[1024];
	n = xod_indent(buf, off);
	xobj_info(xp, info, 1024);
	snprintf(buf + n, 1024 - n, "%s: %s", xobj_typestr(xp), info);
	fprintf(fp, "%s", buf);
}


static int
is_overlaped(struct xobject *xp, int x, int y, int rx, int ry,
	     int rw, int rh)
{
	int maxx = x + xobj_width(xp);
	int maxy = y + xobj_height(xp);

	/* rectangle (rx,ry)-rw*rh is out of the rectangle of object xp */	
	if(maxx <= rx || maxy <= ry || rx + rw <= x || ry + rh <= y)
		return 0;
	return 1;
}

void
xcont_nextrect(struct xobject *xp, int x, int y, int rx, int ry, int rw, int rh, 
	       int *nrx, int *nry, int *nrw, int *nrh)
{
	int maxx = x + xobj_width(xp);
	int maxy = y + xobj_height(xp);
	int rmaxx = rx + rw;
	int rmaxy = ry + rh;
	int nmaxx = xo_min(maxx, rmaxx);
	int nmaxy = xo_min(maxy, rmaxy);

	*nrx = xo_max(x, rx);
	*nry = xo_max(y, ry);
	*nrw = nmaxx - *nrx;
	*nrh = nmaxy - *nry;

	*nrx -= x;
	*nry -= y;
}

/*------------* X Container Function *-------------*/
static void 
xcrfree(struct xoposition *xp) { 
	xobj_free(xp->xp_object);
	FREE("xoposition", xp); 
}

static void xcr_free(struct xobject *xp){ 
	aggregator_delete_by(xcont_list(xp), xcrfree);
	FREE(xobj_typestr(xp), xp); 
}

static void 
xcr_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	      int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	xoplist_iterator itr;

	/*  if the type of object is xcomposite, draw lower 
 	 * object first.
 	 */
	if(xobj_type(xp) == XOBJTYPE_COMPOSITE){
		itr = iterator_last(xcont_list(xp));
		iterator_set_backward(itr);
	}
	else
		itr = iterator_first(xcont_list(xp));

	/*
	 *    xwp: (0, 0), xwp->x_width * xwp->x_height
	 *  +-------------------------------------------+
	 *  |                                           |
	 *  |   xp:(x,y), xobj_width(xp)*xobj_height(xp)|
	 *  |   +-------------------------+             |
	 *  |   |                         |             |
	 *  |   |      (rx,ry),rx*rh      |             |
	 *  |   |     +---------------+   |             |
	 *  |   |     |***************|   |             |
	 *  |   |     |***************|   |             |
	 *  |   |     |***************|   |             |
	 *  |   |     |***************|   |             |
	 *  |   |     +---------------+   |             |
	 *  |   |                         |             |
	 *  |   +-------------------------+             |
	 *  +-------------------------------------------+
	 */

	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xoposition *xpp = iterator_data(itr);
		struct xobject *cp = xpp->xp_object;

		/* the position for this object */
		int cx = x + xpp->xp_offset.x;
		int cy = y + xpp->xp_offset.y;
		int ofx =  xpp->xp_offset.x;
		int ofy =  xpp->xp_offset.y;

		/* this object is overlapped with the target rectangle? */
		if(is_overlaped(cp, ofx, ofy, rx, ry, rw, rh)){
			int nrx, nry, nrw, nrh;
			xcont_nextrect(cp, ofx, ofy, rx, ry, rw, rh, 
				 	&nrx, &nry, &nrw, &nrh);
			xobj_draw_rect(cp, cx, cy, xwp, nrx, nry, nrw, nrh, f);
#if 0
			xobj_draw_rect(cp, cx, cy, xwp, rx, ry, rw, rh, f);
#endif 
		}

		/* it is not overlaped */
		else {
			/* keep going if type is composite */
			if(xobj_type(xp) == XOBJTYPE_COMPOSITE)
				continue; 

			/* 
			 *   iterator is going to lower position.
			 *   type is vlayout and the target rectangle is 
 			 *  too upper for coming object.
			 */
			if(xobj_type(xp) == XOBJTYPE_VLAYOUT && ry + rh <  ofy)
				break;

			/* 
			 *   iterator is going to more right position.
			 *   type is hlayout and the target rectangle is 
 			 *  too left for coming object.
			 */
			if(xobj_type(xp) == XOBJTYPE_HLAYOUT && rx + rw <  ofx)
				break;
		}
	}
}

static void
xcr_dump(struct xobject *xcp, FILE *fp, int oft)
{
	int  			i = 1;
	char 			idt[124], *p;
	xoplist_iterator 	itr;
	struct xoposition	*xpp;

	xo_dump(xcp, fp, oft);
	fprintf(fp, "{\n");
	xod_indent(idt, oft);
	p = idt;
	while(*p)
		*p++ = ' ';

	itr = iterator_first(xcont_list(xcp));
	for(; !iterator_is_end(itr); itr = iterator_next(itr), ++i){
		xpp = iterator_data(itr);
		fprintf(fp, "%s %d. (%d, %d)\n", idt, i, xpp->xp_offset.x, 
			xpp->xp_offset.y);
		xpp->xp_object->x_dump(xpp->xp_object, fp, oft + 1);
	}
	fprintf(fp, "%s}\n", idt);
}

static void 
xcr_relocate(struct xcomposer *xp, int rx, int ry)
{
	xoplist_iterator itr;
	itr = iterator_first(xcont_list(xp));

	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xoposition *xpp = iterator_data(itr);
		struct xobject *cp = xpp->xp_object;
		if(xobj_is_composer(cp))
			xcmp_relocate(cp, rx + xpp->xp_offset.x, 
					ry + xpp->xp_offset.y);
	}
}


static struct xobject *
xcr_find(struct xcomposer *_xcp, int dx, int dy, int mask, xpoint *p)
{
	struct xcontainer *xcp = xcontcast(_xcp);
	xoplist_iterator itr;
	itr = iterator_first(xcont_list(xcp));

	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xoposition *xpp = iterator_data(itr);
		struct xobject  *xp = xpp->xp_object;
		struct xobject  *ret;

		if(xpp->xp_offset.x >= dx || 
		   xpp->xp_offset.x + xobj_width(xp) < dx)
			continue;
		if(xpp->xp_offset.y >= dy || 
		   xpp->xp_offset.y + xobj_height(xp) < dy)
			continue;

		if(xobj_is_composer(xp)){
			ret = xcmp_find(xp, dx - xpp->xp_offset.x, 
					dy - xpp->xp_offset.y, mask, p);

			/* composite must check other elements */
			if(!ret && xobj_type(xcp) == XOBJTYPE_COMPOSITE)
				continue;

			/* otherwise return */
			if(p){
				p->x += xpp->xp_offset.x; 
				p->y += xpp->xp_offset.y; 
			}
			return ret;
		}

		/* primitive */
		else if(mask == 0){
			if(p){
				p->x += xpp->xp_offset.x; 
				p->y += xpp->xp_offset.y; 
			}
			return xp;
		}
	}
	return NULL;
}

static void xcontainer_alloc(struct xcontainer *xcp)
{
	xcmpcast(xcp)->xc_find = xcr_find;
	xcmpcast(xcp)->xc_relocate = xcr_relocate;
	xcont_list(xcp) = xoplist_new(4, 4, stk_malloc, stk_free);
}


/*--------------------------------------------------------------*/


/*------------* X Container Objects *-------------*/
/* position difference for width/height */
static int vpos_diff(int max, int size, unsigned int rule)
{
	int ret = 0;
	if((rule & XCRULE_TOP) == XCRULE_TOP)
		ret = 0;
	else if((rule & XCRULE_BOTTOM) == XCRULE_BOTTOM)
		ret = max - size;
	else  /* XCRULE_CENTER */
		ret = (max - size) / 2;
	return ret;
}

static int hpos_diff(int max, int size, unsigned int rule)
{
	int ret = 0;
	if((rule & XCRULE_LEFT) == XCRULE_LEFT)
		ret = 0;
	else if((rule & XCRULE_RIGHT) == XCRULE_RIGHT)
		ret = max - size;
	else  /* XCRULE_CENTER */
		ret = (max - size) / 2;
	return ret;
}

struct xlayout {
	struct xcontainer _base;
	int    mergin;
	int    rule;
	int    diff;
};

  /* Vertical Layout Container */
static void xvlayout_compose(struct xcomposer *xcp)
{
	xoplist_iterator itr;
	struct xlayout *xlp = (struct xlayout *)xcp;
	int    x = 0, y = 0;
	int    nexpand = 0, total = 0;
	int  maxwidth = -1;
	itr = iterator_first(xcont_list(xcp));

	y = xlp->mergin;	/* vertical mergin */

	/* count expanders and compose child containers */
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xobject *cp = iterator_data(itr)->xp_object;

		if(xobj_is_composer(cp))
			xcmp_compose(cp);

		if(xobj_type(cp) == XOBJTYPE_EXPANDER)
			++nexpand;
		else
			total += xobj_height(cp);

		maxwidth = xo_max(maxwidth, xobj_width(cp));
	}

	/* maximum width */
	if(xcmp_wvariable(xcp))
		xobj_width(xcp) = maxwidth;

	/* variable height */
	if(xcmp_hvariable(xcp))
		xobj_height(xcp) = total + (xlp->mergin * 2);


	/* start of composing */
	itr = iterator_first(xcont_list(xcp));

	/* 
	 * if total height is greater than the height of this container 
	 * 		OR
	 *    number of expanders is 0 and layout rule is top 
	 */
	if(total >= (xobj_height(xcp) - (xlp->mergin * 2)) || 
	   (nexpand == 0 && xlp->rule == XCRULE_TOP)){
		for(; !iterator_is_end(itr); itr = iterator_next(itr)){
			struct xoposition *pp = iterator_data(itr);
			int  max = xobj_width(xcp), 
			     size = xobj_width(pp->xp_object);

			pp->xp_offset.x = x + hpos_diff(max, size, pp->xp_rule);
			pp->xp_offset.y = y;
			y += xobj_height(pp->xp_object);
		}
	}

	/* 
	 * if there is no expander, obey the layout rule. 
	 */ 
	else if(nexpand == 0){
		if(xlp->rule == XCRULE_CENTER)
			y = (xobj_height(xcp) - total)/2;
		else if(xlp->rule == XCRULE_BOTTOM)
			y = xobj_height(xcp) - total - xlp->mergin;

		for(; !iterator_is_end(itr); itr = iterator_next(itr)){
			struct xoposition *pp = iterator_data(itr);
			int  max = xobj_width(xcp), 
			     size = xobj_width(pp->xp_object);

			pp->xp_offset.x = x + hpos_diff(max, size, pp->xp_rule);
			pp->xp_offset.y = y;
			y += xobj_height(pp->xp_object);
		}
	}

	/* there are expanders more than one */
	else {
		int expsize = (xobj_height(xcp) 
				- total - (xlp->mergin * 2)) / nexpand;
		int nrest = (xobj_height(xcp) 
				- total - (xlp->mergin * 2)) % nexpand;

		for(; !iterator_is_end(itr); itr = iterator_next(itr)){
			struct xoposition *pp = iterator_data(itr);
			struct xobject *cp = pp->xp_object;
			int  max = xobj_width(xcp), 
			     size = xobj_width(pp->xp_object);

			pp->xp_offset.x = x + hpos_diff(max, size, pp->xp_rule);
			pp->xp_offset.y = y;
			if(xobj_type(cp) == XOBJTYPE_EXPANDER){
				y += expsize;
				if(nrest > 0){
					++y;
					--nrest;
				}
			}
			else
				y += xobj_height(cp);
		}
	}
}


struct xobject *
xvlayout_new(unsigned int width, unsigned int height, 
	     unsigned int mergin, unsigned int layout_rule)
{
	struct xlayout *ret = MALLOC(xod_typestr(XOBJTYPE_VLAYOUT), 
				     sizeof(struct xlayout));
	struct xcontainer *xcr = (struct xcontainer *)ret;

	if(layout_rule != XCRULE_CENTER && 
	   layout_rule != XCRULE_TOP &&
	   layout_rule != XCRULE_BOTTOM)
		layout_rule = XCRULE_TOP; /* set default */

	xo_set((struct xobject *)ret, XOBJTYPE_VLAYOUT, width, 
		height, xcr_draw, xcr_free, xcr_dump);
	xcontainer_alloc(xcr);
	xcmpcast(xcr)->xc_compose = xvlayout_compose;
	ret->mergin = mergin;
	ret->rule = layout_rule;
	return (struct xobject *)ret;
}


  /* Horizontal Layout Container */
static void xhlayout_compose(struct xcomposer *xcp)
{
	xoplist_iterator itr;
	struct xlayout *xlp = (struct xlayout *)xcp;
	int    x = 0, y = 0;
	int    nexpand = 0, total = 0;
	int   maxheight = -1;
	itr = iterator_first(xcont_list(xcp));

	x = xlp->mergin;	/* horizontal mergin */

	/* count expanders and compose child containers */
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xobject *cp = iterator_data(itr)->xp_object;

		if(xobj_is_composer(cp))
			xcmp_compose(cp);

		if(xobj_type(cp) == XOBJTYPE_EXPANDER)
			++nexpand;
		else
			total += xobj_width(cp);

		maxheight = xo_max(maxheight, xobj_height(cp));
	}

	/* maximum height */
	if(xcmp_hvariable(xcp))
		xobj_height(xcp) = maxheight;

	/* total width is variable */
	if(xcmp_wvariable(xcp))
		xobj_width(xcp) = total + (xlp->mergin * 2);


	/* start of composing */
	itr = iterator_first(xcont_list(xcp));

	/* 
	 * if total width is greater than the width of this container 
	 * 		OR
	 *    number of expanders is 0 and layout rule is top 
	 */
	if(total >= (xobj_width(xcp) - (xlp->mergin * 2)) || 
	   (nexpand == 0 && xlp->rule == XCRULE_LEFT)){
		for(; !iterator_is_end(itr); itr = iterator_next(itr)){
			struct xoposition *pp = iterator_data(itr);
			int  max = xobj_height(xcp), 
			     size = xobj_height(pp->xp_object);

			pp->xp_offset.x = x;
			pp->xp_offset.y = y + vpos_diff(max, size, pp->xp_rule);
			x += xobj_width(pp->xp_object);
		}
	}

	/* 
	 * if there is no expander, obey the layout rule. 
	 */ 
	else if(nexpand == 0){
		if(xlp->rule == XCRULE_CENTER)
			x = (xobj_width(xcp) - total)/2;
		else if(xlp->rule == XCRULE_RIGHT)
			x = xobj_width(xcp) - total - xlp->mergin;

		for(; !iterator_is_end(itr); itr = iterator_next(itr)){
			struct xoposition *pp = iterator_data(itr);
			int  max = xobj_height(xcp), 
			     size = xobj_height(pp->xp_object);

			pp->xp_offset.x = x;
			pp->xp_offset.y = y + vpos_diff(max, size, pp->xp_rule);
			x += xobj_width(pp->xp_object);
		}
	}

	/* there are expanders more than one */
	else {
		int expsize = (xobj_width(xcp) 
				- total - (xlp->mergin * 2)) / nexpand;
		int nrest = (xobj_width(xcp) 
				- total - (xlp->mergin * 2)) % nexpand;

		for(; !iterator_is_end(itr); itr = iterator_next(itr)){
			struct xoposition *pp = iterator_data(itr);
			struct xobject *cp = pp->xp_object;
			int  max = xobj_height(xcp), 
			     size = xobj_height(pp->xp_object);

			pp->xp_offset.x = x;
			pp->xp_offset.y = y + vpos_diff(max, size, pp->xp_rule);
			if(xobj_type(cp) == XOBJTYPE_EXPANDER){
				x += expsize;
				if(nrest > 0){
					++x;
					--nrest;
				}
			}
			else
				x += xobj_width(cp);
		}
	}
}


struct xobject *
xhlayout_new(unsigned int width, unsigned int height, 
	     unsigned int mergin, unsigned int layout_rule)
{
	struct xlayout *ret = MALLOC(xod_typestr(XOBJTYPE_HLAYOUT), 
				     sizeof(struct xlayout));
	struct xcontainer *xcr = (struct xcontainer *)ret;

	if(layout_rule != XCRULE_CENTER && 
	   layout_rule != XCRULE_LEFT &&
	   layout_rule != XCRULE_RIGHT)
		layout_rule = XCRULE_LEFT; /* set default */

	xo_set((struct xobject *)ret, XOBJTYPE_HLAYOUT, width, 
		height, xcr_draw, xcr_free, xcr_dump);
	xcontainer_alloc(xcr);
	xcmpcast(xcr)->xc_compose = xhlayout_compose;
	ret->mergin = mergin;
	ret->rule = layout_rule;
	return (struct xobject *)ret;
}

  /* Composite Container */
static void xcomposite_compose(struct xcomposer *xcp)
{
	xoplist_iterator itr;
	int   	maxw = -1, maxh = -1;
	int	w, h;

	/* find maximum object */
	itr = iterator_first(xcont_list(xcp));
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		struct xobject *cp = iterator_data(itr)->xp_object;

		if(xobj_is_composer(cp))
			xcmp_compose(cp);

		maxw = xo_max(maxw, xobj_width(cp));
		maxh = xo_max(maxh, xobj_height(cp));
	}

	/* variable w/h */
	if(xcmp_wvariable(xcp))
		xobj_width(xcp) = maxw;
	if(xcmp_hvariable(xcp))
		xobj_height(xcp) = maxh;

	w = xobj_width(xcp);
	h = xobj_height(xcp);

	/* count expanders and compose child containers */
	itr = iterator_first(xcont_list(xcp));
	for(; !iterator_is_end(itr); itr = iterator_next(itr)){
		int x, y;
		struct xoposition *xpp = iterator_data(itr);
		struct xobject *xp = xpp->xp_object;

		x = hpos_diff(w, xobj_width(xp), xpp->xp_rule);
		y = vpos_diff(h, xobj_height(xp), xpp->xp_rule);

		xpp->xp_offset.x = x;
		xpp->xp_offset.y = y;
	}
}

struct xobject *
xcomposite_new(unsigned int width, unsigned int heigth)
{
	struct xlayout *ret = MALLOC(xod_typestr(XOBJTYPE_COMPOSITE), 
				     sizeof(struct xlayout));
	struct xcontainer *xcr = (struct xcontainer *)ret;

	xo_set((struct xobject *)ret, XOBJTYPE_COMPOSITE, width, 
		-1, xcr_draw, xcr_free, xcr_dump);
	xcontainer_alloc(xcr);
	xcmpcast(xcr)->xc_compose = xcomposite_compose;
	return (struct xobject *)ret;
}

/*------------* X Composite Pair *-------------*/

#ifdef XOBJ_TEST
#include "xsignal.h"
#include "xreactor.h"

#define handptr_width 32
#define handptr_height 32
#define handptr_x_hot 12
#define handptr_y_hot 1
static unsigned char handptr_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00, 0x48, 0x00, 0x00,
   0x00, 0x48, 0x00, 0x00, 0x00, 0x48, 0x00, 0x00, 0x00, 0x48, 0x00, 0x00,
   0x00, 0x48, 0x00, 0x00, 0x00, 0xc8, 0x01, 0x00, 0x00, 0x48, 0x1e, 0x00,
   0x00, 0x48, 0x72, 0x00, 0x00, 0x48, 0x92, 0x00, 0x80, 0x49, 0x92, 0x00,
   0x40, 0x4a, 0x92, 0x00, 0x40, 0x0a, 0x80, 0x00, 0x40, 0x0c, 0x80, 0x00,
   0x80, 0x0c, 0x80, 0x00, 0x80, 0x08, 0x80, 0x00, 0x80, 0x08, 0x80, 0x00,
   0x00, 0x09, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00,
   0x00, 0x02, 0x40, 0x00, 0x00, 0x02, 0x40, 0x00, 0x00, 0x02, 0x40, 0x00,
   0x00, 0x04, 0x20, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0xfc, 0x1f, 0x00,
   0x00, 0x04, 0x20, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x04, 0x20, 0x00,
   0x00, 0xf8, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00};


#define handptrmask_width 32
#define handptrmask_height 32
static unsigned char handptrmask_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00,
   0x00, 0x78, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00,
   0x00, 0x78, 0x00, 0x00, 0x00, 0xf8, 0x01, 0x00, 0x00, 0xf8, 0x1f, 0x00,
   0x00, 0xf8, 0x7f, 0x00, 0x00, 0xf8, 0xff, 0x00, 0x80, 0xf9, 0xff, 0x00,
   0xc0, 0xfb, 0xff, 0x00, 0xc0, 0xfb, 0xff, 0x00, 0xc0, 0xff, 0xff, 0x00,
   0x80, 0xff, 0xff, 0x00, 0x80, 0xff, 0xff, 0x00, 0x80, 0xff, 0xff, 0x00,
   0x00, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0x00,
   0x00, 0xfe, 0x7f, 0x00, 0x00, 0xfe, 0x7f, 0x00, 0x00, 0xfe, 0x7f, 0x00,
   0x00, 0xfc, 0x3f, 0x00, 0x00, 0xfc, 0x3f, 0x00, 0x00, 0xfc, 0x1f, 0x00,
   0x00, 0xfc, 0x3f, 0x00, 0x00, 0xfc, 0x3f, 0x00, 0x00, 0xfc, 0x3f, 0x00,
   0x00, 0xf8, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00};

static struct bitmap
handptr_bmp = { handptr_width, handptr_height, 0, 0, handptr_bits };
static struct bitmap
handptrmask_bmp = { handptrmask_width, handptrmask_height, 0, 0, 
                    handptrmask_bits };
static xcursor *handptr = NULL;
static xcursor *handptr2 = NULL;

static void
react(xreactptr *me, xreactptr *from, xwindow *x)
{
	printf("reaction receives %d!!\n", me->xrp_signal);
}

int 
main(int argc, char **argv)
{
	int i;
	struct xrootwindow *xrwp = xrootwindow_open(NULL);
	struct xwindow *xwp = xwindow_create(xrwp, 300, 200, 300, 200, 
					     1, 0xffffff, 0, 1, 1);
	xscroll 	  *scp;
	xreactor 	  *rscp;
	xreactor 	  *click;
	xreaction	  *enter, *leave, *press, *pp;
	xpoint		  pos = { 0 , 0};

	struct xcontainer *topv1 = xvlayout_new(400, 300,1, XCRULE_TOP);
	struct xcontainer *topv2 = xvlayout_new(400, 300,1, XCRULE_CENTER);
	struct xcontainer *topv3 = xvlayout_new(400, 300,1, XCRULE_BOTTOM);
	struct xcontainer *toph1 = xhlayout_new(400, 300, 1, XCRULE_LEFT);
	struct xcontainer *toph2 = xhlayout_new(400, 300, 1, XCRULE_CENTER);
	struct xcontainer *toph3 = xhlayout_new(400, 300, 1, XCRULE_RIGHT);

	struct xcontainer *cmp = xcomposite_new(400, 300);
	/*
	struct xcontainer *lh = xhlayout_new(300, 1, XCRULE_LEFT);
	struct xcontainer *ch = xhlayout_new(300, 1, XCRULE_CENTER);
	struct xcontainer *rh = xhlayout_new(300, 1, XCRULE_RIGHT);
	struct xcontainer *tv = xvlayout_new(400, 1, XCRULE_TOP);
	struct xcontainer *cv = xvlayout_new(400, 1, XCRULE_CENTER);
	struct xcontainer *bv = xvlayout_new(400, 1, XCRULE_BOTTOM);
	*/

	struct xattr *xat = xattr_alloc(xrwp);
	struct xattr *xat2 = xattr_alloc(xrwp);
	xobject *s;
	struct xs_bmotion mot = { 0, 0, 0, 10, 10 };

	handptr = xcursor_new(xrwp, &handptr_bmp, &handptrmask_bmp, 
				handptr_x_hot, handptr_y_hot, 0x0, 0xffffff);
	handptr2 = xcursor_new(xrwp, &handptr_bmp, &handptrmask_bmp, 
				handptr_x_hot, handptr_y_hot, 0x0, 0xffffff);

	xattr_set_font(xat, "fixed");
	xattr_set_fgcolor(xat, 0x00ff00);
	xattr_set_font(xat2, "fixed");
	xattr_set_fgcolor(xat2, 0xff0000);

	xcont_add(topv1, xstring_new("TOP-LEFT", 8, xat, 0), XCRULE_LEFT); 
	s = xstring_new("CENTER-CENTER", 13, xat, 0);

	enter = xreaction_new(xcmpstpair_new(s, underline_new(s, xat)),
			      react, NULL);
	leave = xreaction_new(s, react, NULL);
	press = xreaction_new(xstring_new("CENTER-CENTER", 13, xat2, 0), react, NULL);
	click = xclickable_new(s, handptr);


	xreactor_register(click, XSIG_ENTER, enter);
	xreactor_register(click, XSIG_LEAVE, leave);
	xreactor_register(click, XSIG_BTNPRESS, press);
	xreactor_register(click, XSIG_BTNRELEASE, leave);

	xcont_add(topv2, (xobject *)click, XCRULE_CENTER); 
	xcont_add(topv3, xstring_new("BOTTOM-RIGHT", 12, xat, 0), XCRULE_RIGHT); 
	xcont_add(toph1, xstring_new("LEFT-BOTTOM", 11, xat, 0), XCRULE_BOTTOM); 
	xcont_add(toph1, xstring_new("LEFT-BOTTOM", 11, xat, 0), XCRULE_BOTTOM); 


	xcont_add(toph2, xstring_new("CENTER-TOP", 10, xat, 0), XCRULE_TOP); 
	xcont_add(toph3, xstring_new("RIGHT-TOP", 9, xat, 0), XCRULE_TOP); 


	xcont_add(cmp, topv1, 0);
	xcont_add(cmp, topv2, 0);
	xcont_add(cmp, topv3, 0);
	xcont_add(cmp, toph1, 0);
	xcont_add(cmp, toph2, 0);
	xcont_add(cmp, toph3, 0);

	scp = xscroll_new(cmp, 300, 200, 0xffffff);
	rscp = xrscroll_new(scp, handptr, handptr2);
	xcmp_compose(rscp);
	xobj_dump(rscp, stdout, 0);

	xw_map(xwp);
	xobj_draw(rscp, xwp, 0, 0);
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
	pp = xcmp_find(scp, click->xr_curx + 4, click->xr_cury + 4, 0, &pos);
	printf("(pp == click?) = %d, (%d, %d)\n", (pp == click), pos.x, pos.y);

	getchar();


	xreactor_send(rscp, XSIG_ENTER, NULL, xwp);
	printf("enter\n");
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
		getchar();
	xreactor_send(rscp, XSIG_BTNPRESS, NULL, xwp);
	printf("press\n");
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
		getchar();
	xreactor_send(rscp, XSIG_GRABMOTION, &mot, xwp);
	printf("motion\n");
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
		getchar();
	xreactor_send(rscp, XSIG_SCRL_PAGEUP, 0, xwp);
	printf("pageup\n");
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
	pos.x = pos.y = 0;
	pp = xcmp_find(scp, click->xr_curx + 4, click->xr_cury + 4, 0, &pos);
	printf("(pp == click?) = %d, (%d, %d)\n", (pp == click), pos.x, pos.y);
		getchar();
	xreactor_send(rscp, XSIG_SCRL_PAGEDOWN, 0, xwp);
	printf("pagedown\n");
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
		getchar();
	xreactor_send(rscp, XSIG_SCRL_PAGELEFT, 0, xwp);
	printf("pageleft\n");
	xroot_flush(xrwp);
	printf("click: (%d, %d)\n", click->xr_curx, click->xr_cury);
		getchar();
	xreactor_send(rscp, XSIG_SCRL_PAGERIGHT, 0, xwp);
	printf("pageright\n");
	xroot_flush(xrwp);
		getchar();
	xreactor_send(rscp, XSIG_BTNRELEASE, NULL, xwp);
	printf("release\n");
	xroot_flush(xrwp);
		getchar();
	xreactor_send(rscp, XSIG_LEAVE, NULL, xwp);
	printf("leave\n");
	xroot_flush(xrwp);
		getchar();
	xreactor_send(rscp, XSIG_SCRL_RESET, 0, xwp);
	xroot_flush(xrwp);
	printf("reset\n");
		getchar();

	xcursor_free(xrwp, handptr);
	xcursor_free(xrwp, handptr2);
	xattr_free(xat);
	xattr_free(xat2);
	xwindow_destroy(xwp);
	xobj_free(rscp);
	xrootwindow_close(xrwp);
	stk_memdump();
	return 0;
}
#endif
