/*
 *			xeditstr.c
 * $Id: xeditstr.c,v 1.1 2004/01/20 02:55:33 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "xobject.h"
#include "xprimitive.h"

void 		
xeditstr_reset(struct xeditstr *estr)
{ 
	estr->str[0] = ' ';
	estr->str[1] = '\0';
	estr->nchar = 1;
	estr->pos = 0;
}

void 		
xeditstr_set(struct xeditstr *estr, const char *c)
{ 
	snprintf(estr->str, estr->maxchar + 1, "%s ", c);
	estr->str[estr->maxchar] = '\0';
	estr->nchar = strlen(estr->str);
	estr->pos = estr->nchar  - 1;
}

const char *
xeditstr_get(struct xeditstr *estr)
{ 
	return estr->str; 
}

void 		
xeditstr_putchar(struct xeditstr *estr, int c)
{  
	int i = 0;

	if(estr->nchar >= estr->maxchar - 1)
		return;
	
		/* make room */
	for(i = estr->nchar; i >= estr->pos; --i)
		estr->str[i + 1] = estr->str[i];
	estr->str[estr->pos] = c;
	estr->nchar += 1;
	estr->pos += 1;
}

void 		
xeditstr_backspace(struct xeditstr *estr)
{ 
	int i = 0;

	/* no more character before cursor */
	if(estr->pos == 0)
		return;
	
	for(i = estr->pos; i < estr->nchar + 1; ++i)
		estr->str[i-1] = estr->str[i];
	estr->nchar -= 1;
	estr->pos -= 1;
}

void 		
xeditstr_delete(struct xeditstr *estr)
{ 
	int i = 0;
	/* end of string */
	if(estr->pos == estr->nchar - 1)
		return;
	
	for(i = estr->pos; i < estr->nchar ; ++i)
		estr->str[i] = estr->str[i + 1];
	estr->nchar -= 1;
}

void 		
xeditstr_go_top(struct xeditstr *estr)
{ 
	estr->pos = 0;
}

void 		
xeditstr_go_next(struct xeditstr *estr)
{ 
	estr->pos += 1;
	if(estr->pos > estr->nchar - 1)
		estr->pos  = estr->nchar - 1;
}

void 		
xeditstr_go_prev(struct xeditstr *estr)
{ 
	estr->pos -= 1;
	if(estr->pos < 0)
		estr->pos = 0;
}

void 		
xeditstr_go_tail(struct xeditstr *estr)
{ 
	estr->pos = strlen(estr->str) - 1;
}

void 		
xeditstr_kill_before(struct xeditstr *estr)
{ 
	int i = estr->pos, j = 0;

	if(estr->pos == 0)
		return;

	while(i < estr->nchar + 1)
		estr->str[j++] = estr->str[i++];
	estr->pos = 0;
	estr->nchar = strlen(estr->str);
}
void 		
xeditstr_kill_after(struct xeditstr *estr)
{ 
	/* end position of string */
	if(estr->pos == estr->nchar - 1)
		return;

	estr->str[estr->pos] = ' ';
	estr->str[estr->pos+1] = '\0';
	estr->nchar = strlen(estr->str);
}
