/*
 *			xsignal.h
 * $Id: xsignal.h,v 1.7 2004/01/22 10:17:38 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _XP_XSIGNAL_H_
#define _XP_XSIGNAL_H_

typedef int xsignal;
#define xsignal_type(x)		((x) & 0x00ffffff)
#define xsignal_flags(x)	((x) & 0xff000000)
#define xsignal_set_type(x, y)	((x) = y |  xsignal_flags(x))

/* user define signal */
#define XSIG_USER			0x00800000

#define XSIG_REDIRECT			0x01000000
#define xsignal_is_redirect(x)	(((x) & XSIG_REDIRECT) == XSIG_REDIRECT)
#define xsignal_set_redirect(x)		((x) |= XSIG_REDIRECT)
#define xsignal_reset_redirect(x)	((x) &= ~XSIG_REDIRECT)

/*
 *		X-Signal related to xproxy
 *      Signal Name		number		reqiured-datatype
 */
/* 
 *  Signal proxy must accept XSIG_PROXY. 
 * So any other actor than proxiy must not return acceptable for
 * this signal.
 */
#define XSIG_ANY			0xffffff	
#define XSIG_PROXY			0xfffffe	
#define XSIG_PROXY_CACHERESET		0xfffffe	

/* mouse enters in the object */
/*
 *		X-Signal 
 *      Signal Name		number		reqiured-signal-argument
 */
/* mouse enters in the object */
#define XSIG_ENTER		0x1		   /* struct xsigarg */
/* mouse leaves in the object */
#define XSIG_LEAVE		0x2		   /* struct xsigarg */
/* mouse button is pressed in the object */
#define XSIG_BTNPRESS		0x3		  /* struct xs_buttoninfo */
/* mouse button is released in the object */
#define XSIG_BTNRELEASE		0x4	 	  /* struct xs_buttoninfo */
/* key is pressed in the object */
#define XSIG_KEYPRESS		0x5		  /* struct xs_keyinfo */
/* key is released in the object */
#define XSIG_KEYRELEASE		0x6		  /* struct xs_keyinfo */
/* mouse is moving without pressing button */
#define XSIG_MOTION		0x7		    /* struct xsigarg */
/* mouse is moving with pressing a button */
#define XSIG_GRABMOTION		0x8		  /* struct xs_bmotion */


/* common header for signal argument */
struct xsigarg {
	short	x, y; /* current position of pointer */
};

/* used by XSIG_BTNPRESS/XSIG_BTNRELASE */
struct xs_btninfo {
	struct xsigarg  pos;
	int		button;	
};

/* used by XSIG_KEYPRESS/XSIG_KEYRELASE */
struct xs_keyinfo {
	struct xsigarg  pos;
	const char	*key;
	int		modifier;
};

/* used by XSIG_GRABMOTION */
struct xs_bmotion {
	struct xsigarg  pos;
	int		button;	
	short		mx,  my;	/* movement difference */
};


/* 
 *  	signal for xrscroll
 */
#define XSIG_SCRL_RESET		0x1001		/* can't be caught */
#define XSIG_SCRL_PAGEUP	0x1002		/* can't be caught */
#define XSIG_SCRL_PAGEDOWN	0x1003		/* can't be caught */
#define XSIG_SCRL_PAGERIGHT	0x1004		/* can't be caught */
#define XSIG_SCRL_PAGELEFT	0x1005		/* can't be caught */
#define XSIG_SCRL_PAGESCRL	0x1006		/* xs_bmotion (mx, my) */

#endif /* _XP_XSIGNAL_H_ */

