/*
 *			xpopdic_component.c
 * $Id: xpopdic_component.c,v 1.11 2004/01/24 11:53:34 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>

#include "debug.h"
#include "memory.h"
#include "property.h"
#include "bitmap.h"
#include "dictbook.h"
#include "booklist.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "strset.h"
#include "xobjcache.h"
#include "xpopdic_resource.h"
#include "xpopdic_component.h"
#include "xpstatus.h"
#include "xpaction.h"

#define mkstr(x,a)	xstring_na_new(x, strlen(x), a, 0)

/* function declarations for only xpcompnent_initialize() */
struct xpopdic_resource *xpresource_initialize(xrootwindow *);
void xpresource_finalize(xrootwindow *);
xobject *xpsearch_new(struct xpopdic_resource *, struct xpopdic_control *);


static int str_cmp(const char *a, const char *b){ return strcmp(a, b); }
static unsigned int str_hash(const char *a)
{ 
	int i;
	unsigned int ret = 0;
	for(i = 0; i < 4; i++){
		if(a[i])
			ret += a[i];
		else
			break;
	}
	return ret;
}
static void str_free(const char *a){ xp_strfree(a); }

/* string a must be cached in strset */
static void xobj_cachefree(const char *a, xobject *x){ xobj_free(x); }

/* declarations for external function */
struct xpopdic_component *
xpcomponent_new(struct xpopdic_resource *xpr, struct xpopdic_control *xpc)
{
	struct xpopdic_component *ret;
	xobject			 *xo1, *xo2, *xo3;
	xactor			 *xa = NULL;
	/* allocate a component structure */
	ret = MALLOC("xpopdic_component", 
			sizeof(struct xpopdic_component));

	/* string cache */
	ret->xo_strs = strset_new(13, str_hash, str_cmp, 
				  xpdic_malloc, xpdic_free);

	/* english word object cache */
	ret->xo_words = xobjcache_new(13, str_hash, str_cmp, 
				      xpdic_malloc, xpdic_free);

	/* create stk objects */
	ret->xo_expdr = xexpander_new();	/* expander */
	xobj_incref(ret->xo_expdr);
		/* space between words in sentences */
	/* ret->xo_space = xspace_new(xattr_get_font_width(xpr->xatbody), 1);*/
	ret->xo_space = xstring_na_new(" ", 1, xpr->xatbody, 1);
	xobj_incref(ret->xo_space);
	ret->xo_colon = xstring_na_new(":", 1, xpr->xatbody, 0);
	xobj_incref(ret->xo_colon);
	ret->xo_slash = xstring_na_new("/", 1, xpr->xatsubinfo, 0);
	xobj_incref(ret->xo_slash);
	ret->xo_bottom = mkstr("--->> Bottom <<---", xpr->xatptr);
	xobj_incref(ret->xo_bottom);

		/* clickable pointer */
	xo1 = xbitmap_new(xpr->entptr1, xpr->xatptr);
	xo2 = xbitmap_new(xpr->entptr2, xpr->xatptr);
	xo3 = xbitmap_new(xpr->entptr3, xpr->xatptr);
	xa = xclickable_new(xo1);
	xact_register(xa, XSIG_ENTER, eptract.enter, xpc, xo3);
	xact_register(xa, XSIG_LEAVE, eptract.leave, xpc, xo1);
	xact_register(xa, XSIG_BTNPRESS, eptract.bpress, xpc, xo2);
	xact_register(xa, XSIG_BTNRELEASE, eptract.brelease, xpc, xo3);
	ret->xo_ptr = (xobject *)xa;
	/* increment reference to prevent someone free this object */
	xobj_incref(xa);

	/* up button */
	xo1 = xbitmap_new(xpr->up, xpr->xaticon);
	xo2 = xcmpstpair_new(underline_new(xo1, xpr->xaticon), xo1, 0);
	xo3 = xbitmap_new(xpr->up, xpr->xaticon_clicked);
	xa = xclickable_new(xo1);
	xact_register(xa, XSIG_ENTER, iconacts[XPACT_UP].enter, xpc, xo2);
	xact_register(xa, XSIG_LEAVE, iconacts[XPACT_UP].leave, xpc, xo1);
	xact_register(xa, XSIG_BTNPRESS, iconacts[XPACT_UP].bpress, xpc, xo3);
	xact_register(xa, XSIG_BTNRELEASE, iconacts[XPACT_UP].brelease, xpc, xo2);
	ret->xo_up = xposmark_new((xobject *)xa);
	/* increment reference to prevent someone free this object */
	xobj_incref(ret->xo_up);

	/* next button */
	xo1 = xbitmap_new(xpr->next, xpr->xaticon);
	xo2 = xcmpstpair_new(underline_new(xo1, xpr->xaticon), xo1, 0);
	xo3 = xbitmap_new(xpr->next, xpr->xaticon_clicked);
	xa = xclickable_new(xo1);
	xact_register(xa, XSIG_ENTER, iconacts[XPACT_NEXT].enter, xpc, xo2);
	xact_register(xa, XSIG_LEAVE, iconacts[XPACT_NEXT].leave, xpc, xo1);
	xact_register(xa, XSIG_BTNPRESS, iconacts[XPACT_NEXT].bpress, xpc, xo3);
	xact_register(xa, XSIG_BTNRELEASE, iconacts[XPACT_NEXT].brelease, xpc, xo2);
	ret->xo_next = xposmark_new((xobject *)xa);
	/* increment reference to prevent someone free this object */
	xobj_incref(ret->xo_next);

	/* prev button */
	xo1 = xbitmap_new(xpr->prev, xpr->xaticon);
	xo2 = xcmpstpair_new(underline_new(xo1, xpr->xaticon), xo1, 0);
	xo3 = xbitmap_new(xpr->prev, xpr->xaticon_clicked);
	xa = xclickable_new(xo1);
	xact_register(xa, XSIG_ENTER, iconacts[XPACT_PREV].enter, xpc, xo2);
	xact_register(xa, XSIG_LEAVE, iconacts[XPACT_PREV].leave, xpc, xo1);
	xact_register(xa, XSIG_BTNPRESS, iconacts[XPACT_PREV].bpress, xpc, xo3);
	xact_register(xa, XSIG_BTNRELEASE, iconacts[XPACT_PREV].brelease, xpc, xo2);
	ret->xo_prev = xposmark_new((xobject *)xa);
	/* increment reference to prevent someone free this object */
	xobj_incref(ret->xo_prev);

	/* quit button */
	xo1 = xbitmap_new(xpr->quit, xpr->xaticon);
	xo2 = xcmpstpair_new(underline_new(xo1, xpr->xaticon), xo1, 0);
	xo3 = xbitmap_new(xpr->quit, xpr->xaticon_clicked);
	xa = xclickable_new(xo1);
	xact_register(xa, XSIG_ENTER, iconacts[XPACT_QUIT].enter, xpc, xo2);
	xact_register(xa, XSIG_LEAVE, iconacts[XPACT_QUIT].leave, xpc, xo1);
	xact_register(xa, XSIG_BTNPRESS, iconacts[XPACT_QUIT].bpress, xpc, xo3);
	xact_register(xa, XSIG_BTNRELEASE, iconacts[XPACT_QUIT].brelease, xpc, xo2);
	ret->xo_quit = xposmark_new((xobject *)xa);
	/* increment reference to quitent someone free this object */
	xobj_incref(ret->xo_quit);

	/* status bar */
	xa = xpstatus_new(xpr);
	ret->xo_status = xposmark_new((xobject *)xa);
	/* increment reference to quitent someone free this object */
	xobj_incref(ret->xo_status);

	/* search page */
	ret->xo_search = xpsearch_new(xpr, xpc);
	xobj_incref(ret->xo_search);
	return ret;
}


void 
xpcomponent_delete(struct xpopdic_component *cp)
{
	xobj_free(cp->xo_status);
	xobj_free(cp->xo_search);
	xobj_free(cp->xo_ptr);
	xobj_free(cp->xo_up);
	xobj_free(cp->xo_next);
	xobj_free(cp->xo_prev);
	xobj_free(cp->xo_quit);
	xobj_free(cp->xo_expdr);
	xobj_free(cp->xo_space);
	xobj_free(cp->xo_colon);
	xobj_free(cp->xo_slash);
	xobj_free(cp->xo_bottom);

	/* free string/xobject cache */
	aggregator_delete_by(cp->xo_strs, str_free);
	aggregator_delete_by(cp->xo_words, xobj_cachefree);
	
	FREE("xpopdic_component", cp);
}

