/*
 *			xpopdic_options.c
 * $Id: xpopdic_options.c,v 1.12 2004/01/26 11:56:20 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "property.h"

long name2pixel(const char *color)
{
	Colormap	cmap;
	XColor	  c0, c1;
	Display *display = XOpenDisplay((char *)getenv("Display"));

	cmap = DefaultColormap(display, 0);
	XAllocNamedColor(display, cmap, color, &c1, &c0);
	XCloseDisplay(display);
	return (c1.pixel);
}

/* resource structure for Xt and command arguments */
typedef struct xpdresouce {
	/* important information */
	int		x, y;		/* position to popup: */
					/* -1 means following mouse. */
	int		width;		/* width of xpopdic window */
	int		height;		/* height of xpopdic window */
	int		bdwidth;	/* border width */
	int		bsidemargin;	/* Body side mergin */
	int		btbmargin;	/* Body top/bottom mergin */
	int		bodybdwidth;	/* Body border width */
	int		intval;		/* interval to check cutbuffer. */
	const char 	*dict;  	/* dictionry names */
	Pixel		bgc;		/* bg color of Body */
	Pixel		bdc;		/* border line color */

	/* options: not0=enable, 0=disable */
	int		shadow;		/* use window shadow */
	int		ndtpcache;	/* use diskcache for NDTP */
	int		listview;	/* use list-view */
	int		qonleave;	/* close on leave out of window */
	int		browsable;	/* word browsable */
	int		alwaysopen;	/* Book is always open */
	char		*cachedir;	/* cache directory */
	int		notitle;	/* no title */
	int		nosubinfo;	/* no subinfo */

	/* fonts */
	char 		*titlef;	/* font for title */
	char 		*dictf;		/* font for dictionary name */
	char 		*statf;		/* font for status bar */
	char 		*bodyf;		/* font for body */
	char 		*bodywf;	/* body font for wide characters */

	/* foreground colors */
	Pixel		titlec;		/* Title color */
	Pixel		dictc;		/* Dictionary name color */
	Pixel		statc;		/* color of font for StatusBar */
	Pixel		bodyc;		/* color of font for Body */
	Pixel		clickc;		/* font color at clicked */
	Pixel		gaijic;		/* gaiji color */
	Pixel		ptrc;		/* clickable pointer of bitmap */

	/* body colors */
	Pixel		bodybgc;	/* Body Background color */
	Pixel		bodybdc;	/* Body Border color */
} AppDat, *P;


struct optentry {
		char 	   *name;	/* property name */
		char 	   *class;	/* Xresouce name */
		int	   type;	/* 0 = int, 1 = bool 2 = string 3=pixcel */
		int	   needarg;	/* artument required? */
		char 	   *optchr;	/* option character */
		int	   idefault;	/* int/bool default */
		char       *sdefault;	/* string default */
		Cardinal   offset;	/* offset to the member */
		char 	   *msg;	/* help message */
}  xpopdic_resource[] = {
{ "notitle", "NoTitle",   1, 0,	 "p", 1, NULL, XtOffset(P, notitle), "Enable title" },
{ "nosubinfo", "NoSubInfo",1, 0, "q", 0, NULL, XtOffset(P, nosubinfo), "Disable subinformation"},
{ "shadow", "Shadow",     1, 0,	 "z", 0, NULL, XtOffset(P, shadow), "Enable shadow" },
{ "ndtpcache", "NDTPCache",1,0,	 "c", 1, NULL, XtOffset(P, ndtpcache), "Disable ndtpcache" },
{ "listview", "ListView", 1, 0,	 "l", 1, NULL, XtOffset(P, listview), "Disable entry listing" },
/*{ "qonleave", "QuitOnLeave",1,0, "q", 1, NULL, XtOffset(P, qonleave), "Enable close on leave" },   // No support */
{ "alwaysopen", "AlwaysOpen",1,0,"a", 1, NULL, XtOffset(P, alwaysopen), "Close books(session) after each transaction."},
{ "browsable", "Browsable",1, 0, "Z", 1, NULL, XtOffset(P, browsable), "Disable word-browsable" },
{ "dict", "Dictionary",   2, 1,  "D", 0, "all@localhost:2010", XtOffset(P, dict), "Dictionary Path" },
{ "ndtpcachedir","NDTPCacheDir",  2, 1,  "C", 0, "/var/tmp/.xpopdic", XtOffset(P, cachedir), "Cache Directory" },
{ "intval", "Interval",   0, 1,	 "i", 1, NULL, XtOffset(P, intval), "Interval to check buffers." },
{ "x", "X", 		  0, 1,  "x", -1, NULL, XtOffset(P, x), "x position to fix" },
{ "y", "Y", 		  0, 1,  "y", -1, NULL, XtOffset(P, y), "y position to fix" },
{ "width", "Width", 	  0, 1,  "w", 300, NULL, XtOffset(P, width), "window width" },
{ "height", "Height", 	  0, 1,	 "h", 160, NULL, XtOffset(P, height), "window height" },
{ "bdwidth", "BDWidth",   0, 1,	 "b", 1, NULL, XtOffset(P, bdwidth), "window border width" },
{ "bsidemargin", "BodySideMargin", 0, 1,"O", 4, NULL, XtOffset(P, bsidemargin), "Body Side Margin" },
{ "btbmargin", "BodyTopBottomMargin",0, 1,"M", 4, NULL, XtOffset(P, btbmargin), "Body Top/Bottom Margin" },
{ "bodybdwidth","BodyBDWidth",0,1,"j", 1, NULL, XtOffset(P, bodybdwidth), "Body Border width" },
{ "bgc", "BackgroundColor", 3, 1,"B", 0, "#E0E0E0", XtOffset(P, bgc), "Background Color" },
{ "bdc", "BorderColor",   3, 1,	 "C", 0, "#8888FF", XtOffset(P, bdc), "Border Color" },
{ "bodybgc", "BodyBackgroundColor", 3, 1,"K", 0, "#8888ff", XtOffset(P, bodybgc), "Body Background Color" },
{ "bodybdc", "BodyBorderColor", 3, 1,"J", 0, "#000000", XtOffset(P, bodybdc), "Body Border Color" },
{ "titlef",  "TitleFont",  2, 1, "t", 0, "-adobe-courier-bold-o-normal--18-180-75-75-m-110-iso8859-1", XtOffset(P, titlef), "Title Font"},
{ "dictf", "DictNameFont", 2, 1, "n", 0, "k14", XtOffset(P, dictf), "Dictionary Name Font"},
{ "statf", "StatusFont",   2,1,  "s", 0, "-adobe-times-medium-i-normal--12-120-75-75-p-63-iso8859-1", XtOffset(P, statf), "Status Font"},
{ "bodyf", "BodyFont",   2, 1,   "f", 0, "7x14", XtOffset(P, bodyf), "Body Font"},
{ "bodywf", "BodyWideFont",2, 1, "k", 0, "k14", XtOffset(P,bodywf), "Body Font(Wide)"},
{ "titlec", "TitleColor",  3, 1, "T", 0, "#0000FF", XtOffset(P, titlec), "Title Color" },
{ "dictc", "DictNameColor",3, 1, "N", 0, "#0000F0", XtOffset(P, dictc), "Dictionary Name Color" },
{ "statc", "StatusColor",  3, 1, "S", 0, "red", XtOffset(P, statc), "Status Color" },
{ "bodyc", "BodyFGColor",  3, 1, "F", 0, "yellow", XtOffset(P, bodyc), "Body Foreground Color" },
{ "clickc", "ClickColor",  3, 1, "L", 0, "red", XtOffset(P, clickc), "Clicked Font Color " },
{ "gaijic", "GaijiColor",  3, 1, "G", 0, "green", XtOffset(P, gaijic), "Gaiji Font Color " },
{ "ptrc",   "PointerColor",3, 1, "P", 0, "orange", XtOffset(P, ptrc), "Entry Pointer Color" },
{ NULL, NULL, 0, 0, NULL, 0, NULL, 0, NULL}
};

#define NENTRYS		sizeof(xpopdic_resource)/sizeof(struct optentry)

static String 
transtype(int i)
{
	switch(i){
	case 0: return XtRInt;
	case 1: return XtRBoolean;
	case 2: return XtRString;
	case 3: return XtRPixel;
	}
	return 0;
}

static String
transdtype(int i)
{
	switch(i){
	case 0: case 1: return XtRImmediate;
	case 2: case 3: return XtRString;
	}
	return 0;
}

static Cardinal
transsize(int i)
{
	switch(i){
	case 0: return sizeof(int);
	case 1: return sizeof(Boolean);
	case 2: return sizeof(String);
	case 3: return sizeof(Pixel);
	}
	return 0;
}

static void *
getaddr(struct optentry *p)
{
	if(p->type == 0)
		return (void *)p->idefault;
	else if(p->type == 1)
		return (void *)(p->idefault > 0? True: False);
	return p->sdefault;
}


int equal_type(const char *t1, const char *t2){ return !strcmp(t1, t2); }

inline void *_shift(void *adp, int offset){ return ((char*)adp) + offset; }


static void
xtbuild(XtResource *dst)
{
	int i = 0;
	struct optentry *p = xpopdic_resource;
	for(; p->name != NULL; ++p){
		dst[i].resource_name = p->name;
		dst[i].resource_class = p->class;
		dst[i].resource_type = transtype(p->type); 
		dst[i].resource_size = transsize(p->type); 
		dst[i].resource_offset = p->offset;
		dst[i].default_type = transdtype(p->type);
		dst[i].default_addr = getaddr(p);
		++i;
	}
}

static void
xtload(void)
{
	register unsigned int 	i;
	int	 	argc = 1;
	char 		*argv[] = { "xpopdic", 0};
	XtResource	tmp[NENTRYS-1], resources[NENTRYS-1];
	AppDat		appdata, *adp = &appdata;
	Widget		top = XtInitialize("xpopdic", "XPopDic", 
					   NULL, 0, &argc, argv);

	xtbuild(tmp);
	xtbuild(resources);

	XtGetApplicationResources(top, &appdata, tmp, XtNumber(tmp),NULL,0);

	for(i = 0; i < XtNumber(resources); i++){
		XtResource* xp = &resources[i]; 
		struct property *oldp = NULL;

		if(equal_type(xp->resource_type, XtRInt) ||
		   equal_type(xp->resource_type, XtRPixel))
			oldp = property_put(
				 property_integer_alloc(xp->resource_class, 
				*(int *)(_shift(adp, xp->resource_offset))));

		else if(equal_type(xp->resource_type, XtRBoolean))
			oldp = property_put(property_integer_alloc(xp->resource_class, 
			  *(Boolean*)(_shift(adp, xp->resource_offset)) == True));
		else if(equal_type(xp->resource_type, XtRString))
			oldp = property_put(property_string_alloc(xp->resource_class, 
			 *(char**)(_shift(adp, xp->resource_offset))));
		if(oldp)
			property_free(oldp);
	}
}

#define INTPGET(x)	prty_integer_val(property_get(x))
#define STRPGET(x)	prty_string_val(property_get(x))

static void
optout(struct optentry *p)
{
	fprintf(stderr, "\t  %s-%s %s              %s", 
		(p->needarg? "": "  "), p->optchr, 
		(p->needarg? "arg": " "), p->msg);

	if(p->type == 0 && p->idefault != -1)
		fprintf(stderr, "(%d)", INTPGET(p->class));
	else if(p->type == 2 && p->sdefault != 0)
		fprintf(stderr, "(\"%s\")", STRPGET(p->class));
	else if(p->type == 3)
		fprintf(stderr, "(\"#%06x\")", INTPGET(p->class));
	fprintf(stderr, "\n");
}

void usage_exit(void)
{
	struct optentry *p = xpopdic_resource;
#ifdef XPOPDIC_VERSION
	fprintf(stderr, "xpopdic %s\n", XPOPDIC_VERSION);
#else 
#endif /* XPOPDIC_VERSION */

#define FOUT(x)		fprintf(stderr, x)
	FOUT("xpopdic <Options as below>\n");
	FOUT("\t  options          description(default)\n");
	FOUT("\t-----------      --------------------------\n");
	FOUT("\t    -v                show version\n"); 
	FOUT("\t    -d                enable debug\n"); 
	FOUT("\t    -m                test-mode\n"); 

	for(; p->name != NULL; ++p)
		optout(p);
	exit(1);
}



static void putintprop(const char *key, int val)
{
	struct property *prp = property_put(property_integer_alloc(key, val));
	if(prp)
		property_free(prp);
}
static void putstrprop(const char *key, const char * val)
{
	struct property *prp = property_put(property_string_alloc(key, val));
	if(prp)
		property_free(prp);
}

void showver_exit(void)
{
#ifdef XPOPDIC_VERSION
	fprintf(stderr, "xpopdic %s\n", XPOPDIC_VERSION);
#else 
	fprintf(stderr, "xpopdic-test\n");
#endif /* XPOPDIC_VERSION */
	exit(1);
}

void
construct_optlist(char *b)
{
	struct optentry *p = xpopdic_resource;
	for(; p->name != NULL; ++p){
		strcat(b, p->optchr);
		if(p->needarg)
			strcat(b, ":");
	}
}

void 
optanalysis(int argc, char **argv)
{
	int  c;
	char optlist[256];

	/* load first */
	xtload();

	optlist[0] = '\0';
	strcpy(optlist, "vdm");
	construct_optlist(optlist);

	while((c = getopt(argc, argv, optlist)) > 0){
		struct optentry *p = xpopdic_resource;
		switch(c){
		case 'v':
			showver_exit();
		case 'd':
			putintprop("Debug", 1);
			continue;
		case 'm':
			putintprop("Test", 1);
			continue;
		}

		/* no such option */
		if(index(optlist, c) == NULL)
			usage_exit();
		
		/* find the entry */
		for(; p->name != NULL; ++p)
			if(*p->optchr == c)
				break;

		if(!p->name)
			usage_exit();
		
		switch(p->type){
		case 0: 	/* int */
			putintprop(p->class, atoi(optarg));
			break;
		case 1:		/* bool: oposite of default  */
			putintprop(p->class, !p->idefault);
			break;
		case 2:		/* string */
			putstrprop(p->class, optarg);
			break;
		case 3:		/* pixel name */
			putintprop(p->class, (int)name2pixel(optarg));
			break;
		}
	}
}

