/*
 *			xpopdic_resource.c
 * $Id: xpopdic_resource.c,v 1.12 2004/02/12 00:45:58 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include "debug.h"
#include "memory.h"
#include "bitmap.h"
#include "property.h"
#include "xwindow.h"
#include "xpopdic_resource.h"

/* icon */
#include "bitmaps/up.bmp"
#include "bitmaps/next.bmp"
#include "bitmaps/prev.bmp"
#include "bitmaps/quit.bmp"
#include "bitmaps/link1.bmp"
#include "bitmaps/link2.bmp"
#include "bitmaps/link3.bmp"
#include "bitmaps/glass1.bmp"
#include "bitmaps/glass2.bmp"
#include "bitmaps/glass3.bmp"
#include "bitmaps/glass4.bmp"

/* bitmap data */
static struct bitmap up_bmp = { up_width, up_height, -1, -1, up_bits };
static struct bitmap next_bmp = { next_width, next_height, -1, -1, next_bits };
static struct bitmap prev_bmp = { prev_width, prev_height, -1, -1, prev_bits };
static struct bitmap quit_bmp = { quit_width, quit_height, -1, -1, quit_bits };
static struct bitmap entptr1_bmp = { link1_width, link1_height, -1, -1, link1_bits };
static struct bitmap entptr2_bmp = { link2_width, link2_height, -1, -1, 
				  link2_bits };
static struct bitmap entptr3_bmp = { link3_width, link3_height, -1, -1, 
				  link3_bits };
static struct bitmap glass1_bmp = { glass1_width, glass1_height, -1, -1, 
				  glass1_bits };
static struct bitmap glass2_bmp = { glass2_width, glass2_height, -1, -1, 
				  glass2_bits };
static struct bitmap glass3_bmp = { glass3_width, glass3_height, -1, -1, 
				  glass3_bits };
static struct bitmap glass4_bmp = { glass4_width, glass4_height, -1, -1, 
				  glass4_bits };

/* static resource data */
static struct xpopdic_resource xrp;
static void check_resource(void);

#define INTPROP(x)	prty_integer_val(property_get(x))
#define STRPROP(x)	prty_string_val(property_get(x))

static xattr *
xatalloc(xrootwindow *p, const char *font, const char *fgc, const char *bgc)
{
	xattr *ret = xattr_alloc(p);

	if(font != NULL && !xattr_set_font(ret, STRPROP(font))){
		fprintf(stderr, "xattr_set_font(%s): can't load font %s. use fixed instead.\n", font, STRPROP(font));
		xattr_set_font(ret, "fixed");
	}
	
	if(fgc)
		xattr_set_fgcolor(ret, INTPROP(fgc));
	if(bgc)
		xattr_set_bgcolor(ret, INTPROP(bgc));

	return ret;
}

/* declarations for external function */
struct xpopdic_resource *
xpresource_initialize(xrootwindow *xwp)
{
	xrp.x 		= INTPROP("x");
	xrp.y 		= INTPROP("y");
	xrp.width 	= INTPROP("width");
	xrp.height 	= INTPROP("height");
	xrp.bdwidth 	= INTPROP("bdwidth");
	xrp.bsidemargin	= INTPROP("BodySideMargin");
	xrp.btbmargin	= INTPROP("BodyTopBottomMargin");
	xrp.bodywidth	= xrp.width - xrp.bsidemargin * 2; 
	xrp.bodybdwidth	= INTPROP("BodyBDWidth");

	xrp.notitle 	= INTPROP("notitle");
	xrp.nosubinfo 	= INTPROP("nosubinfo");
	xrp.intval 	= INTPROP("interval");
	xrp.shadow 	= INTPROP("shadow");
	/* xrp.quitonleave = INTPROP("quitonleave");*/
	xrp.browsable 	= INTPROP("browsable");
	xrp.bgc 	= INTPROP("BackgroundColor");
	xrp.bdc 	= INTPROP("BorderColor");

	/* allocate */
	xrp.xattitle 	= xatalloc(xwp, "TitleFont", 
				   "TitleColor", "BackgroundColor");
	xrp.xatsubinfo 	= xatalloc(xwp, "BodyFont", 
				   NULL, "BackgroundColor");
	xattr_set_fgcolor_byname(xrp.xatsubinfo, "#000000");
	xrp.xatdict 	= xatalloc(xwp, "DictNameFont", 
				   "DictNameColor", "BackgroundColor");
	xrp.xatstat 	= xatalloc(xwp, "StatusFont", 
				   "StatusColor", "BackgroundColor");
	xrp.xatbody 	= xatalloc(xwp, "BodyFont", 
				   "BodyFGColor", "BodyBackgroundColor");
	xrp.xatwbody 	= xatalloc(xwp, "BodyFont", 
				   "BodyFGColor", "BodyBackgroundColor");
	xrp.xatwbody 	= xatalloc(xwp, "BodyWideFont", 
				   "BodyFGColor", "BodyBackgroundColor");
	xrp.xatclick 	= xatalloc(xwp, "BodyFont", 
				   "ClickColor", "BodyBackgroundColor");
	xrp.xatwclick 	= xatalloc(xwp, "BodyWideFont", 
				   "ClickColor", "BodyBackgroundColor");
	xrp.xatgaiji	= xatalloc(xwp, "BodyFont", "GaijiColor", "BodyBackgroundColor");
	xrp.xatptr	= xatalloc(xwp, "BodyFont", "PointerColor", "BodyBackgroundColor");
	xrp.xatborder	= xatalloc(xwp, NULL, "BorderColor", "BackgroundColor");
	xattr_set_font(xrp.xatborder, "fixed");
	/* xattr_set_line_width(xrp.xatborder, xrp.bdwidth); */
	xattr_set_line_width(xrp.xatborder, 1); 

	xrp.xatbodybgc	= xatalloc(xwp, NULL, "BodyBackgroundColor", "BodyBackgroundColor");
	xrp.xatbodybdc	= xatalloc(xwp, NULL, "BodyBorderColor", "BodyBorderColor");
	xattr_set_line_width(xrp.xatbodybdc, xrp.bodybdwidth);

	/* attributes for search window */
	xrp.xatwhite = xattr_alloc(xwp);
	xattr_set_font(xrp.xatwhite, "fixed");
	xattr_set_fgcolor_byname(xrp.xatwhite, "#ffffff");
	xattr_set_bgcolor_byname(xrp.xatwhite, "#000000");
	xrp.xatblack = xattr_alloc(xwp);
	xattr_set_font(xrp.xatblack, "fixed");
	xattr_set_fgcolor_byname(xrp.xatblack, "#000000");
	xattr_set_bgcolor_byname(xrp.xatblack, "#ffffff");
	xrp.xatblue = xattr_alloc(xwp);
	xattr_set_font(xrp.xatblue, "fixed");
	xattr_set_fgcolor_byname(xrp.xatblue, "#0000ff");

	xrp.xaturl = xattr_alloc(xwp);
	xattr_set_font(xrp.xaturl, "fixed");
	xattr_set_fgcolor(xrp.xaturl, INTPROP("TitleColor"));

	/* bitmaps for icons */
	xrp.up 		= &up_bmp;
	xrp.next 	= &next_bmp;
	xrp.prev 	= &prev_bmp;
	xrp.quit 	= &quit_bmp;

	/* entry link icon */
	xrp.entptr1 	= &entptr1_bmp;
	xrp.entptr2 	= &entptr2_bmp;
	xrp.entptr3 	= &entptr3_bmp;

	/* icon for status bar */
	xrp.glass1 	= &glass1_bmp;
	xrp.glass2 	= &glass2_bmp;
	xrp.glass3 	= &glass3_bmp;
	xrp.glass4 	= &glass4_bmp;

	/* attributes for icons on a status bar */
	xrp.xatglass1 = xattr_alloc(xwp);
	xrp.xatglass2 = xattr_alloc(xwp);
	xrp.xatglass3 = xattr_alloc(xwp);
	xrp.xatglass4 = xattr_alloc(xwp);
	xattr_set_fgcolor_byname(xrp.xatglass1, "#0000ff");
	xattr_set_bgcolor_byname(xrp.xatglass1, "#ffffff");
	xattr_set_fgcolor_byname(xrp.xatglass2, "#00ff00");
	xattr_set_bgcolor_byname(xrp.xatglass2, "#ffffff");
	xattr_set_fgcolor_byname(xrp.xatglass3, "#0f4f4f");
	xattr_set_bgcolor_byname(xrp.xatglass3, "#ffffff");
	xattr_set_fgcolor_byname(xrp.xatglass4, "#ff0000");
	xattr_set_bgcolor_byname(xrp.xatglass4, "#ffffff");


	/* attributes for icons on a status bar */
	xrp.xaticon = xattr_alloc(xwp);
	xrp.xaticon_clicked = xattr_alloc(xwp);
	xattr_set_fgcolor_byname(xrp.xaticon, "#ffffff");
	xattr_set_bgcolor_byname(xrp.xaticon, "#000000");
	xattr_set_fgcolor_byname(xrp.xaticon_clicked, "#ff0000");
	xattr_set_bgcolor_byname(xrp.xaticon_clicked, "#000000");

	check_resource();

	return &xrp;
}


void
xpresource_finalize(xrootwindow *xwp)
{
	/* free attributes */
	xattr_free(xrp.xattitle);
	xattr_free(xrp.xatsubinfo);
	xattr_free(xrp.xatdict);
	xattr_free(xrp.xatstat);
	xattr_free(xrp.xatbody);
	xattr_free(xrp.xatwbody);
	xattr_free(xrp.xatclick);
	xattr_free(xrp.xatwclick);
	xattr_free(xrp.xatgaiji);
	xattr_free(xrp.xatptr);
	xattr_free(xrp.xatborder);
	xattr_free(xrp.xatwhite); 
	xattr_free(xrp.xatblack); 
	xattr_free(xrp.xatblue); 
	xattr_free(xrp.xaturl); 
	xattr_free(xrp.xatbodybgc);
	xattr_free(xrp.xatbodybdc);

	xattr_free(xrp.xatglass1);
	xattr_free(xrp.xatglass2);
	xattr_free(xrp.xatglass3);
	xattr_free(xrp.xatglass4);
	xattr_free(xrp.xaticon);
	xattr_free(xrp.xaticon_clicked);
}


static struct xpopdic_resource xrp;
static void 
check_resource(void)
{
	if(xrp.width < 300){
		fprintf(stderr, "Width must be greater than 300.\n");
		exit(1);
	}
	if(xrp.height < 150){
		fprintf(stderr, "Height must be greater than 150.\n");
		exit(1);
	}
}
