/*
 *                      xppage_trans.c
 * $Id: xppage_trans.c,v 1.9 2004/01/24 11:53:34 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* 
 *	structure of xptrans
 *
 *   keyaction
 *	|
 *   vlayout 
 *	|
 *	+---  hlayout
 *	|        +-- xpstatus expander quit
 *	|
 *	+---  hlayout
 *	|        +-- expander Title expander
 *	|
 *	+---  hlayout
 *	|        +-- line string line
 *	|
 *	+---  vlayout
 *	         +-- string 
 *	         +-- string 
 *	         +-- string 
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <time.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "wordhist.h"
#include "dictbook.h"
#include "booklist.h"
#include "dictlink.h"
#include "strlist.h"
#include "strset.h"
#include "xobjcache.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "xpopdic_resource.h"
#include "xpopdic_component.h"
#include "xpopdic_control.h"
#include "xpobject.h"
#include "xpaction.h"
#include "xppage.h"
#define mkstr(x,y)	xstring_na_new(x, strlen(x), y, 0)
#define mkwstr(x,y)	xwstring_na_new(x, strlen(x)/2, y, 0)

static int 
get_nentry_parent(struct dictnode *p)
{
	p = p->dn_parent;
	while(p->dn_type != DNTYPE_LIST)
		p = p->dn_parent;
	return p->dn_number;
}

xobject *
xppage_trans_border(struct xpopdic_control *cp, struct dictnode *dnode)
{
	xobject *border = xhlayout_new(cp->xpr->width, -1, 0, XCRULE_CENTER);
	unsigned char max = get_nentry_parent(dnode);
	unsigned char cur = dnode->dn_number + 1;
	xobject *maxo = mkstr(num2str[max], cp->xpr->xatsubinfo);
	xobject *curo = mkstr(num2str[cur], cp->xpr->xatsubinfo);
	xobject *dicto = mkwstr(dnode->dn_book->db_name, cp->xpr->xatdict);
	int w = cp->xpr->width - xobj_width(maxo)- xobj_width(curo) 
	    - xobj_width(dicto) - xobj_width(cp->xpc->xo_slash) 
	    - (15 * 2) - (10 * 4);

	xcont_add(border, xline_new(0, 0, 15, 0, cp->xpr->xatborder), 
		  XCRULE_CENTER);
	xcont_add(border, xspace_new(10, 4), XCRULE_CENTER);
	xcont_add(border, dicto, XCRULE_CENTER);
	xcont_add(border, xspace_new(10, 4), XCRULE_CENTER);
	xcont_add(border, xline_new(0, 0, w, 0, cp->xpr->xatborder), 
		  XCRULE_CENTER);
	xcont_add(border, xspace_new(10, 4), XCRULE_CENTER);
	xcont_add(border, curo, XCRULE_CENTER);
	xcont_add(border, cp->xpc->xo_slash, XCRULE_CENTER);
	xcont_add(border, maxo, XCRULE_CENTER);
	xcont_add(border, xspace_new(10, 4), XCRULE_CENTER);
	xcont_add(border, xline_new(0, 0, 15, 0, cp->xpr->xatborder), 
		  XCRULE_CENTER);
	return border;
}

xobject *
xppage_trans_body(struct xpopdic_control *cp, struct dictnode *dnode, 
		unsigned int w, unsigned int h)
{
	xobject *body = xvlayout_new(w, -1, 0, XCRULE_TOP);
	int th = 0;
	strlist_iterator itr = iterator_first(dnode->dn_content);

	for(; !iterator_is_end(itr); itr = iterator_next(itr))
		xpput_line((iterator_data(itr)), body,  w, dnode->dn_book, cp);

	/* total height */
	xcmp_compose(body); 
	th = xcont_theight(body); 

	if(th + 3 > h)
		xcont_add(body, cp->xpc->xo_bottom, XCRULE_CENTER);

	/* add space a little at bottom */
	xcont_add(body, xspace_new(3, 3), XCRULE_CENTER);

	return body;
}

