/*
 *			xpparse.c
 * $Id: xpparse.c,v 1.1 2004/01/20 13:03:24 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "xpparse.h"

/*
 * xpis_wchr():
 *	- return 1 if the character is a wide character.
 *	- otherwise return 0.
 */
int xpis_wchr(int c){ return ( c & 0x80 ); }

/*
 * xpis_extern():
 *	- return 1 if the character is a starting symbol('<') for an 
 *	external entry.
 *	- otherwise return 0.
 */
int xpis_extern(const char *p)
{
	int colon = 0;
	/* <xxxxx> */
	if(*p != '<' || index(p, '>') == NULL)
		return 0;

	++p;
	if(strncmp(p, "gaiji", 5) == 0)
		return 1;

	while(*p != '>'){
		if(xpis_wchr(*p) || 
		   !(isdigit(*p) || isalpha(*p) || *p == ':'))
			return 0;
		if(*p == ':')
			colon = 1;
		++p;
	}
	return colon;
}

/* 
 * xpget_word(): only alphabet sequence
 *	- xpget_word() finds the end of the word starting with the
 *	start poiner indicated by "startp".
 *	- save a length of the word into len;
 *	- return a pointer next to the end of the word.
 */
const char *xpget_word(const char *p, int *rlen)
{
	int len = 0;
	while(*p && !xpis_wchr(*p) && isalpha(*p)){
		++p;
		++len;
	}
	*rlen = len;
	return p;
}

/* 
 * xpget_wstring(): wide character sequence
 *	- xpget_wstring() finds the end of the wstring starting with the
 *	start poiner indicated by "startp".
 *	- save a length of the wstring into *len;
 *	- return a pointer next to the end of the string.
 */
const char *
xpget_wstring(const char *p, int *blen)
{
	int len = 0;
	while(*p && xpis_wchr(*p)){
		p += 2;
		len += 2;
	}
	*blen = len / 2;
	return p;
}

/* 
 * xpget_noalpha(): ascii character sequence except '<', space and alphabet.
 *	- xpget_string() finds the end of the ascii string starting with the
 *	start poiner indicated by "startp".
 *	- save a length of the string into len;
 *	- return a pointer next to the end of the string.
 */
const char *
xpget_noalpha(const char *p, int *rlen)
{
	int len = 0;
	while(*p && !xpis_wchr(*p) && !isalpha(*p) && 
	      !isspace(*p) && !xpis_extern(p)){
		++p;
		++len;
	}
	*rlen = len;
	return p;
}


/* 
 * xpget_entptr(): get entry pointer.
 *	- xpget_entptr() extracts entry id from "<entryid>" string 
 *	indicated by "startp".
 *	- if the entry id is for a gaiji entry, set *type 1.
 *	- else if the entry id is for a translation entry, set *type 2.
 *	- Neither of them, it means it is not entry pointer. set *type 0.
 *	- return a pointer next to the end of the string.
 */
const char *
xpget_entptr(const char *startp, char *entryid, int len, int *type)
{
	int  n = 0;
	const char *p = startp;
	if(*p++ != '<')
		return NULL;

	if(type)
		*type = 0;

	if(!strncmp(p, "gaiji:", 6)){
		if(type)
			*type = 1;
		p += 6;
	}
	
	while(*p != '>'){
		if(entryid && n < len)
			entryid[n++] = *p++;
		if(*p == ':' && type)
			*type = 2;
			
		if(*p == '\0'){
			fprintf(stderr, "Entry(\"%s\") includes a terminate character inside!!\n", startp);
			break;
		}
	}
	if(entryid)
		entryid[n] = '\0';
	return p + 1;
}

