/*
 *			xpparse.h
 * $Id: xpparse.h,v 1.3 2004/01/21 17:29:12 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef _XP_PARSE_H_
#define _XP_PARSE_H_
/* xpopdic parsing subroutine */

/*
 * xpis_wchr():
 *	- return 1 if the character is a wide character.
 *	- otherwise return 0.
 */
int xpis_wchr(int c);

/*
 * xpis_extern():
 *	- return 1 if the character is a starting symbol('<') for an 
 *	external entry.
 *	- otherwise return 0.
 */
int xpis_extern(const char *p);


/* 
 * xpget_word(): only alphabet sequence
 *	- xpget_word() finds the end of the word starting with the
 *	start poiner indicated by "startp".
 *	- save a length of the word into len;
 *	- return a pointer next to the end of the word.
 */
const char *xpget_word(const char *startp, int *len);

/* 
 * xpget_wstring(): wide character sequence
 *	- xpget_wstring() finds the end of the wstring starting with the
 *	start poiner indicated by "startp".
 *	- save a length of the wstring into len;
 *	- return a pointer next to the end of the string.
 */
const char *xpget_wstring(const char *startp, int *len);

/* 
 * xpget_noalpha(): ascii character sequence except '<', space and alphabet.
 *	- xpget_string() finds the end of the ascii string starting with the
 *	start poiner indicated by "startp".
 *	- save a length of the string into len;
 *	- return a pointer next to the end of the string.
 */
const char *xpget_noalpha(const char *startp, int *len);


/* 
 * xpget_entptr(): get entry pointer.
 *	- xpget_entptr() extracts entry id from "<entryid>" string 
 *	indicated by "startp".
 *	- if the entry id is for a gaiji entry, set *type 1.
 *	- else if the entry id is for a translation entry, set *type 2.
 *	- Neither of them, it means it is not entry pointer. set *type 0.
 *	- return a pointer next to the end of the string.
 */
const char *xpget_entptr(const char *startp, char *entryid, int len, int *type);


#endif /* _XP_PARSE_H_ */
