/*
 *                      xpsearch.c
 * $Id: xpsearch.c,v 1.5 2004/01/22 12:49:28 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* 
 *	structure of xpsearch 
 *
 *   keyaction
 *	|
 *   vlayout [window size]
 *	|
 *   cmpstpair < centering >
 *	|
 *	+---  string editor 
 *	|
 *	+---  rectanble[white] 
 */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "dictbook.h"
#include "booklist.h"
#include "strlist.h"
#include "strset.h"
#include "xobjcache.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "xpopdic_resource.h"
#include "xpopdic_component.h"
#include "xpopdic_control.h"
#include "xpstatus.h"
#include "xppage.h"

static void keypress(struct xactor *dst, xpoint dstpos, 
               xsignal sig, struct xsigarg *siginfo, void *actarg,
	       struct xobject *top, xwindow *xwp);

xobject *
xpsearch_new(struct xpopdic_resource *xpr, struct xpopdic_control *cp)
{
	xobject *vlayout = xvlayout_new(xpr->width, xpr->height, 0, XCRULE_CENTER);
	xobject *str = NULL, *rect = NULL, *pos;
	xactor 	*key = NULL;

	str = xeditstr_new(xpr->width / 2, xpr->xatblue, xpr->xatwhite,  
			   XCRULE_CENTER);
	pos = xposmark_new(str);
	xpm_data(pos) = cp;	/* save here */

	rect = xrect_new(xpr->width / 2 + 4, xobj_height(str) + 4, 1, xpr->xatwhite);
	xcont_add(vlayout, xcmpstpair_new(pos, rect, 0), XCRULE_CENTER);
	key = xkeyaction_new(vlayout);
	xact_register(key, XSIG_KEYPRESS, keypress, pos, NULL);

	return (xobject *)key;
}


static void 
keypress(struct xactor *dst, xpoint dstpos, xsignal sig, struct xsigarg *siginfo, 
	 void *actarg, struct xobject *top, xwindow *xwp)
{
	struct xs_keyinfo *kip = (struct xs_keyinfo *)siginfo;
	xobject *strpos = (xobject *)actarg;
	xeditstr *estr = (xeditstr *)xpm_obj(strpos);
	struct xpopdic_control *cp = (struct xpopdic_control *)xpm_data(strpos);
	struct xppage *curpage = (struct xppage *)cp->xo_cur;

	if(xsignal_type(sig) != XSIG_KEYPRESS)
		return;

	if(strlen(kip->key) > 1){
		if(!strcasecmp(kip->key, "Return")){
			/* disable myself */
			xobj_disable(dst);

			/* activate current window */
			xobj_activate(curpage->xp_main);

			estr->str[strlen(estr->str)-1] = '\0'; /* remove space */
			if(strlen(estr->str) > 0)
				xpc_search(cp, estr->str);
			stk_redraw();
			xeditstr_reset(estr);
			return;
		}
		else if(!strcasecmp(kip->key, "Backspace"))
			xeditstr_backspace(estr);
		else if(!strcasecmp(kip->key, "Delete"))
			xeditstr_delete(estr);
		else if(!strcasecmp(kip->key, "Left"))
			xeditstr_go_prev(estr);
		else if(!strcasecmp(kip->key, "Right"))
			xeditstr_go_next(estr);
		else if(!strcasecmp(kip->key, "Space"))
			xeditstr_putchar(estr, ' ');
		else if(!strcasecmp(kip->key, "period"))
			xeditstr_putchar(estr, '.');
	}
	else if(kip->modifier & ~(XKM_SHIFT | XKM_CAPS)){
		if(kip->modifier & ~XKM_CTRL)
			return;

		switch(*kip->key){
		case 'b':
			xeditstr_go_prev(estr);
			break;
		case 'f':
			xeditstr_go_next(estr);
			break;
		case 'd':
			xeditstr_delete(estr);
			break;
		case 'e':
			xeditstr_go_tail(estr);
			break;
		case 'a':
			xeditstr_go_top(estr);
			break;
		case 'k':
			xeditstr_kill_after(estr);
			break;
		case 'u':
			xeditstr_kill_before(estr);
			break;
		}
	}
	else 
		xeditstr_putchar(estr, *kip->key);

	xobj_draw_object(top, strpos, xpm_x(strpos), xpm_y(strpos), xwp, 0);
}

